@@ -226,10 +226,10 @@ MYX_VARIABLES* myx_get_server_variables(MYSQL *mysql)
 {
   MYX_VARIABLES *vars;
 
-  /*if (mysql_full_version_is_later_or_equal_than(mysql, 4, 0, 3))
+  if (mysql_full_version_is_later_or_equal_than(mysql, 5, 0, 3))
     vars= myx_get_variables(mysql, "SHOW GLOBAL VARIABLES");
-  else*/
-  vars= myx_get_variables(mysql, "SHOW VARIABLES");
+  else
+    vars= myx_get_variables(mysql, "SHOW VARIABLES");
 
   
   if (vars)
