@@ -1,4 +1,4 @@
-/* Copyright (C) 2003,2004 MySQL AB
+/* Copyright (C) 2003, 2004, 2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,10 +14,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-#define BUFFER_LEN 256 //used by get_backup_content
+#define BACKUP_CONTENT_BUFFER_LEN 256 //used by get_backup_content
 #define DEFAULT_SCHEMA_NAME "DEFAULT_SCHEMA"
 #define DEFAULT_CATALOG_NAME "DEFAULT_CATALOG"
 #define ADMIN_DUMP_VERSION "1.4"
+#define TABLE_NAME_PCRE "(`[^`]+`|\"[^\"]+\"|\\S+)"
 
 #define INITIAL_BUFFER_LENGTH 1024*1024 /* initial size of 1M */
 
@@ -39,6 +40,7 @@
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
 #include <time.h>
+#include <string.h>
 
 //XXX add gettext support
 #define _(s) s
@@ -113,12 +115,14 @@ typedef struct
   unsigned int current_table_index;
   int table_finished;
   char *current_table_quoted;
+  char *current_schema;
   char *current_schema_quoted;
   int current_table_rows_processed;
   int current_table_rows;
   char *extended_insert;
   int starting;
   char quote_char;
+  int ignore;
 } MYX_BS_STATUS;
 
 /* forward declarations */
@@ -144,11 +148,12 @@ static int  compare_backup_tables(const void *a, const void *b);
 static int write_path_info_to_file(MYX_BS_STATUS *status,
                                    MYX_BACKUP_ERROR *error);
 static MYX_BACKUP_CONTENT* copy_backup_content(MYX_BACKUP_CONTENT *content);
-
+static MYX_BACKUP_CONTENT* select_all_tables(MYSQL* mysql, MYX_BACKUP_CONTENT *content);
 static int in_backup_content(const char *schema_name,
                              MYX_BACKUP_CONTENT *backup_content);
 
-static MYX_BACKUP_PROFILE *read_in_backup_profile(xmlNodePtr backup_node);
+static MYX_BACKUP_PROFILE *read_in_backup_profile_1_0(xmlNodePtr backup_node);
+static MYX_BACKUP_PROFILE *read_in_backup_profile_1_1(xmlNodePtr backup_node);
 static void read_in_table(xmlNodePtr table_node, MYX_BACKUP_TABLE *table);
 static char* check_statement(const char *line,
                              int line_len, pcre *re, pcre_extra *pe);
@@ -176,10 +181,14 @@ MYX_RBS_STATUS * myx_new_rbs_status(MYSQL *mysql, const char *filename,
                                       ( const char *msg,void *user_data),
                                     void *user_data);
 int myx_free_rbs_status(MYX_RBS_STATUS *rs);
-int myx_restore_backup_from_sql_file_incremental(MYX_RBS_STATUS *rs,
-                                                 int granularity,
-                                                 MYX_BACKUP_ERROR *error);
-
+int myx_restore_backup_from_sql_file_incremental(
+                        MYX_RBS_STATUS *rs,
+                        int (*progress_report)(bigint bytes_read,
+                                               bigint bytes_total,
+                                               void *user_data),
+                        bigint file_size,
+                        void *user_data,
+                        MYX_BACKUP_ERROR *error);
 MYX_BS_STATUS * myx_new_bs_status(MYSQL *mysql, const char *filename,
                                   MYX_BACKUP_CONTENT *content,
                                   MYX_BACKUP_OPTIONS options,
@@ -191,7 +200,6 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
 
 int myx_set_bs_options(MYX_BS_STATUS *status, int options); //TODO: remove that function?
 
-
 /* taken from Viktor Vagins patch */
 #define SAFE_IO(func) \
 {func; if (errno == ENOSPC){*error=MYX_BACKUP_OUTPUTDEVICE_FULL;return -1;}}
@@ -231,6 +239,8 @@ MYX_ADMIN_LIB_ERROR myx_save_profile(const char *profile_name,
 
   root_node= doc->children= xmlNewDocRawNode(doc,NULL,"backup_profile",NULL);
 
+  xmlNewTextChild(root_node,NULL,"version", "1.1");
+  
   xmlNewTextChild(root_node,NULL,"profile_name", backup_profile->profile_name);
   xmlNewTextChild(root_node,NULL,"last_used", backup_profile->last_used);
   NewTextChild_int_content(root_node,NULL, "options", backup_profile->options);
@@ -241,9 +251,9 @@ MYX_ADMIN_LIB_ERROR myx_save_profile(const char *profile_name,
 
   if (backup_profile->backup_content)
   {
-    MYX_BACKUP_TABLE * table= backup_profile->backup_content->tables;
-    MYX_BACKUP_TABLE * tables_end= table +
-                                   backup_profile->backup_content->tables_num;
+    MYX_BACKUP_CONTENT *bc = backup_profile->backup_content;
+    MYX_BACKUP_TABLE *table= bc->tables;
+    MYX_BACKUP_TABLE *tables_end= table + bc->tables_num;
     for (; table < tables_end; table++)
     {
       table_node= xmlNewTextChild(tables_node, NULL, "table", NULL);
@@ -306,14 +316,23 @@ MYX_BACKUP_PROFILE *myx_load_profile(const char *profile_name,
         }
         else
         {
+          xmlNodePtr version = try_to_get_child(doc, root, "version");
           *error_code= MYX_ADMIN_NO_ERROR;
-          backup_profile= read_in_backup_profile(root);
+          if(version)
+          {
+            backup_profile= read_in_backup_profile_1_1(root);
+          }
+          else
+          {
+            backup_profile= read_in_backup_profile_1_0(root);
+          }
         }
       }
       xmlFreeDoc(doc);
     }
   }
   g_free(filename);
+
   return backup_profile;
 }
 
@@ -425,7 +444,7 @@ MYX_BACKUP_ERROR myx_make_backup(MYSQL *mysql, const char *filename,
       int new_table= (last_table_index != (int)bs_status->current_table_index);
 
       interval_counter--;
-      if (progress_report && (new_table || interval_counter <= 0))
+      if (progress_report &&  bs_status->current_table_quoted && (new_table || interval_counter <= 0))
       {
         if (new_table)
           last_table_index= bs_status->current_table_index;
@@ -494,7 +513,15 @@ MYX_BS_STATUS *myx_new_bs_status(MYSQL *mysql, const char *filename,
   b->sql_file= file;
 
   b->mysql= mysql;
-  b->backup_content= copy_backup_content(content);
+  if(options & MYX_B_COMPLETE_SCHEMATAS)
+  {
+    b->backup_content= select_all_tables(mysql, content);
+  }
+
+  //else
+  {
+    b->backup_content= copy_backup_content(content);
+  }
   b->options= options;
   b->current_table_index= -1;
   b->starting= 1; /* set to display that we have not fetched anything yet */
@@ -565,6 +592,44 @@ static const char * show_master_status_fields[]=
 static const char ** show_master_status_fields_end=
              show_master_status_fields + sizeof(show_master_status_fields)/sizeof(char*);
 
+/**
+ * @brief check if should NOT dump table's contents
+ *
+ * @param mysql mysql server connection handle
+ * @param table_name unquoted quoted name of the table to check
+ * 
+ * @return 1 if table should ignore the table
+ */
+static int check_if_ignore_table(MYSQL *mysql, const char *table_name)
+{
+  char buff[1024];
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  int result= 1;
+
+  sprintf(buff, "show table status like \"%s\"", table_name);
+  if (mysql_query(mysql, buff))
+  {
+    return 0;      /* assume table is ok */
+  }
+  res = mysql_store_result(mysql);
+  if (!(row= mysql_fetch_row(res)))
+  {
+    return 0;        /* assume table is ok */
+  }
+  if (!(row[1]))
+      result= 1;
+  else
+  {
+    if (strcmp(row[1], "MRG_MyISAM") &&
+        strcmp(row[1], "MRG_ISAM")) {
+      result= 0;
+    }
+  }
+  mysql_free_result(res);
+  return result;
+}
+
 /*
  *----------------------------------------------------------------------
  * backs up num-rows from the current table
@@ -631,7 +696,7 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
       else if (status->options & MYX_B_SINGLE_TRANSACTION)
       {
         // make sure the correct ISOLATION LEVEL is set
-        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 0, 5) == 0)
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 0, 5))
         {
           if (myx_mysql_query(status->mysql, "SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ"))
           {
@@ -641,15 +706,13 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
         }
 
         // start the transaction
-        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 1, 8) == 0)
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 1, 8))
         {
           if (myx_mysql_query(status->mysql, "START TRANSACTION WITH CONSISTENT SNAPSHOT"))
           {
             *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
             return -1;  /* fatal */
           }
-
-
         }
         else
         {
@@ -673,7 +736,7 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
           return -1;  /* fatal */
         }
         // make sure the correct ISOLATION LEVEL is set
-        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 0, 5) == 0)
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 0, 5))
         {
           if (myx_mysql_query(status->mysql, "SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ"))
           {
@@ -681,10 +744,21 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
             return -1;  /* fatal */
           }
         }
-        if (myx_mysql_query(status->mysql, "START TRANSACTION WITH CONSISTENT SNAPSHOT"))
+        if (mysql_full_version_is_later_or_equal_than(status->mysql, 4, 1, 8))
         {
-          *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
-          return -1;  /* fatal */
+          if (myx_mysql_query(status->mysql, "START TRANSACTION WITH CONSISTENT SNAPSHOT"))
+          {
+            *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+            return -1;  /* fatal */
+          }
+        }
+        else
+        {
+          if (myx_mysql_query(status->mysql, "BEGIN"))
+          {
+            *error= MYX_BACKUP_CANNOT_START_TRANSACTION;
+            return -1;  /* fatal */
+          }
         }
         if (myx_mysql_query(status->mysql, "SHOW MASTER STATUS"))
         {
@@ -695,7 +769,7 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
         {
           MYSQL_RES *res;
 
-          if (res= mysql_store_result(status->mysql))
+          if((res= mysql_store_result(status->mysql)))
           {
             MYSQL_ROW row;
             MYSQL_FIELD *fields;
@@ -712,7 +786,7 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
             build_field_subst(show_master_status_fields, show_master_status_fields_end,
               fields, fields + num_fields, fi);
 
-            if (row= mysql_fetch_row(res))
+            if((row= mysql_fetch_row(res)))
             {
               if (fi[0] > -1)
                 binlog_filename= myx_convert_dbstr_utf8(status->mysql, row[fi[0]]);
@@ -756,38 +830,57 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
     current_schema=
             status->backup_content->tables[status->current_table_index].schema;
     g_free(status->current_table_quoted);
-    status->current_table_quoted= quote_identifier_with(current_table,
-                                                        status->quote_char);
-    g_free(status->current_schema_quoted);
-    status->current_schema_quoted= quote_identifier_with(current_schema,
-                                                         status->quote_char);
-    status->current_table_rows_processed= 0;
-
-    /* find out how many rows this table has.. */
-    sel_stmt= g_strconcat("SELECT count(*) FROM ",
-                          status->current_schema_quoted,
-                          ".", status->current_table_quoted, NULL);
-    if ( myx_mysql_query(status->mysql, sel_stmt) ||
-         !(result= mysql_store_result(status->mysql)) )
+    if(current_table)
     {
-      g_free(sel_stmt);
-      *error= MYX_BACKUP_MYSQL_ERROR;
-      return 0;
-    }
-    g_free(sel_stmt);
-
-    row= mysql_fetch_row(result);
-    if (row)
-    {
-      status->current_table_rows= atoi(row[0]);
+      status->current_table_quoted= quote_identifier_with(current_table,
+                                                          status->quote_char);
     }
     else
     {
+      status->current_table_quoted= NULL;
+    }
+    g_free(status->current_schema);
+    g_free(status->current_schema_quoted);
+    status->current_schema_quoted= quote_identifier_with(current_schema,
+                                                         status->quote_char);
+    status->current_schema= g_strdup(current_schema);
+    status->current_table_rows_processed= 0;
+
+    if(current_table)
+    {
+      status->ignore = check_if_ignore_table(status->mysql, current_table);
+      /* find out how many rows this table has.. */
+      sel_stmt= g_strconcat("SELECT count(*) FROM ",
+                            status->current_schema_quoted,
+                            ".", status->current_table_quoted, NULL);
+      if ( myx_mysql_query(status->mysql, sel_stmt) ||
+          !(result= mysql_store_result(status->mysql)) )
+      {
+        g_free(sel_stmt);
+        *error= MYX_BACKUP_MYSQL_ERROR;
+        return 0;
+      }
+      g_free(sel_stmt);
+
+      row= mysql_fetch_row(result);
+      if (row)
+      {
+        status->current_table_rows= atoi(row[0]);
+      }
+      else
+      {
+        mysql_free_result(result);
+        *error= MYX_BACKUP_MYSQL_ERROR;
+        return 0;
+      }
       mysql_free_result(result);
-      *error= MYX_BACKUP_MYSQL_ERROR;
-      return 0;
+    }
+    else
+    {
+      status->current_table_rows= 0;
+      status->current_table_rows_processed= 0;
+      status->ignore= 1;
     }
-    mysql_free_result(result);
 
     /* write the path-information for this table */
     if (!(status->options & MYX_B_DONT_WRITE_FULL_PATH) &&
@@ -796,56 +889,65 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
       return -1;
     }
 
-    /* get the create-statement of the new table */
-    if (!(status->options & MYX_B_NO_CREATES) &&
-        write_create_statement_to_file(status, error))
+    if(current_table)
     {
-      return -1;
-    }
+      /* get the create-statement of the new table */
+      if (!(status->options & MYX_B_NO_CREATES) &&
+          write_create_statement_to_file(status, error))
+      {
+        return -1;
+      }
 
-    if (status->options & MYX_B_COMMENT)
-    {
-      SAFE_IO(fprintf(status->sql_file, "\n--\n"
-        "-- Dumping data for table %s.%s\n"
-        "--\n", 
-        status->current_schema_quoted,
-        status->current_table_quoted));
-    }
+      if (status->options & MYX_B_COMMENT)
+      {
+        SAFE_IO(fprintf(status->sql_file, "\n--\n"
+          "-- Dumping data for table %s.%s\n"
+          "--\n", 
+          status->current_schema_quoted,
+          status->current_table_quoted));
+      }
 
-    /* get the rows of the new table */
-    sel_stmt= g_strconcat("SELECT /*!40001 SQL_NO_CACHE */ * FROM ",
-                          status->current_schema_quoted, ".",
-                          status->current_table_quoted, NULL);
-    if (myx_mysql_query(status->mysql, sel_stmt) ||
-        !(status->mysql_result= mysql_use_result(status->mysql)) )
-    {
+      /* get the rows of the new table */
+      sel_stmt= g_strconcat("SELECT /*!40001 SQL_NO_CACHE */ * FROM ",
+                            status->current_schema_quoted, ".",
+                            status->current_table_quoted, NULL);
+      if (myx_mysql_query(status->mysql, sel_stmt) ||
+          !(status->mysql_result= mysql_use_result(status->mysql)) )
+      {
+        g_free(sel_stmt);
+        *error= MYX_BACKUP_MYSQL_ERROR;
+        return 0;
+      }
       g_free(sel_stmt);
-      *error= MYX_BACKUP_MYSQL_ERROR;
-      return 0;
-    }
-    g_free(sel_stmt);
 
-    if (status->options &  MYX_B_DISABLE_KEYS)
-    {
-      SAFE_IO(fprintf(status->sql_file,
-                      "\n/*!40000 ALTER TABLE %s DISABLE KEYS */;\n",
-                      status->current_table_quoted));
-    }
+      if (status->options &  MYX_B_DISABLE_KEYS)
+      {
+        SAFE_IO(fprintf(status->sql_file,
+                        "\n/*!40000 ALTER TABLE %s DISABLE KEYS */;\n",
+                        status->current_table_quoted));
+      }
 
-    if (status->options & MYX_B_ADD_LOCKS)
-    {
-      SAFE_IO(fprintf(status->sql_file,"LOCK TABLES %s WRITE;\n",
-              status->current_table_quoted));
+      if (status->options & MYX_B_ADD_LOCKS)
+      {
+        SAFE_IO(fprintf(status->sql_file,"LOCK TABLES %s WRITE;\n",
+                status->current_table_quoted));
+      }
     }
   }
 
-  /* we will fetch num_rows rows every time 
-                                    (but less if the table has no more rows) */
-  for (;num_rows && (row= mysql_fetch_row(status->mysql_result));
-       num_rows--, status->current_table_rows_processed++)
+  if(status->current_table_quoted)
   {
-    if (write_row_to_file(status, row, error))
-      return -1;
+    /* we will fetch num_rows rows every time 
+                                    (but less if the table has no more rows) */
+    for (;num_rows && (row= mysql_fetch_row(status->mysql_result));
+        num_rows--, status->current_table_rows_processed++)
+    {
+      if(!status->ignore)
+      {
+        if (write_row_to_file(status, row, error))
+          return -1;
+      }
+    }
   }
 
   /* if backing up this table has ended */
@@ -879,41 +981,45 @@ int myx_make_backup_to_sql_file_incremental(MYX_BS_STATUS *status,
       SAFE_IO(fprintf(status->sql_file, "%s\n", tmp_str));
       g_free(tmp_str);
     }
-    if (status->options & MYX_B_ADD_LOCKS)
-      SAFE_IO(fprintf(status->sql_file,"UNLOCK TABLES;\n"));
-
-    if (status->options &  MYX_B_DISABLE_KEYS)
+    
+    if(status->current_table_quoted)
     {
-      SAFE_IO(fprintf(status->sql_file,
-                      "/*!40000 ALTER TABLE %s ENABLE KEYS */;\n\n",
-                      status->current_table_quoted));
-    }
+      if (status->options & MYX_B_ADD_LOCKS)
+        SAFE_IO(fprintf(status->sql_file,"UNLOCK TABLES;\n"));
 
-    /* if this is the last table */
-    if (status->current_table_index+1 >= status->backup_content->tables_num)
-    {
-      if (status->options & MYX_B_LOCK_ALL_TABLES)
+      if (status->options &  MYX_B_DISABLE_KEYS)
       {
-        if (myx_mysql_query(status->mysql, "UNLOCK TABLES"))
+        SAFE_IO(fprintf(status->sql_file,
+                        "/*!40000 ALTER TABLE %s ENABLE KEYS */;\n\n",
+                        status->current_table_quoted));
+      }
+
+      /* if this is the last table */
+      if (status->current_table_index+1 >= status->backup_content->tables_num)
+      {
+        if (status->options & MYX_B_LOCK_ALL_TABLES)
         {
-          *error=  MYX_BACKUP_MYSQL_ERROR;
+          if (myx_mysql_query(status->mysql, "UNLOCK TABLES"))
+          {
+            *error=  MYX_BACKUP_MYSQL_ERROR;
+            return -1;
+          }
+        }
+        else if ((
+          (status->options & MYX_B_SINGLE_TRANSACTION) ||
+          (status->options & MYX_B_POINT_IN_TIME_BACKUP)) &&
+          myx_mysql_query(status->mysql, "COMMIT"))
+        {
+          *error= MYX_BACKUP_MYSQL_ERROR;
+          return -1;  /* fatal */
+        }
+
+        if (myx_mysql_query(status->mysql, "/*!40101 SET sql_mode=@OLD_SQL_MODE */"))
+        {
+          *error= MYX_BACKUP_MYSQL_ERROR;
           return -1;
         }
       }
-      else if ((
-        (status->options & MYX_B_SINGLE_TRANSACTION) ||
-        (status->options & MYX_B_POINT_IN_TIME_BACKUP)) &&
-        myx_mysql_query(status->mysql, "COMMIT"))
-      {
-        *error= MYX_BACKUP_MYSQL_ERROR;
-        return -1;  /* fatal */
-      }
-
-      if (myx_mysql_query(status->mysql, "/*!40101 SET sql_mode=@OLD_SQL_MODE */"))
-      {
-        *error= MYX_BACKUP_MYSQL_ERROR;
-        return -1;
-      }
     }
   }
 
@@ -1076,7 +1182,7 @@ int myx_get_backup_content_from_sql_file_incremental(MYX_BCS_STATUS *bc_status,
                                                      MYX_BACKUP_ERROR *error)
 {
   int len, tables_found;
-  char buffer[BUFFER_LEN];
+  char buffer[BACKUP_CONTENT_BUFFER_LEN];
   char *bufptr= (char*)buffer;
   int buffer_len= sizeof(buffer);
   int count= 0;
@@ -1087,7 +1193,7 @@ int myx_get_backup_content_from_sql_file_incremental(MYX_BCS_STATUS *bc_status,
          (len= myx_get_next_sql_statement_file(&bc_status->we,
                                                bc_status->intl_file,
                                                &bufptr, &buffer_len,
-                                               0,(int *)error)) )
+                                               0, 0, 0, 0, (int *)error)) )
   {    
     if (*error)
       return -1;
@@ -1209,8 +1315,9 @@ MYX_BACKUP_ERROR myx_restore_backup(MYSQL *mysql, const char *filename,
     if (mysql_full_version_is_later_or_equal_than(mysql, 4, 1, 1))
       store_old_cs_set_utf8(mysql);
 
-    while ( myx_restore_backup_from_sql_file_incremental(rs, 1, &error) > 0)
+    while (myx_restore_backup_from_sql_file_incremental(rs, progress_report, file_size, ruser_data, &error) > 0)
     {
+/*
       if ((progress_report) && (rs->we.bytes_read >= next_call))
       {
         next_call+= report_interval;
@@ -1222,6 +1329,12 @@ MYX_BACKUP_ERROR myx_restore_backup(MYSQL *mysql, const char *filename,
           return 0;
         }
       }
+*/
+    }
+    
+    if(error)
+    {
+      myx_set_mysql_error(g_strdup(mysql_error(mysql)), mysql_errno(mysql));
     }
 
     if (mysql_full_version_is_later_or_equal_than(mysql, 4, 1, 1))
@@ -1294,16 +1407,16 @@ MYX_RBS_STATUS *myx_new_rbs_status(MYSQL *mysql, const char *filename,
   /* setup the regular expressions */
   /* drop table may only contain one table at the moment */
   rs->re_drop= pcre_compile
-                          ("drop\\s+table\\s+(?:if\\s+exists\\s+)?(\\S+)\\s*;",
+                          ("drop\\s+table\\s+(?:if\\s+exists\\s+)?"TABLE_NAME_PCRE"\\s*;",
                            PCRE_ANCHORED|PCRE_CASELESS|PCRE_DOTALL,
                            &error_str, &erroffset, NULL);
-  rs->re_insert= pcre_compile("insert\\s+into\\s+(\\S+)",
+  rs->re_insert= pcre_compile("insert\\s+into\\s+"TABLE_NAME_PCRE,
                               PCRE_ANCHORED|PCRE_CASELESS,
                               &error_str, &erroffset, NULL);
-  rs->re_alter= pcre_compile("(?:\\/\\*\\!\\d+\\s+)?alter\\s+table\\s+(\\S+)",
+  rs->re_alter= pcre_compile("(?:\\/\\*\\!\\d+\\s+)?alter\\s+table\\s+"TABLE_NAME_PCRE,
                              PCRE_ANCHORED|PCRE_CASELESS, &error_str,
                              &erroffset, NULL);
-  rs->re_lock= pcre_compile("lock\\s+tables\\s+(\\S+)\\s+write\\s*;",
+  rs->re_lock= pcre_compile("lock\\s+tables\\s+"TABLE_NAME_PCRE"\\s+write\\s*;",
                             PCRE_ANCHORED|PCRE_CASELESS, &error_str,
                             &erroffset, NULL);
   rs->re_unlock= pcre_compile("unlock\\s+(tables)",
@@ -1353,6 +1466,38 @@ MYX_RBS_STATUS *myx_new_rbs_status(MYSQL *mysql, const char *filename,
   return rs;
 }
 
+static void ensure_charset_present(char **pbuf, int *plen, const char *defcharset)
+{
+  int vector[6];
+  const char *error;
+  int erroffset;
+  char *buf, *c;
+  size_t cslen;
+
+  pcre *re= pcre_compile(".*(.*).*DEFAULT[\\s]+CHARSET=[a-zA-Z0-9]+.*", /* the pattern */
+                        PCRE_CASELESS|PCRE_MULTILINE,                   /* default options */
+                        &error,                                         /* for error message */
+                        &erroffset,                                     /* for error offset */
+                        NULL);                                          /* use default character tables */
+  if(pcre_exec(re, NULL, *pbuf, *plen, 0, 0, vector, 6) > 0)
+  {
+    // DEFAULT CHARSET is present in the statement
+    return;
+  }
+  cslen= strlen (defcharset);
+  buf= g_malloc((gulong)(*plen + cslen + 32)); // 128 fro "DEFAULT CHARSET="
+  strcpy(buf, *pbuf);
+  for(c= buf + *plen - 1; (*c == ' ') && (c > buf); c--);
+  strcpy(c, " DEFAULT CHARSET=");
+  c += sizeof(" DEFAULT CHARSET=")-1;
+  strcpy(c, defcharset);
+  c += cslen;
+  strcpy(c, ";");
+  g_free(*pbuf);
+  *pbuf= buf;
+  *plen= (int)strlen(buf);
+}
+
 int myx_free_rbs_status(MYX_RBS_STATUS *rs)
 {
   if (rs)
@@ -1387,39 +1532,52 @@ int myx_free_rbs_status(MYX_RBS_STATUS *rs)
  * 0 if there is no more sql-statement
  * a negative number in case of an error
  */
-int myx_restore_backup_from_sql_file_incremental(MYX_RBS_STATUS *rs,
-                                                 int granularity,
-                                                 MYX_BACKUP_ERROR *error)
+int myx_restore_backup_from_sql_file_incremental(
+                  MYX_RBS_STATUS *rs,
+                  int (*progress_report)(bigint bytes_read,
+                                         bigint bytes_total,
+                                         void *user_data),
+                  bigint file_size,
+                  void *user_data,
+                  MYX_BACKUP_ERROR *error)
 {
   int len;
-  int count, found;
-  char *tmp;
+  int count = 0, found;
+  char *tmp, *create;
   char *warning;
 
   /* We will only execute statements we know. Otherwise we could do something
    * bad without knowing it.
    */
-
   for (count= 0;
-       count < granularity &&
+       count < 1 &&
        (len= myx_get_next_sql_statement_file(&rs->we,rs->intl_file,
                                              &rs->buffer,
                                              &rs->buffer_len,1,
-                                             (int *)error));
+                                             progress_report,
+                                             file_size,
+                                             user_data,
+                                             (int *)error)) > 0;
        count++)
   {
     if (*error)
       return -1;
 
     assert(rs->buffer);
+    create= NULL;
 
     /* do some checks now */
     if ((tmp= check_statement(rs->buffer,len, rs->re_insert, NULL)) ||
         (tmp= check_statement(rs->buffer,len, rs->re_drop,   NULL)) ||
         (tmp= check_statement(rs->buffer,len, rs->re_alter,  NULL)) ||
         (tmp= check_statement(rs->buffer,len, rs->re_lock,   NULL)) ||
-        (tmp= check_statement(rs->buffer,len, rs->re_create, NULL)))
+        (tmp= create= check_statement(rs->buffer,len, rs->re_create, NULL)))
     {
+      if(create)
+      {
+        ensure_charset_present(&rs->buffer, &len, rs->intl_file->charset);
+        rs->buffer_len= len;
+      }
       tmp= unquote_identifier(tmp);
       found= find_table(rs->backup_content, tmp,
                         rs->current_schema_in_sqlfile,
@@ -1525,7 +1683,7 @@ static int in_backup_content(const char *schema_name,
   return 0;
 }
 
-static MYX_BACKUP_PROFILE *read_in_backup_profile(xmlNodePtr backup_node)
+static MYX_BACKUP_PROFILE *read_in_backup_profile_1_1(xmlNodePtr backup_node)
 {
   MYX_BACKUP_PROFILE *bprofile;
   xmlNodePtr cur;
@@ -1564,12 +1722,56 @@ static MYX_BACKUP_PROFILE *read_in_backup_profile(xmlNodePtr backup_node)
   return bprofile;
 }
 
+static MYX_BACKUP_PROFILE *read_in_backup_profile_1_0(xmlNodePtr backup_node)
+{
+  MYX_BACKUP_PROFILE *bprofile;
+  xmlNodePtr cur;
+  xmlDocPtr doc;
+
+  doc= backup_node->doc;
+  bprofile= g_malloc0(sizeof(MYX_BACKUP_PROFILE));
+  bprofile->backup_content= g_malloc0(sizeof(MYX_BACKUP_CONTENT));
+
+  for (cur= backup_node->children; cur; cur= cur->next)
+  {
+    try_to_get_string_field(doc,cur, "profile_name",&bprofile->profile_name);
+    try_to_get_string_field(doc,cur, "last_used",   &bprofile->last_used);
+    try_to_get_int_field   (doc,cur, "options",     &bprofile->options);
+    try_to_get_int_field   (doc,cur, "backup_type",
+                            (int*)&bprofile->backup_type);
+
+    if ( !xmlStrcmp(cur->name, "tables") &&  cur->type == XML_ELEMENT_NODE)
+    {
+      xmlNodePtr cur2;
+      MYX_BACKUP_TABLE * table;
+      bprofile->backup_content->tables_num= get_child_count(cur,"table");
+      table= bprofile->backup_content->tables=
+                         g_malloc0(sizeof(MYX_BACKUP_TABLE) *
+                                   bprofile->backup_content->tables_num);
+      for (cur2= cur->children; cur2; cur2= cur2->next)
+      {
+        if (!xmlStrcmp(cur2->name, "table") )
+        {
+          read_in_table(cur2, table);
+          table++;
+        }
+      }
+    }
+  }
+
+  return bprofile;
+}
+
+
 static void read_in_table(xmlNodePtr table_node, MYX_BACKUP_TABLE *table)
 {
   xmlNodePtr cur;
   xmlDocPtr doc;
 
   doc= table_node->doc;
+  table->table= NULL;
+  table->schema= NULL;
+  table->catalog= NULL;
 
   for (cur= table_node->children; cur; cur= cur->next)
   {
@@ -1579,6 +1781,97 @@ static void read_in_table(xmlNodePtr table_node, MYX_BACKUP_TABLE *table)
   }
 }
 
+static GPtrArray *select_all_schemata(MYX_BACKUP_CONTENT *bc)
+{
+  unsigned int i;
+  GPtrArray *array= g_ptr_array_new();
+  GHashTable *hash= g_hash_table_new(g_str_hash, g_str_equal);
+  
+  for(i= 0; i < bc->tables_num; i++)
+  {
+    if(!g_hash_table_lookup(hash, bc->tables[i].schema))
+    {
+      MYX_BACKUP_TABLE *bt= g_malloc(sizeof(MYX_BACKUP_TABLE));
+      bt->catalog= bc->tables[i].catalog;
+      bt->schema= bc->tables[i].schema;
+      bt->table= NULL;
+      g_ptr_array_add(array, bt);
+      g_hash_table_insert(hash, bc->tables[i].schema, bc->tables[i].schema);
+    }
+  }
+
+  g_hash_table_destroy(hash);
+  return array;
+}
+
+static MYX_BACKUP_CONTENT* select_all_tables(MYSQL* mysql, 
+                                             MYX_BACKUP_CONTENT *bc)
+{
+  unsigned int i, j;
+  MYSQL_RES *rs= NULL;
+  MYSQL_ROW row;
+
+  GPtrArray *all_tables = g_ptr_array_new();
+  GPtrArray *all_schemas = select_all_schemata(bc);
+
+  for(i= 0; i < all_schemas->len; i++)
+  {
+    MYX_BACKUP_TABLE *bt= 
+              (MYX_BACKUP_TABLE *)g_ptr_array_index(all_schemas, i);
+    if(mysql_select_db(mysql, bt->schema) 
+      || mysql_query(mysql, "SHOW TABLES")
+      || !(rs= mysql_store_result(mysql)))
+    {
+      return NULL;
+    }
+    j= 0;
+    while((row= mysql_fetch_row(rs)))
+    {
+      MYX_BACKUP_TABLE *btnew= 
+                (MYX_BACKUP_TABLE *)g_malloc(sizeof(MYX_BACKUP_TABLE));
+      btnew->catalog= bt->catalog;
+      btnew->schema= bt->schema;
+      btnew->table= row[0];
+      g_ptr_array_add(all_tables, btnew);
+      ++j;
+    }
+    if(!j)  // empty schema
+    {
+      MYX_BACKUP_TABLE *btnew= 
+                (MYX_BACKUP_TABLE *)g_malloc(sizeof(MYX_BACKUP_TABLE));
+      btnew->catalog= bt->catalog;
+      btnew->schema= bt->schema;
+      btnew->table= NULL;
+      g_ptr_array_add(all_tables, btnew);
+    }
+  }
+
+  bc->tables = g_malloc(sizeof(MYX_BACKUP_TABLE)*all_tables->len);
+  bc->tables_num = all_tables->len;
+
+  for(i= 0; i < all_tables->len; i++)
+  {
+    MYX_BACKUP_TABLE *bt= (MYX_BACKUP_TABLE *)g_ptr_array_index(all_tables, i);
+    bc->tables[i].catalog= g_strdup(bt->catalog);
+    bc->tables[i].schema= g_strdup(bt->schema);
+    bc->tables[i].table= g_strdup(bt->table);
+    g_free(bt);
+  }
+
+  g_ptr_array_free(all_tables, 0);
+  for(i= 0; i < all_schemas->len; i++)
+  {
+    g_free(g_ptr_array_index(all_schemas, i));
+  }
+  g_ptr_array_free(all_schemas, 0);
+  if(rs)
+  {
+    mysql_free_result(rs);
+  }
+
+  return bc;
+}
+
 static MYX_BACKUP_CONTENT* copy_backup_content(MYX_BACKUP_CONTENT *content)
 {
   MYX_BACKUP_TABLE * src_table= content->tables;
@@ -1592,7 +1885,14 @@ static MYX_BACKUP_CONTENT* copy_backup_content(MYX_BACKUP_CONTENT *content)
   {
     table->catalog= g_strdup(src_table->catalog);
     table->schema= g_strdup(src_table->schema);
-    table->table= g_strdup(src_table->table);
+    if(src_table->table && src_table->table[0])
+    {
+      table->table= g_strdup(src_table->table);
+    }
+    else
+    {
+      table->table= NULL;
+    }
   }
 
   return copy;
@@ -1616,12 +1916,53 @@ static int  compare_backup_tables(const void *a, const void *b)
   }
 }
 
+static int write_create_schema_to_file(MYSQL *mysql, 
+                                       FILE *sql_file, 
+                                       const char *schema_quoted,
+                                       MYX_BACKUP_ERROR *error)
+{
+  MYSQL_ROW row;
+  MYSQL_RES *result;
+  char *database_name;
+  char *stmt= g_strconcat("SHOW CREATE DATABASE WITH IF NOT EXISTS ",
+                            schema_quoted,  NULL);
+
+  if (myx_mysql_query(mysql, stmt) ||
+      !(result= mysql_store_result(mysql)) )
+  {
+    SAFE_IO(fprintf(sql_file, "--\n"
+      "-- Create schema %s\n"
+      "--\n\n", schema_quoted));
+
+    SAFE_IO(fprintf(sql_file,
+                    "CREATE DATABASE /*!32312 IF NOT EXISTS*/ %s;\n",
+                    schema_quoted));
+  }
+  else
+  {
+    row= mysql_fetch_row(result);
+    if (!row)
+    {
+      g_free(stmt);
+      *error= MYX_BACKUP_MYSQL_ERROR;
+      return -1;
+    }
+    database_name= myx_convert_dbstr_utf8(mysql, row[1]);
+    SAFE_IO(fprintf(sql_file, "%s;\n", database_name));
+    g_free(database_name);
+  }
+  g_free(stmt);
+  SAFE_IO(fprintf(sql_file, "USE %s;\n",
+                  schema_quoted));
+
+  return 0;
+}
+
 static int write_path_info_to_file(MYX_BS_STATUS *status,
                                    MYX_BACKUP_ERROR *error)
 {
+  int result;
   MYX_BACKUP_TABLE *current, *last;
-  char *database_name;
-
   current= status->backup_content->tables + status->current_table_index;
   last= current - 1;
 
@@ -1629,38 +1970,21 @@ static int write_path_info_to_file(MYX_BS_STATUS *status,
   if ( status->current_table_index == 0 ||
        strcmp(current->schema, last->schema) )
   {
-    MYSQL_ROW row;
-    MYSQL_RES *result;
-    char *stmt= g_strconcat("SHOW CREATE DATABASE WITH IF NOT EXISTS ",
-                            status->current_schema_quoted,  NULL);
+    if(myx_identifier_needs_quotes(status->current_schema))
+    {
+      result= write_create_schema_to_file(status->mysql, status->sql_file, 
+                                     status->current_schema_quoted, error);
+    } 
+    else 
+    {
+      result= write_create_schema_to_file(status->mysql, status->sql_file, 
+                                     status->current_schema, error);
+    }
     
-    if (myx_mysql_query(status->mysql, stmt) ||
-        !(result= mysql_store_result(status->mysql)) )
+    if(result)
     {
-      SAFE_IO(fprintf(status->sql_file, "--\n"
-        "-- Create schema %s\n"
-        "--\n\n", status->current_schema_quoted));
-
-      SAFE_IO(fprintf(status->sql_file,
-                      "CREATE DATABASE /*!32312 IF NOT EXISTS*/ %s;\n",
-                      status->current_schema_quoted));
+      return -1;
     }
-    else
-    {
-      row= mysql_fetch_row(result);
-      if (!row)
-      {
-        g_free(stmt);
-        *error= MYX_BACKUP_MYSQL_ERROR;
-        return -1;
-      }
-      database_name= myx_convert_dbstr_utf8(status->mysql, row[1]);
-      SAFE_IO(fprintf(status->sql_file, "%s;\n", database_name));
-      g_free(database_name);
-    }
-    g_free(stmt);
-    SAFE_IO(fprintf(status->sql_file, "USE %s;\n",
-                    status->current_schema_quoted));
   }
   return 0;
 }
@@ -1690,9 +2014,9 @@ static int write_sql_file_header(MYX_BS_STATUS *status,
   SAFE_IO(fprintf(status->sql_file, "/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;\n"));
 
   if (status->options & MYX_B_ANSI_QUOTES)
-    g_sprintf(Buffer, "%s, ANSI_QUOTES", Buffer);
+    g_sprintf(Buffer, "%s,ANSI_QUOTES", Buffer);
   if (status->options & MYX_B_COMPATIBILITY_MODE)
-    g_sprintf(Buffer, "%s, MYSQL323", Buffer);
+    g_sprintf(Buffer, "%s,MYSQL323", Buffer);
   g_sprintf(Buffer, "%s' */;\n\n\n", Buffer);
   SAFE_IO(fprintf(status->sql_file, Buffer));
 
@@ -1989,6 +2313,7 @@ static int interact_with_server(MYX_RBS_STATUS *rs,
         {
           /* error is already set by comfort_query */
           g_free(q_use);
+
           g_free(q_cre);
           return -1;
         }
@@ -2068,6 +2393,7 @@ static char* check_statement(const char *line, int line_len,
                       o_vector, O_VECTOR_COUNT) ) > 0)
   {
     pcre_get_substring(line, o_vector, rc, 1, &ret_val);
+
     return_value= g_strdup(ret_val);
     pcre_free_substring((char*)ret_val);
     return return_value;
@@ -2082,7 +2408,7 @@ static int prepare_line_is_create_statement(pcre **re, pcre_extra **pe)
   const char *error;
   int erroffset;
 
-  if (! (*re = pcre_compile("create\\s+table\\s+(\\S+)", /* the pattern */
+  if (! (*re = pcre_compile("create\\s+table\\s+"TABLE_NAME_PCRE, /* the pattern */
                             PCRE_CASELESS|PCRE_ANCHORED, /* default options */
                             &error, /* for error message */
                             &erroffset, /* for error offset */
@@ -2208,6 +2534,7 @@ MYX_BACKUP_CONTENT * myx_get_restore_drop_list(MYSQL *mysql, MYX_BACKUP_CONTENT
     }
   }
 
+
   for (k= 0; k < schemas_num; k++)
   {
     if (schemas[k].tables)
