@@ -419,7 +419,10 @@ char * q_select_db_50[]=
   SELECT_DB_PRIV_41( "Create_tmp_table_priv" )   WHERE_V_41(),
   SELECT_DB_PRIV_41(      "Lock_tables_priv" )   WHERE_V_41(),
   SELECT_DB_PRIV_41(      "Create_view_priv" )   WHERE_V_41(),
-  SELECT_DB_PRIV_41(        "Show_view_priv" )   WHERE_V_41()
+  SELECT_DB_PRIV_41(        "Show_view_priv" )   WHERE_V_41(),
+  SELECT_DB_PRIV_41(   "Create_routine_priv" )   WHERE_V_41(),
+  SELECT_DB_PRIV_41(    "Alter_routine_priv" )   WHERE_V_41(),
+  SELECT_DB_PRIV_41(          "Execute_priv" )   WHERE_V_41()
 };
 
 Query_privs q_select_db_privs[q_v_count]=
@@ -527,6 +530,24 @@ Query_privs q_select_columns_privs[q_v_count]=
   }
 };
 
+
+
+static char *replace_escaped(MYSQL *mysql, const char *query, 
+                             const char *var, const char *value)
+{
+  char *esc_value= g_malloc((gulong)strlen(value)*2+8);
+  char *res;
+  
+  strcpy(esc_value, "'");
+  mysql_real_escape_string(mysql, esc_value+1, value, (gulong)strlen(value));
+  strcat(esc_value, "'");
+
+  res= str_g_subst(query, var, esc_value);
+  g_free(esc_value);
+  
+  return res;
+}
+
 char * myx_resize_vector_block(char * ptr,
                                size_t size_of_block,
                                size_t count_of_blocks)
@@ -571,18 +592,34 @@ void myx_free_vector_block(char * ptr)
   }
 }
 
-#define MYX_VECTOR_RESERVE(ptr,size)\
-  (((char*)(ptr))= myx_resize_vector_block((char*)(ptr),sizeof(*(ptr)),\
-                                           (size_t)(size)))
+#ifdef __GNUC__
+// same as the alternative version, but this won't trigger warnings from gcc
+# define MYX_VECTOR_RESERVE(ptr,size)\
+    (ptr)= (typeof(ptr))myx_resize_vector_block((char*)(ptr),sizeof(*(ptr)),\
+                                               (size_t)(size))
 
-#define MYX_VECTOR_PUSH_BACK(ptr,num,ptr_to_top)\
-{\
-  (num)++;\
-  ((char*)(ptr))= myx_resize_vector_block((char*)(ptr),sizeof(*(ptr)),(num));\
-  (ptr_to_top)= (ptr) + (num) - 1;\
-}
+# define MYX_VECTOR_PUSH_BACK(ptr,num,ptr_to_top)\
+      {\
+        (num)++;\
+        (ptr)= (typeof(ptr))myx_resize_vector_block((char*)(ptr),sizeof(*(ptr)),(num));\
+        (ptr_to_top)= (ptr) + (num) - 1;\
+      }
 
-#define MYX_VECTOR_FREE(ptr) myx_free_vector_block((char*)ptr)
+# define MYX_VECTOR_FREE(ptr) myx_free_vector_block((char*)ptr)
+#else
+# define MYX_VECTOR_RESERVE(ptr,size)\
+    (((char*)(ptr))= myx_resize_vector_block((char*)(ptr),sizeof(*(ptr)),\
+                                               (size_t)(size)))
+
+# define MYX_VECTOR_PUSH_BACK(ptr,num,ptr_to_top)\
+      {\
+        (num)++;\
+        ((char*)(ptr))= myx_resize_vector_block((char*)(ptr),sizeof(*(ptr)),(num));\
+        (ptr_to_top)= (ptr) + (num) - 1;\
+      }
+
+# define MYX_VECTOR_FREE(ptr) myx_free_vector_block((char*)ptr)
+#endif
 
 static const char * select_privileges_names[]=
 {
@@ -661,13 +698,18 @@ int get_privileges_from_query(MYSQL *mysql, MYX_USER * user,
   my_ulonglong num_rows;
   MYSQL_FIELD * fields;
   int fi[4];
-
-  size_t len= strlen(query);
-  if (myx_mysql_real_query(mysql, query, (unsigned int)len) ||
+  char *q;
+  size_t len;
+  
+  q= replace_escaped(mysql, query, "@goal_user", user->user_name);
+  len= strlen(q);
+  if (myx_mysql_real_query(mysql, q, (unsigned int)len) ||
       !(res= mysql_store_result(mysql)))
   {
+    g_free(q);
     return -1;
   }
+  g_free(q);
 
   num_rows= mysql_num_rows(res);
   if (num_rows==0)
@@ -681,13 +723,13 @@ int get_privileges_from_query(MYSQL *mysql, MYX_USER * user,
   {
     char * host=        fi[0]==-1 ? "" : row[fi[0]];
     char * object_name= fi[1]==-1 ? "" : row[fi[1]];
-    char * priv_name=   fi[2]==-1 ? "" :
+    char * priv_name=   fi[2]==-1 ? NULL :
                                      myx_convert_dbstr_utf8(mysql, row[fi[2]]);
     if (!object_name)
       object_name= "";
     if (priv_name[0]!='+')
     {
-      char * priv_value=  fi[3]==-1 ? "" :
+      char * priv_value=  fi[3]==-1 ? NULL :
                                      myx_convert_dbstr_utf8(mysql, row[fi[3]]);
       MYX_USER_OBJECT_PRIVILEGES *o_priv= safe_add_object(mysql,user,
                                                           object_name,
@@ -695,7 +737,7 @@ int get_privileges_from_query(MYSQL *mysql, MYX_USER * user,
       unsigned char ** priv;
       MYX_VECTOR_PUSH_BACK(o_priv->user_privileges,
                            o_priv->user_privileges_num,priv);
-      *priv= g_strdup_printf("%s=%s", priv_name, priv_value);
+      *priv= g_strdup_printf("%s=%s", priv_name ? priv_name : "", priv_value ? priv_value : "");
       g_free(priv_value);
     }
     g_free(priv_name);
@@ -726,33 +768,31 @@ MYX_USER * myx_get_user_opt_privileges(MYSQL *mysql,
                                        int get_privileges)
 {
   MYX_USER *user= 0;
-  unsigned char sqlcmd[255];
-  unsigned char * sqlcmd_end;
   unsigned long * field_lengths;
+  char *command;
   MYSQL_RES* res;
   MYSQL_ROW row;
   QUERY_VARIANTS v;
 
-  sqlcmd_end= strmov(sqlcmd, "set @goal_user=\'");
-  sqlcmd_end+= mysql_real_escape_string(mysql,sqlcmd_end,
-                                        user_name,
-                                        (unsigned long)strlen(user_name));
-  sqlcmd_end= strmov(sqlcmd_end, "\'");
-  if (myx_mysql_query(mysql, sqlcmd))
-    return NULL;
-
   v= mysql_version_is_later_or_equal_than(mysql,5,0) ? q_v_5_0 :
      mysql_version_is_later_or_equal_than(mysql,4,1) ? q_v_4_1 : q_v_base;
 
+  /* can't use local variables, because it won't work with replication on 4.0
+   * servers */
+  command= replace_escaped(mysql, q_select_user_info[v], "@goal_user", user_name);
+  
   // --------------------------------------------------------------------------
   //Get additional user infos
-  if (myx_mysql_query(mysql, q_select_user_info[v]))
+  if (myx_mysql_query(mysql, command))
   { // error
+    g_free(command);
     if (!(myx_check_mysql_user_info_table(mysql)))
       return NULL; // Check if user_info tables exists. if not, create it
   }
   else
   {
+    g_free(command);
+
     if ( !(res= mysql_store_result(mysql)) )
       return NULL; // error
     if ( (row= mysql_fetch_row(res)))
@@ -858,6 +898,31 @@ int myx_free_user_priv(MYX_USER_OBJECT_PRIVILEGES *user_priv)
   return 0;
 }
 
+
+static char *replace_username(MYSQL *mysql, const char *query,
+                            const char *new_name, const char *cur_name)
+{
+  char *cmd1= replace_escaped(mysql, query, "@new_user", new_name);
+  char *cmd2= replace_escaped(mysql, cmd1, "@cur_user", cur_name);
+  g_free(cmd1);
+  
+  return cmd2;
+}
+
+
+static int replace_username_execute(MYSQL *mysql, const char *query,
+                                      const char *new_name, const char *cur_name)
+{
+  char *cmd= replace_username(mysql, query, new_name, cur_name);
+  int rc;
+  
+  rc= myx_mysql_query(mysql, cmd);
+  g_free(cmd);
+  
+  return rc;
+}
+
+
 const char * user_info_update_parts[6]=
 {
   "UPDATE mysql." USER_INFO_TABLENAME " SET User=@new_user, Full_name='",
@@ -887,56 +952,61 @@ const char ** user_info_replace_parts[2]=
 };
 
 int replace_user_info(MYSQL *mysql, MYX_USER *user, unsigned long name_len,
-                      const char * replace_query_parts[6])
+                      const char * replace_query_parts[6],
+                      const char *new_user, const char *cur_user)
 {
   int res;
   unsigned long full_name_len= (unsigned long)strlen(user->full_name);
   unsigned long descr_len= (unsigned long)strlen(user->description);
   unsigned long email_len= (unsigned long)strlen(user->email);
   unsigned long contact_len= (unsigned long)strlen(user->contact_information);
-
+  char *tmp;
   char* sqlcmd= g_malloc0(name_len+full_name_len+descr_len+
                           email_len+contact_len+user->icon_length*2+200);
 
-  char * sqlcmd_end= strmov(sqlcmd, replace_query_parts[0]);
+  char * sqlcmd_end;
+  
+  tmp= replace_username(mysql, replace_query_parts[0], new_user, cur_user);
+  sqlcmd_end= strmov(sqlcmd, tmp);
+  g_free(tmp);
   sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                         user->full_name, full_name_len);
-  sqlcmd_end= strmov(sqlcmd_end,replace_query_parts[1]);
+
+  tmp= replace_username(mysql, replace_query_parts[1], new_user, cur_user);
+  sqlcmd_end= strmov(sqlcmd_end, tmp);
+  g_free(tmp);
   sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                         user->description, descr_len);
-  sqlcmd_end= strmov(sqlcmd_end,replace_query_parts[2]);
+
+  tmp= replace_username(mysql, replace_query_parts[2], new_user, cur_user);
+  sqlcmd_end= strmov(sqlcmd_end, tmp);
+  g_free(tmp);
   sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                         user->email, email_len);
-  sqlcmd_end= strmov(sqlcmd_end,replace_query_parts[3]);
+
+  tmp= replace_username(mysql, replace_query_parts[3], new_user, cur_user);
+  sqlcmd_end= strmov(sqlcmd_end, tmp);
+  g_free(tmp);
   sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                         user->contact_information,
                                         contact_len);
-  sqlcmd_end= strmov(sqlcmd_end,replace_query_parts[4]);
+  tmp= replace_username(mysql, replace_query_parts[4], new_user, cur_user);
+  sqlcmd_end= strmov(sqlcmd_end, tmp);
+  g_free(tmp);
   sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                         user->icon, user->icon_length);
-  sqlcmd_end= strmov(sqlcmd_end,replace_query_parts[5]);
+  tmp= replace_username(mysql, replace_query_parts[5], new_user, cur_user);
+  sqlcmd_end= strmov(sqlcmd_end, tmp);
+  g_free(tmp);
 
   res= myx_mysql_real_query(mysql, sqlcmd,(unsigned int)(sqlcmd_end-sqlcmd));
   g_free(sqlcmd);
   return res;
 }
 
-int myx_set_local_variable(MYSQL *mysql, const char * name, 
-                           const char * value, unsigned long value_len)
-{
-  unsigned char sqlcmd[500];
-  unsigned char * sqlcmd_end;
-  sqlcmd_end= strmov(sqlcmd, "set @");
-  sqlcmd_end= strmov(sqlcmd_end, name);
-  sqlcmd_end= strmov(sqlcmd_end, "=\'");
-  sqlcmd_end+= mysql_real_escape_string(mysql,sqlcmd_end,
-                                        value,value_len);
-  sqlcmd_end= strmov(sqlcmd_end, "\'");
-  return myx_mysql_query(mysql, sqlcmd);
-}
 
 int myx_set_user(MYSQL *mysql, MYX_USER *user,
-                 const char *previous_user_name, int new_user)
+                 const char *previous_user_name, int is_new_user)
 {
   MYX_USER *current_user;
   unsigned char sqlcmd[500];
@@ -951,8 +1021,9 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
   unsigned char *long_sqlcmd, *sqlcmd_end;
   MYSQL_RES* res;
   unsigned long cur_name_len, name_len= (unsigned long)strlen(user->user_name);
-  int cur_user_has_been_set= 0;
   int utf8_bin= 0;
+  const char *new_user= user->user_name;
+  const char *cur_user= previous_user_name;
   
   if (mysql_full_version_is_later_or_equal_than(mysql,4,1,11)
       && (!mysql_version_is_later_or_equal_than(mysql,5,0)
@@ -965,34 +1036,37 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       goto error;
   }
 
-  if (myx_set_local_variable(mysql,"new_user",user->user_name,name_len))
-    goto error;
-
-  if (!new_user) // Update existing User
+  if (!is_new_user) // Update existing User
   {
     if (!(current_user= myx_get_user(mysql, previous_user_name)))
       goto error;
-    cur_name_len= (unsigned long)strlen(current_user->user_name);
+    cur_user= current_user->user_name;
+    cur_name_len= (unsigned long)strlen(cur_user);
 
     // check if name has changed
     if (cur_name_len!=name_len ||
         strncmp(user->user_name, current_user->user_name,cur_name_len))
     {
       //Update user name in mysql.user table
-      if (myx_set_local_variable(mysql, "cur_user", 
-                                 current_user->user_name, cur_name_len) ||
-          myx_mysql_query(mysql,"UPDATE mysql.user SET User=@new_user"
-                                " WHERE User=@cur_user") ||
-          myx_mysql_query(mysql,"UPDATE mysql.db SET User=@new_user"
-                                " WHERE User=@cur_user") ||
-          myx_mysql_query(mysql,"UPDATE mysql.tables_priv SET User=@new_user"
-                                " WHERE User=@cur_user") ||
-          myx_mysql_query(mysql,"UPDATE mysql.columns_priv SET User=@new_user"
-                                " WHERE User=@cur_user"))
+      if (replace_username_execute(mysql,
+                          "UPDATE mysql.user SET User=@new_user"
+                          " WHERE User=@cur_user",
+                          new_user, cur_user) ||
+          replace_username_execute(mysql,
+                          "UPDATE mysql.db SET User=@new_user"
+                          " WHERE User=@cur_user",
+                          new_user, cur_user) ||
+          replace_username_execute(mysql,
+                          "UPDATE mysql.tables_priv SET User=@new_user"
+                          " WHERE User=@cur_user",
+                          new_user, cur_user) ||
+          replace_username_execute(mysql,
+                          "UPDATE mysql.columns_priv SET User=@new_user"
+                          " WHERE User=@cur_user",
+                          new_user, cur_user))
       {
         goto error;
       }
-      cur_user_has_been_set= 1;
     }
 
     //Check if all user/host combination has already been created
@@ -1006,20 +1080,24 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       // If not, insert a new user/host combination,
       if (num_host==-1) // take password from existing Host
       {
-        unsigned long host_len= (unsigned long)strlen(user->hosts[i]);
-        sqlcmd_end= strmov(sqlcmd,
-                           "INSERT INTO mysql.user(User, Host, Password)"
-                           " SELECT @new_user,'");
-        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, 
-                                              user->hosts[i],host_len);
-        sqlcmd_end= strmov(sqlcmd_end,
-                           "', Password FROM mysql.user"
-                           " WHERE User=@new_user ORDER BY Host LIMIT 1");
-        if (myx_mysql_real_query(mysql, sqlcmd,
-                                 (unsigned int)(sqlcmd_end-sqlcmd)))
+        char *cmd, *cmd2;
+        
+        cmd= replace_escaped(mysql, 
+                             "INSERT INTO mysql.user(User, Host, Password)"
+                             " SELECT @new_user, @host_name, Password"
+                             " FROM mysql.user"
+                             " WHERE User=@new_user ORDER BY Host LIMIT 1",
+                             "@new_user", new_user);
+        
+        cmd2= replace_escaped(mysql, cmd, "@host_name", user->hosts[i]);
+        g_free(cmd);
+
+        if (myx_mysql_query(mysql, cmd2))
         {
+          g_free(cmd2);
           goto error;
         }
+        g_free(cmd2);
       }
     }
 
@@ -1034,19 +1112,22 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       //If there is a obsolete row, delete it
       if (num_host==-1)
       {
-        unsigned long host_len= (unsigned long)strlen(current_user->hosts[i]);
-        sqlcmd_end= strmov(sqlcmd,
-                           "DELETE FROM mysql.user"
-                           " WHERE User=@new_user AND Host='");
-        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
-                                              current_user->hosts[i],host_len);
-        sqlcmd_end= strmov(sqlcmd_end, "'");
+        char *cmd, *cmd2;
+        
+        cmd= replace_escaped(mysql, 
+                             "DELETE FROM mysql.user"
+                             " WHERE User=@new_user AND Host=@host_name",
+                             "@new_user", user->user_name);
+        cmd2= replace_escaped(mysql, cmd,
+                              "@host_name", current_user->hosts[i]);
+        g_free(cmd);
 
-        if (myx_mysql_real_query(mysql, sqlcmd,
-                                 (unsigned int)(sqlcmd_end-sqlcmd)))
+        if (myx_mysql_query(mysql, cmd2))
         {
+          g_free(cmd2);
           goto error;
         }
+        g_free(cmd2);
       }
     }
 
@@ -1063,26 +1144,39 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         strcmp(user->contact_information, current_user->contact_information) ||
         icon_changed)
     {
+      char *cmd;
       //Check if there already is an entry in the mysql.user_info table
-      if ((cur_user_has_been_set || 
-           !myx_set_local_variable(mysql, "cur_user", 
-                                   current_user->user_name, cur_name_len)) &&
-          !myx_mysql_query(mysql,"SELECT User FROM mysql." USER_INFO_TABLENAME
-                                " WHERE User=@cur_user") &&
-          (res= mysql_store_result(mysql)))
+
+//XXX      if ((cur_user_has_been_set || 
+//XXX           !myx_set_local_variable(mysql, "cur_user", 
+//XXX                                   current_user->user_name, cur_name_len)) &&
+//XXX          !myx_mysql_query(mysql,"SELECT User FROM mysql." USER_INFO_TABLENAME
+//XXX                                " WHERE User=@cur_user") &&
+//XXX          (res= mysql_store_result(mysql)))
+
+      cmd= replace_escaped(mysql, 
+                           "SELECT User FROM mysql." USER_INFO_TABLENAME
+                           " WHERE User=@cur_user",
+                           "@cur_user", cur_user);
+
+      if (myx_mysql_query(mysql, cmd)
+          && (res= mysql_store_result(mysql)))
       {
+        g_free(cmd);
         u_variant= mysql_num_rows(res) ? 0 : 1;
         mysql_free_result(res);
       }
       else
       {
+        g_free(cmd);
         g_message("Error executing query %s:\n%s",sqlcmd,
                   myx_mysql_error(mysql));
         goto error;
       }
 
       if (replace_user_info(mysql,user,name_len,
-                            user_info_replace_parts[u_variant]))
+                            user_info_replace_parts[u_variant],
+                            new_user, cur_user))
       {
         goto error;
       }
@@ -1090,18 +1184,21 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
 
     if (strcmp(user->password, "________"))
     {
-      unsigned long pas_len= (unsigned long)strlen(user->password);
-      //Update password in mysql.user table
-      sqlcmd_end= strmov(sqlcmd, "UPDATE mysql.user SET Password=Password('");
-      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
-                                            user->password,pas_len);
-      sqlcmd_end= strmov(sqlcmd_end, "') WHERE User=@new_user");
-
-      if (myx_mysql_real_query(mysql, sqlcmd,
-                               (unsigned int)(sqlcmd_end-sqlcmd))) 
+      char *cmd, *cmd2;
+      
+      cmd= replace_escaped(mysql, 
+                           "UPDATE mysql.user SET Password=Password(@password)"
+                           " WHERE User=@new_user",
+                           "@new_user", new_user);
+      cmd2= replace_escaped(mysql, cmd, "@password", user->password);
+      g_free(cmd);
+      
+      if (myx_mysql_query(mysql, cmd2))
       {
+        g_free(cmd2);
         goto error;
       }
+      g_free(cmd2);
     }
   }
   else //Create new user
@@ -1111,8 +1208,13 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
     for(i= 0; i<user->hosts_num; i++)
     {
       unsigned long host_len= (unsigned long)strlen(user->hosts[i]);
-      sqlcmd_end= strmov(sqlcmd, "INSERT INTO mysql.user(User, Host, Password)"
-                                 " VALUES(@new_user, '");
+      
+      sqlcmd_end= strmov(sqlcmd, 
+                         "INSERT INTO mysql.user(User, Host, Password)"
+                         " VALUES('");
+      sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
+                                            new_user, (gulong)strlen(new_user));
+      sqlcmd_end= strmov(sqlcmd_end, "', '");
       sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                             user->hosts[i], host_len);
       sqlcmd_end= strmov(sqlcmd_end, "', Password('");
@@ -1132,7 +1234,8 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         user->contact_information[0] || user->icon_length>0)
     {
       if (replace_user_info(mysql,user,name_len,
-                            user_info_insert_parts))
+                            user_info_insert_parts,
+                            new_user, cur_user))
       {
         goto error;
       }
@@ -1143,20 +1246,38 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
   // now update/insert privileges
   if (user->user_object_privileges_num>0)
   {
+    char *cmd;
+    
+    cmd= replace_escaped(mysql, "DELETE FROM mysql.db WHERE User=@new_user",
+                         "@new_user", new_user);
     //Delete previous settings from mysql.db table
-    if (myx_mysql_query(mysql, "DELETE FROM mysql.db WHERE User=@new_user"))
+    if (myx_mysql_query(mysql, cmd))
     {
-      char *error= myx_mysql_error(mysql);
-      int error_num= myx_mysql_errno(mysql);
+      g_free(cmd);
+//      char *error= myx_mysql_error(mysql);
+//      int error_num= myx_mysql_errno(mysql);
 
       goto error;
     }
+    g_free(cmd);
 
-    if (myx_mysql_query(mysql, "DELETE FROM mysql.tables_priv WHERE User=@new_user"))
+    cmd= replace_escaped(mysql, "DELETE FROM mysql.tables_priv WHERE User=@new_user",
+                         "@new_user", new_user);
+    if (myx_mysql_query(mysql, cmd))
+    {
+      g_free(cmd);
       goto error;
+    }
+    g_free(cmd);
 
-    if (myx_mysql_query(mysql, "DELETE FROM mysql.columns_priv WHERE User=@new_user"))
+    cmd= replace_escaped(mysql, "DELETE FROM mysql.columns_priv WHERE User=@new_user",
+                         "@new_user", new_user); 
+    if (myx_mysql_query(mysql, cmd))
+    {
+      g_free(cmd);
       goto error;
+    }
+    g_free(cmd);
 
     long_sqlcmd= g_malloc0(sizeof(char)*700);
 
@@ -1181,7 +1302,10 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
           sqlcmd_end= strmov(sqlcmd_end,
                              j<privs->user_privileges_num-1 ? "', " : "' ");
         }
-        sqlcmd_end= strmov(sqlcmd_end, "WHERE User=@new_user AND Host='");
+        sqlcmd_end= strmov(sqlcmd_end, "WHERE User='");
+        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, 
+                                              new_user, (gulong)strlen(new_user));
+        sqlcmd_end= strmov(sqlcmd_end, "' AND Host='");
         sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, 
                                               privs->host,host_len);
         sqlcmd_end= strmov(sqlcmd_end, "'");
@@ -1206,7 +1330,10 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
           if (j<privs->user_privileges_num-1)
             sqlcmd_end= strmov(sqlcmd_end, ", ");
         }
-        sqlcmd_end= strmov(sqlcmd_end, ") VALUES(@new_user,'");
+        sqlcmd_end= strmov(sqlcmd_end, ") VALUES('");
+        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
+                                              new_user, (gulong)strlen(new_user));
+        sqlcmd_end= strmov(sqlcmd_end, "', '");
         sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end,
                                               privs->host, host_len);
         sqlcmd_end= strmov(sqlcmd_end, "', '");
@@ -1233,7 +1360,7 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
       else // Schema Table privileges
       {
         unsigned long schema_len, table_len, column_len;
-        s= g_malloc0(sizeof(char)*(64+64+64+2));
+        s= g_malloc0(sizeof(char)*(64+64+64+2+(gulong)strlen(privs->object_name)));
         s= strcpy(s, privs->object_name);
         schema_name= strtok(s, ".");
         schema_len= (unsigned long)strlen(schema_name);
@@ -1262,7 +1389,10 @@ int myx_set_user(MYSQL *mysql, MYX_USER *user,
         sqlcmd_end= strmov(sqlcmd_end, "', '");
         sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, schema_name,
                                               schema_len);
-        sqlcmd_end= strmov(sqlcmd_end, "', @new_user, '");
+        sqlcmd_end= strmov(sqlcmd_end, "', '");
+        sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, new_user,
+                                              (gulong)strlen(new_user));
+        sqlcmd_end= strmov(sqlcmd_end, "', '");
         sqlcmd_end+= mysql_real_escape_string(mysql, sqlcmd_end, table_name,
                                               table_len);
         sqlcmd_end= strmov(sqlcmd_end, "', now(), '");
