@@ -214,6 +214,7 @@ bool MABackupPanel::init()
   _data->signal_catalogs_refreshed().connect(SigC::slot(*this,&MABackupPanel::schemata_reloaded));
 
   get_button("target_button")->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::select_target_path));
+  get_toggle("backup_all_check")->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::backup_all_check_toggled));
 
   // setup signals
   Gtk::Button *btn;
@@ -306,7 +307,34 @@ void MABackupPanel::update_sensitivity(bool flag)
   get_widget("notebook")->set_sensitive(flag);
   get_widget("save_button")->set_sensitive(flag);
   
-  update_button_bar_sensitivity();
+  update_buttonbar_sensitivity();
+}
+
+void MABackupPanel::update_backup_all_state()
+{
+  bool check_backup_all= true;
+
+  if (get_widget("notebook")->is_sensitive())
+  {
+    // check whether there's some selected table
+    Gtk::TreeModel::Children ch= _selection_store->children();
+    for (Gtk::TreeIter p= ch.begin(); p != ch.end(); ++p)
+    {
+      Gtk::TreeModel::Row pr= *p;
+      Gtk::TreeModel::Children cch= pr.children();
+      for (Gtk::TreeIter i= cch.begin(); i != cch.end(); ++i)
+      {
+        Gtk::TreeModel::Row cr= *i;
+        if (cr[_sel_columns._state] != MGCellRendererTristate::ON)
+        {
+          check_backup_all= false;
+          break;
+        }
+      }
+    }
+  }
+
+  get_toggle("backup_all_check")->set_active(check_backup_all);
 }
 
 void MABackupPanel::update_button_sensitivity()
@@ -361,6 +389,24 @@ void MABackupPanel::update_button_sensitivity()
   }
 }
 
+void MABackupPanel::backup_all_check_toggled()
+{
+  bool b= get_toggle("backup_all_check")->get_active();
+  if(b)
+  {
+    Gtk::TreeModel::Children ch= _selection_store->children();
+    for (Gtk::TreeIter p= ch.begin(); p != ch.end(); ++p)
+    {
+      Gtk::TreeModel::Row pr= *p;
+      Gtk::TreeModel::Children cch= pr.children();
+      for (Gtk::TreeIter i= cch.begin(); i != cch.end(); ++i)
+      {
+        Gtk::TreeModel::Row cr= *i;
+        cr[_sel_columns._state] = MGCellRendererTristate::ON;
+      }
+    }
+  }
+}
 
 void MABackupPanel::select_target_path()
 {
@@ -377,32 +423,42 @@ void MABackupPanel::select_target_path()
 }
 
 
-void MABackupPanel::update_button_bar_sensitivity()
+void MABackupPanel::update_buttonbar_sensitivity()
 {
-  bool ok= false;
+  bool enable_start= false;
 
   if (get_widget("notebook")->is_sensitive())
   {
     // check whether there's some selected table
     Gtk::TreeModel::Children ch= _selection_store->children();
-    for (Gtk::TreeIter p= ch.begin(); !ok && p != ch.end(); ++p)
+    for (Gtk::TreeIter p= ch.begin(); p != ch.end(); ++p)
     {
       Gtk::TreeModel::Row pr= *p;
       Gtk::TreeModel::Children cch= pr.children();
-      for (Gtk::TreeIter i= cch.begin(); i != cch.end(); ++i)
+      if(cch.size() > 0)
       {
-        Gtk::TreeModel::Row cr= *i;
-        if (cr[_sel_columns._state] == MGCellRendererTristate::ON)
+        for (Gtk::TreeIter i= cch.begin(); i != cch.end(); ++i)
         {
-          ok= true;
+          Gtk::TreeModel::Row cr= *i;
+          if (cr[_sel_columns._state] == MGCellRendererTristate::ON)
+          {
+            enable_start= true;
+            break;
+          }
+        }
+      }
+      else
+      {
+        if (pr[_sel_columns._state] == MGCellRendererTristate::ON)
+        {
+          enable_start= true;
           break;
         }
-        break;
       }
     }
   }
 
-  get_widget("start_button")->set_sensitive(ok);
+  get_widget("start_button")->set_sensitive(enable_start);
 }
 
 
@@ -412,7 +468,7 @@ void MABackupPanel::add_schema_to_list(const Glib::ustring &catalog,
 { 
   Gtk::TreeIter s_iter, t_iter;
   Gtk::TreeModel::Row s_row, t_row;
-  unsigned long total_rows= 0, total_length= 0;
+  unsigned long long total_rows= 0, total_length= 0;
 
   // add schema
   s_iter= _selection_store->append();
@@ -430,22 +486,23 @@ void MABackupPanel::add_schema_to_list(const Glib::ustring &catalog,
   for (unsigned int i= 0; tables && i < tables->schema_tables_num; ++i)
   {
     MYX_TABLE_STATUS *table= &tables->schema_tables[i];
+
     t_iter= _selection_store->append(s_row.children());
     t_row= *t_iter;
 
     t_row[_sel_columns._icon]= _table_icon;
     t_row[_sel_columns._object]= (char*)table->table_name;
     t_row[_sel_columns._type]= (table->table_type)? ((char*)table->table_type):"";
-    t_row[_sel_columns._rows]= atol((char*)table->rows);
-    t_row[_sel_columns._length]= format_value(atol((char*)table->data_length));
+    t_row[_sel_columns._rows]= atoll((char*)table->rows);
+    t_row[_sel_columns._length]= format_value(atoll((char*)table->data_length));
     t_row[_sel_columns._time]= (char*)table->update_time;
 
-    t_row[_sel_columns._lengthn]= atol((char*)table->data_length);
+    t_row[_sel_columns._lengthn]= atoll((char*)table->data_length);
     t_row[_sel_columns._state]= checked?MGCellRendererTristate::ON:MGCellRendererTristate::OFF;
     t_row[_sel_columns._what]= SelectionColumns::CTable;
     
-    total_rows+= atol((char*)table->rows);
-    total_length+= atol((char*)table->data_length);
+    total_rows+= atoll((char*)table->rows);
+    total_length+= atoll((char*)table->data_length);
   }
 
   s_row[_sel_columns._rows]= total_rows;
@@ -471,7 +528,7 @@ void MABackupPanel::add_object()
     add_schema_to_list(catalog, schema, true);
   }
 
-  update_button_bar_sensitivity();
+  //update_buttonbar_sensitivity(); //set dirty does that
   set_dirty();
 }
 
@@ -482,7 +539,7 @@ void MABackupPanel::remove_object()
   
   _selection_store->erase(iter);
   
-  update_button_bar_sensitivity();
+  //update_buttonbar_sensitivity(); //set dirty does that
   set_dirty();
 }
 
@@ -672,12 +729,12 @@ void MABackupPanel::show_profile(MYX_BACKUP_PROFILE *profile)
         seen[key]++;
       }
     }
-
+    
     Gtk::TreeModel::Children top_nodes= _selection_store->children();
-
     for (unsigned int i= 0; i < profile->backup_content->tables_num; i++)
     {
       MYX_BACKUP_TABLE *table= profile->backup_content->tables+i;
+      
       Gtk::TreeIter piter, citer;
       Gtk::TreeModel::Row prow, crow;
       Glib::ustring catalog(table->catalog?:"def");
@@ -691,33 +748,40 @@ void MABackupPanel::show_profile(MYX_BACKUP_PROFILE *profile)
         
         if (prow[_sel_columns._object] == schema)
         {
-          Gtk::TreeModel::Children sub_nodes= prow.children();
-          Glib::ustring tname(table->table);
-          // find table node
-          for (citer= sub_nodes.begin(); citer != sub_nodes.end(); ++citer)
-          {
-            crow= *citer;
-            
-            if (crow[_sel_columns._object] == tname)
-            {
-              crow[_sel_columns._state]= MGCellRendererTristate::ON;
-              break;
-            }
-          }
-          // if all nodes are checked, check parent too
-          if (seen[key]==sub_nodes.size())
+          if(!table->table)
           {
             prow[_sel_columns._state]= MGCellRendererTristate::ON;
           }
-          else if (seen[key] > 0) // at least one checked
-          {
-            prow[_sel_columns._state]= MGCellRendererTristate::MAYBE;
-          }
           else
           {
-            prow[_sel_columns._state]= MGCellRendererTristate::OFF;
+            Gtk::TreeModel::Children sub_nodes= prow.children();
+            Glib::ustring tname(table->table);
+            // find table node
+            for (citer= sub_nodes.begin(); citer != sub_nodes.end(); ++citer)
+            {
+              crow= *citer;
+              
+              if (crow[_sel_columns._object] == tname)
+              {
+                crow[_sel_columns._state]= MGCellRendererTristate::ON;
+                break;
+              }
+            }
+            // if all nodes are checked, check parent too
+            if (seen[key]==sub_nodes.size())
+            {
+              prow[_sel_columns._state]= MGCellRendererTristate::ON;
+            }
+            else if (seen[key] > 0) // at least one checked
+            {
+              prow[_sel_columns._state]= MGCellRendererTristate::MAYBE;
+            }
+            else
+            {
+              prow[_sel_columns._state]= MGCellRendererTristate::OFF;
+            }
+            break;
           }
-          break;
         }
       }
     }
@@ -728,47 +792,64 @@ void MABackupPanel::show_profile(MYX_BACKUP_PROFILE *profile)
   update_sensitivity(true);
 }
 
-
-
 MYX_BACKUP_CONTENT *MABackupPanel::gather_selected_tables()
 {
   Gtk::TreeIter siter, titer;
-  int alloced= 8, count= 0;
+  int i;
   MYX_BACKUP_CONTENT *content;
-  MYX_BACKUP_TABLE *tables= (MYX_BACKUP_TABLE*)g_malloc0(alloced*sizeof(MYX_BACKUP_TABLE));
-
+  GPtrArray *ta= g_ptr_array_new();
+  Glib::ustring str;
+  
   for (siter= _selection_store->children().begin();
        siter != _selection_store->children().end();
        ++siter)
   {
     Gtk::TreeModel::Row row= *siter;
 
-    for (titer= row.children().begin(); titer != row.children().end(); ++titer)
+    if(row.children().empty())
     {
-      Gtk::TreeModel::Row rrow= *titer;
-      Glib::ustring str;
-
-      if (rrow[_sel_columns._state] == MGCellRendererTristate::ON)
+      MYX_BACKUP_TABLE *bt= 
+              (MYX_BACKUP_TABLE *)g_malloc(sizeof(MYX_BACKUP_TABLE));
+      
+      str= row[_sel_columns._object];
+      bt->catalog= g_strdup("def");//XXX
+      bt->schema= g_strdup(str.c_str());
+      bt->table= NULL;
+      g_ptr_array_add(ta, bt);
+    }
+    else
+    {
+      for (titer= row.children().begin(); titer != row.children().end(); ++titer)
       {
-        if (count == alloced)
-        {
-          alloced += 10;
-          tables= (MYX_BACKUP_TABLE*)g_realloc(tables, alloced*sizeof(MYX_BACKUP_TABLE));
-        }
+        Gtk::TreeModel::Row rrow= *titer;
 
-        tables[count].catalog= g_strdup("def");//XXX
-        str= row[_sel_columns._object];
-        tables[count].schema= g_strdup(str.c_str());
-        str= rrow[_sel_columns._object];
-        tables[count].table= g_strdup(str.c_str());
-        count++;
+        if (rrow[_sel_columns._state] == MGCellRendererTristate::ON)
+        {
+          MYX_BACKUP_TABLE *bt= 
+                  (MYX_BACKUP_TABLE *)g_malloc(sizeof(MYX_BACKUP_TABLE));
+          bt->catalog= g_strdup("def");//XXX
+          str= row[_sel_columns._object];
+          bt->schema= g_strdup(str.c_str());
+          str= rrow[_sel_columns._object];
+          bt->table= g_strdup(str.c_str());;
+          g_ptr_array_add(ta, bt);
+        }
       }
     }
   }
 
   content= (MYX_BACKUP_CONTENT*)g_malloc0(sizeof(MYX_BACKUP_CONTENT));
-  content->tables_num= count;
-  content->tables= tables;
+  content->tables_num= ta->len;
+  content->tables= (MYX_BACKUP_TABLE *)
+                   g_malloc(sizeof(MYX_BACKUP_TABLE)*ta->len);
+
+  for(i= 0; i < ta->len; i++)
+  {
+    MYX_BACKUP_TABLE *bt= (MYX_BACKUP_TABLE *)g_ptr_array_index(ta, i);
+    content->tables[i].catalog= bt->catalog;
+    content->tables[i].schema= bt->schema;
+    content->tables[i].table= bt->table;
+  }
 
   return content;
 }
@@ -810,7 +891,6 @@ MYX_BACKUP_PROFILE *MABackupPanel::gather_profile()
   
   profile->backup_content= gather_selected_tables();
 
-  
   return profile;
 }
 
@@ -983,7 +1063,8 @@ void MABackupPanel::row_toggled(const Glib::ustring &path)
       rpar[_sel_columns._state]= MGCellRendererTristate::MAYBE;
   }
   
-  update_button_bar_sensitivity();
+  //update_buttonbar_sensitivity(); //set dirty does that
+  update_backup_all_state();
   set_dirty();
 }
 
@@ -992,10 +1073,9 @@ void MABackupPanel::set_dirty()
 {
   _dirty= true;
   
-  update_button_bar_sensitivity();
+  update_buttonbar_sensitivity();
 }
 
-
 int MABackupPanel::backup_progress_cb(const char *table,
                                       int num_tables_total,
                                       int num_tables,
@@ -1056,9 +1136,14 @@ bool MABackupPanel::update_backup_progress(void *data)
 
 void *MABackupPanel::backup_thread(void *data)
 {
-  BackupThreadArgument *arg= (BackupThreadArgument*)data;
+  BackupThreadArgument *arg;
   MYX_BACKUP_ERROR err;
-  MInstanceInfo *inst= arg->me->_data->get_instance();
+  MInstanceInfo *inst;
+
+  mysql_thread_init();
+
+  arg= (BackupThreadArgument*)data;
+  inst= arg->me->_data->get_instance();
 
   inst->lock_mysql(true);
 
@@ -1067,6 +1152,8 @@ void *MABackupPanel::backup_thread(void *data)
                                     1, backup_progress_cb, &arg->pdata);
   inst->lock_mysql(false);
 
+  mysql_thread_end();
+  
   return (void*)err;
 }
 
@@ -1091,7 +1178,7 @@ void MABackupPanel::perform_backup()
   _row_progress->set_fraction(0);
 
   if (!prefs.disable_backup_timestamps)
-    strftime(suffix, sizeof(suffix), "_%F_%T.sql", localtime(&now));
+    strftime(suffix, sizeof(suffix), " %Y%m%d %H%M.sql", localtime(&now));
   else
     suffix[0]= 0;
 
@@ -1119,7 +1206,7 @@ void MABackupPanel::perform_backup()
                                                          50);
     dlg->show();
 
-    err= (MYX_BACKUP_ERROR)(int)disp.dispatch(backup_thread, &arg);
+    err= (MYX_BACKUP_ERROR)(long)disp.dispatch(backup_thread, &arg);
 
     dlg->hide();
 
@@ -1132,7 +1219,7 @@ void MABackupPanel::perform_backup()
   
   _app->pop_status();
 
-  Glib::ustring msg= get_backup_error_text(err);
+  Glib::ustring msg= get_backup_error_text(_data->get_instance()->get_mysql(), err);
 
   if (msg.empty())
   {
@@ -1210,21 +1297,40 @@ bool MABackupPanel::gather_schedule(MCrontab::Entry &schedule)
     myg_show_error(*_app->window(), _("Please select a profile first."));
     return false;
   }
-
+#if 0
   schedule.command= get_argv0();
   if (Glib::str_has_suffix(schedule.command, "mysql-administrator-bin"))
     schedule.command= schedule.command.substr(0, schedule.command.size()-4);
-  schedule.command+= " -bp"+profile;
-  
+#else
+  schedule.command= get_argv0();
+  if (Glib::str_has_suffix(schedule.command, "mysql-administrator"))
+  {
+    schedule.command= schedule.command.substr(0, 
+      schedule.command.size()-strlen("mysql-administrator"));
+  } 
+  else if(Glib::str_has_suffix(schedule.command, "mysql-administrator-bin"))
+  {
+    schedule.command= schedule.command.substr(0, 
+      schedule.command.size()-strlen("mysql-administrator-bin"));
+  }
+  else
+  {
+    schedule.command= "/usr/local/";
+  }
+  schedule.command+= "mabackup";
+#endif
+
   if (!target.empty())
-    schedule.command+= " -bt"+target;
+    schedule.command+= " -d "+target;
   if (!prefix.empty())
-    schedule.command+= " -bx"+prefix;
-  
+    schedule.command+= " -x "+prefix;
+
   Gtk::Combo *combo= static_cast<Gtk::Combo*>(get_widget("connection_combo"));
   if (!combo->get_entry()->get_text().empty())
-    schedule.command+= " -c"+combo->get_entry()->get_text();
+    schedule.command+= " -c "+combo->get_entry()->get_text();
 
+  schedule.command+= " "+profile;
+  
   return true;
 }
 
@@ -1488,14 +1594,15 @@ void MABackupPanel::profile_selected(MGBrowserList *sender,
 }
 
 
-Glib::ustring MABackupPanel::get_backup_error_text(MYX_BACKUP_ERROR err)
+Glib::ustring MABackupPanel::get_backup_error_text(MYSQL *mysql, MYX_BACKUP_ERROR err)
 { 
   Glib::ustring msg;
   
   switch (err)
   {
   case MYX_BACKUP_MYSQL_ERROR:
-    msg= _("MySQL error during backup.");
+    //msg= _("MySQL error during backup.");
+    msg= myx_mysql_error(mysql);
     break;
   case MYX_BACKUP_CANT_OPEN_FILE:
     msg= _("Could not open file for writing backup.");
@@ -1592,10 +1699,12 @@ bool MABackupPanel::execute_backup(MYSQL *mysql,
   if (bkerr != MYX_BACKUP_NO_ERROR)
   {
     char *tmp= NULL;
-    myg_log("ERROR: error while performing backup: %s%s",
-            get_backup_error_text(bkerr).c_str(),
-            bkerr != MYX_BACKUP_MYSQL_ERROR 
-            ? "" : ufmt(" %s", tmp=myx_mysql_error(mysql)).c_str());
+    //myg_log("ERROR: error while performing backup: %s%s",
+    //        get_backup_error_text(mysql, bkerr).c_str(),
+    //        bkerr != MYX_BACKUP_MYSQL_ERROR 
+    //        ? "" : ufmt(" %s", tmp=myx_mysql_error(mysql)).c_str());
+    myg_log("ERROR: error while performing backup: %s",
+            get_backup_error_text(mysql, bkerr).c_str());
     g_free(tmp);
   }
   else
