@@ -405,6 +405,10 @@ bool MACatalogsPanel::init()
 
     _table_menu.append(*(item=myg_make_stock_image_item(Gtk::Stock::PROPERTIES, _("_Edit Table"))));
     item->signal_activate().connect(SigC::slot(*this,&MACatalogsPanel::edit_table));
+
+    _table_menu.append(*(item=myg_make_stock_image_item(Gtk::Stock::PROPERTIES, _("Edit _Table Data"))));
+    item->signal_activate().connect(SigC::slot(*this,&MACatalogsPanel::edit_table_data));
+
     _table_menu.append(*(item=Gtk::manage(new Gtk::MenuItem(_("Show Create Script")))));
     item->signal_activate().connect(SigC::slot(*this,&MACatalogsPanel::show_create_table));
     _table_menu.append(*myg_make_separator_item());
@@ -1080,6 +1084,32 @@ void MACatalogsPanel::edit_table()
   }
 }
 
+void MACatalogsPanel::edit_table_data()
+{
+  Gtk::TreeView *tree= get_tree("table_tree");
+  if (tree->get_selection()->count_selected_rows() != 1)
+    return;
+  
+  Gtk::TreeIter iter= _table_store->get_iter(*tree->get_selection()->get_selected_rows().begin());
+
+  if (iter)
+  {    
+    Gtk::TreeRow row = *iter;
+    Glib::ustring catalog, schema;
+    Glib::ustring table = row[_table_columns._name];
+    Gtk::TreeIter it = _schema_browser->get_selected();
+
+    catalog = _schema_browser->get_catalog(it);
+    schema = _schema_browser->get_schema(it);
+
+    Glib::spawn_command_line_async(ufmt("mysql-query-browser-bin --mqb-dir=%s --query=\"SELECT * FROM \\`%s\\`.\\`%s\\`\"",
+																		getenv("MA_DIR"), schema.c_str(), table.c_str()));
+
+		//printf("table = %s\n", table.c_str());
+		//printf("catalog = %s\n", catalog.c_str());
+		//printf("schema = %s\n", schema.c_str());
+  }
+}
 
 static void *execute_query(MYSQL *mysql, const char *query, MYX_LIB_ERROR *error)
 {
@@ -1129,13 +1159,20 @@ void MACatalogsPanel::drop_table()
 }
 
 
+void MACatalogsPanel::table_editor_closed()
+{
+  table_edit_mode(false);
+  reload_table_list();
+}
+
+
 void MACatalogsPanel::table_edit_mode(bool flag)
 {
   if (!_editor)
   {
     _editor= new MGTableEditor(true);
 //    _editor->get_widget()->hide();
-    _editor->signal_close().connect(SigC::bind<bool>(SigC::slot(*this,&MACatalogsPanel::table_edit_mode), false));
+    _editor->signal_close().connect(SigC::slot(*this,&MACatalogsPanel::table_editor_closed));
 //    ((Gtk::Box*)_xml->get_widget("panel_frame"))->pack_start(*_editor->get_widget());
   }
 
