@@ -1607,7 +1607,7 @@ void MAHealthPanel::variable_edited(const Glib::ustring& path,
 
   variable= row[_value_columns._name];
   
-  if ((int)_instance->perform_data_fetch3((MInstanceInfo::DataFetcher3)myx_set_variable,
+  if ((long)_instance->perform_data_fetch3((MInstanceInfo::DataFetcher3)myx_set_variable,
                                           (void*)variable.c_str(), 
                                           (void*)new_text.c_str())!=0)
   {
