@@ -14,9 +14,9 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
-   
 #include "myadmin.h"
 #include "MAdministrator.h"
+#include "MABackupPanel.h"
 #include "MARestorePanel.h"
 #include "MDataInterface.h"
 #include "MInstanceInfo.h"
@@ -205,7 +205,8 @@ void MARestorePanel::show_content_info(const std::string &file,
 
     trow[_sel_columns._icon]= _table_icon;
     trow[_sel_columns._state]= MGCellRendererTristate::ON;
-    trow[_sel_columns._object]= (char*)table->table;
+	trow[_sel_columns._object]= (char*)table->table;
+	
 //    trow[_sel_columns._type]= "";//XXX
 //    trow[_sel_columns._rows]= 0;
     trow[_sel_columns._what]= SelectionColumns::CTable;
@@ -466,7 +467,13 @@ bool MARestorePanel::update_status(void *data)
 
   if (ldata->ready)
   {
-    _progress->set_fraction((double)ldata->bytes / ldata->bytes_total);
+		double d = (double)ldata->bytes/(double)ldata->bytes_total;
+		if(d < 0.01)	// workaround for a gtkmm (2.2.12) bug (?)
+									// if 0.004 <= d < 0.007 it displays progress fraction == 100%
+		{
+			d = 0.01;
+		}
+		_progress->set_fraction(d);
     _progress->set_text(ufmt(_("%'lli of %'lli bytes"), ldata->bytes, ldata->bytes_total));
     ldata->ready= false;
   }
@@ -487,9 +494,13 @@ void MARestorePanel::close_progress()
 
 void *MARestorePanel::load_thread(void *data)
 {
-  LoadThreadArgument *arg= (LoadThreadArgument*)data;
+  LoadThreadArgument *arg;
   MYX_BACKUP_ERROR err;
   MYX_BACKUP_CONTENT *content;
+  
+  mysql_thread_init();
+
+  arg= (LoadThreadArgument*)data;
 
   content= myx_get_backup_content(arg->path.c_str(), arg->charset.c_str(),
                                 MYX_BT_SQL_SCRIPT,
@@ -497,6 +508,7 @@ void *MARestorePanel::load_thread(void *data)
                                 progress_cb, &arg->pdata, &err);
   if (err != MYX_BACKUP_NO_ERROR)
     g_message("Error reading backup content file: %i", err);
+  mysql_thread_end();
   return content;
 }
 
@@ -675,7 +687,7 @@ void MARestorePanel::perform_restore()
 
   op_aborted= false;
 
-  {
+  //{
     MYX_BACKUP_ERROR err;
 
     SigC::Connection con= Glib::signal_timeout().connect(SigC::bind<void*>
@@ -684,16 +696,26 @@ void MARestorePanel::perform_restore()
 
     dlg->show();
 
-    err= (MYX_BACKUP_ERROR)(int)disp.dispatch(restore_thread, &arg);
-
+    err= (MYX_BACKUP_ERROR)(long)disp.dispatch(restore_thread, &arg);
     con.disconnect();
-  }
+  //}
 
   myx_free_backup_content(arg.content);
   
   _app->pop_status();
 
-  if (op_aborted)
+  if(err)
+  {
+    Glib::ustring msg= MABackupPanel::get_backup_error_text(_data->get_instance()->get_mysql(), err);
+    Gtk::MessageDialog errdlg(*_app->window(), _("Error during backup.\n")+msg,
+                           Gtk::MESSAGE_ERROR,
+                           Gtk::BUTTONS_OK,
+                           true);
+    errdlg.run();
+    _app->set_status(_("Restore failure."));
+    dlg->hide();
+  }
+  else if (op_aborted)
   {
     _app->set_status(_("Cancelled."));
     dlg->hide();
