@@ -155,17 +155,32 @@ bool MAServerConnectionsPanel::init()
   tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MAServerConnectionsPanel::update_sensitivity));
 
   _default_user_icon= PIXCACHE->load("16x16_User.png");
+  
+  if (prefs.connection_list_auto_refresh > 0)
+    _timer= Glib::signal_timeout().connect(SigC::slot(*this,&MAServerConnectionsPanel::refresh_thread_list_b), prefs.connection_list_auto_refresh*1000);
+
+  _app->signal_prefs_changed().connect(SigC::slot(*this,&MAServerConnectionsPanel::prefs_changed));
 
   return true;
 }
 
 
+void MAServerConnectionsPanel::prefs_changed()
+{
+  if (_timer)
+    _timer.disconnect();
+
+  if (prefs.connection_list_auto_refresh > 0)
+    _timer= Glib::signal_timeout().connect(SigC::slot(*this,&MAServerConnectionsPanel::refresh_thread_list_b), prefs.connection_list_auto_refresh*1000);
+}
+
+
 void MAServerConnectionsPanel::update_sensitivity()
 {
   bool user_ok= false, thread_ok= false;
   bool copy_ok= false;
   Gtk::TreeIter iter;
-  
+
   if (_current_page == 0)
   {   
     if ((iter= _thread_tree->get_selection()->get_selected()))
@@ -197,8 +212,8 @@ void MAServerConnectionsPanel::update_sensitivity()
   get_widget("kill_user_button")->set_sensitive(user_ok);
 
   get_widget("kill_thread_button")->set_sensitive(thread_ok);
-  
-  get_widget("copy_button")->set_sensitive(thread_ok);
+
+  get_widget("copy_button")->set_sensitive(copy_ok);
 }
 
 
@@ -232,13 +247,21 @@ MAServerConnectionsPanel::UserInfo &MAServerConnectionsPanel::get_user_info(cons
 }
 
 
+bool MAServerConnectionsPanel::refresh_thread_list_b()
+{
+  refresh_thread_list();
+  return true;
+}
+
 void MAServerConnectionsPanel::refresh_thread_list()
 {
   MYX_PROCESS_LIST *plist;
   Glib::ustring old_sel_user;
   Gtk::TreeIter old_sel_user_iter;
   
-  plist= (MYX_PROCESS_LIST*)_instance->perform_data_fetch((MInstanceInfo::DataFetcher)myx_get_process_list);
+//  plist= (MYX_PROCESS_LIST*)_instance->perform_data_fetch((MInstanceInfo::DataFetcher)myx_get_process_list);
+
+  plist= myx_get_process_list(_inst->get_mysql());
   if (plist)
   {
     Gtk::TreeIter iter;
@@ -267,11 +290,11 @@ void MAServerConnectionsPanel::refresh_thread_list()
       row[_thr_columns._col_user_icon]= !info.icon.is_null() ? info.icon: _default_user_icon;
       row[_thr_columns._col_user]= (char*)plist->process_infos[i].user;
       row[_thr_columns._col_host]= (char*)plist->process_infos[i].host;
-      row[_thr_columns._col_db]= (char*)plist->process_infos[i].db;
-      row[_thr_columns._col_command]= crop_string((char*)plist->process_infos[i].command, MAX_COMMAND_LENGTH);
-      row[_thr_columns._col_time]= (char*)plist->process_infos[i].time;
-      row[_thr_columns._col_state]= (char*)plist->process_infos[i].state;
-      row[_thr_columns._col_info]= crop_string((char*)plist->process_infos[i].info, MAX_COMMAND_LENGTH);
+      row[_thr_columns._col_db]= (char*)plist->process_infos[i].db?:"";
+      row[_thr_columns._col_command]= crop_string((char*)plist->process_infos[i].command?:"", MAX_COMMAND_LENGTH);
+      row[_thr_columns._col_time]= (char*)plist->process_infos[i].time?:"";
+      row[_thr_columns._col_state]= (char*)plist->process_infos[i].state?:"";
+      row[_thr_columns._col_info]= crop_string((char*)plist->process_infos[i].info?:"", MAX_COMMAND_LENGTH);
       row[_thr_columns._col_data]= plist->process_infos+i;
      
       if (users.find((char*)plist->process_infos[i].user)==users.end())
