@@ -750,8 +750,7 @@ bool MAUserAdministrationPanel::populate_user_host(MGUserBrowserList *sender,
   {
     for (unsigned int i= 0; i < user->hosts_num; i++)
     {
-      if (strcmp((char*)user->hosts[i], "%")!=0) // dont add %
-        hosts.push_back("@ "+Glib::ustring((char*)user->hosts[i]));
+      hosts.push_back("@ "+Glib::ustring((char*)user->hosts[i]));
     }
     return true;
   }
@@ -1478,7 +1477,7 @@ void MAUserAdministrationPanel::create_new_user()
     user->email= (unsigned char*)g_strdup("");
     user->contact_information= (unsigned char*)g_strdup("");
 
-    add_host_to_user(user, "%");
+    add_host_to_user(user, "localhost");
     
     _data->add_new_user(user, new_name);
 
@@ -1576,13 +1575,14 @@ bool MAUserAdministrationPanel::validate()
   Glib::ustring pw1, pw2;
   Glib::ustring msg;
 
-  if (_current_user->hosts_num == 0)
+  do
   {
-    msg= _("You need to add at least one host from where the user can connect to.");
-  }
+    if (_current_user->hosts_num == 0)
+    {
+      msg= _("You need to add at least one host from where the user can connect to.");
+      break;
+    }
 
-  if (msg.empty())
-  {
     // check password
     pw1= ((Gtk::Entry*)get_widget("password_entry"))->get_text();
     pw2= ((Gtk::Entry*)get_widget("confirm_entry"))->get_text();
@@ -1593,8 +1593,18 @@ bool MAUserAdministrationPanel::validate()
 
       ((Gtk::Entry*)get_widget("password_entry"))->set_text((char*)_current_user->password);
       ((Gtk::Entry*)get_widget("confirm_entry"))->set_text((char*)_current_user->password);
+      break;
     }
-  }
+
+    // check duplicate name
+    uname= ((Gtk::Entry*)get_widget("user_entry"))->get_text();
+    if(_user_browser->exists_user(uname))
+    {
+      msg= _("Duplicate user name, please enter another name.");
+      break;
+    }
+  } 
+  while(0);
 
   if (!msg.empty())
   {
@@ -1706,10 +1716,11 @@ bool MAUserAdministrationPanel::apply_changes()
 
       _user_names= _data->get_user_names();
       _user_browser->set_user_list(_user_names);
+      _user_is_new= false;
+      return true;
     }
   }
-
-  return true;
+  return false;
 }
 
 
@@ -1747,6 +1758,8 @@ void MAUserAdministrationPanel::discard_changes()
     if (_user_browser)
       _user_browser->refresh();
   }
+  
+  _user_is_new= false;
 }
 
 
@@ -1999,7 +2012,7 @@ void MAUserAdministrationPanel::user_selected(MGBrowserList *sender,
   }
 
   bool reget_node= false;
-  
+
   // if the user requested a save before switching, then save
   // WARNING: this can change the user list
   if (_apply_changes_on_user_switch)
@@ -2071,7 +2084,6 @@ void MAUserAdministrationPanel::user_selected(MGBrowserList *sender,
   }
   else
     unshow_user();
-  _user_is_new= false;
 }
 
 
@@ -2140,6 +2152,10 @@ bool MAUserAdministrationPanel::user_will_change_delegate(MGBrowserList *sender,
                                 //_current_user_name.c_str())))
     {
     case Gtk::RESPONSE_APPLY:
+      if(!validate())
+      {
+        return false;
+      }
       _apply_changes_on_user_switch= true;
       break;
 
