@@ -234,7 +234,7 @@ void MInstanceInfo::set_connection(MYSQL *mysql,
                      "but this application was designed to work with MySQL "
                      "servers 4.x and newer.\n"
                      "Please note that older versions are not supported and "
-                     "may not run well."), 
+                     "may not work properly."), 
                    major_version, myx_get_mysql_minor_version(mysql)),
               Gtk::MESSAGE_WARNING,
               Gtk::BUTTONS_OK,
@@ -242,7 +242,7 @@ void MInstanceInfo::set_connection(MYSQL *mysql,
         dlg.run();
       }
       
-      _mysql_tid= (int)perform_data_fetch((DataFetcher)myx_get_thread_id);
+      _mysql_tid= (long)perform_data_fetch((DataFetcher)myx_get_thread_id);
     }
     myx_free_pc_info(server);
 
