@@ -20,7 +20,6 @@
     IBOutlet NSTextField *monthlyTimeText;
     IBOutlet NSTextField *nameText;
     IBOutlet NSTextField *prefixText;
-//    IBOutlet id profileList;
     IBOutlet MTableView *profileTable;
     IBOutlet NSButton *scheduleButton;
     IBOutlet NSPopUpButton *schedulePopUp;
@@ -33,7 +32,10 @@
     IBOutlet NSPopUpButton *typePopUp;
     IBOutlet NSButton *unscheduleButton;
     IBOutlet NSTextField *weeklyTimeText;
-    
+    IBOutlet NSTextField *scheduleDailyTime;
+	IBOutlet NSMatrix *scheduleWeekdays;
+    IBOutlet NSTextField *scheduleWeeklyTime;
+
     IBOutlet NSMatrix *modeMatrix;
     IBOutlet NSMatrix *optionMatrix;
     
@@ -49,10 +51,14 @@
 
     BOOL _backupAborted;
     MYX_BACKUP_PROFILE *_profile;
+	MYX_USER_CONNECTIONS *_pconns;
 
     MABackupDataSource *_selectionDS;
     NSMutableArray *_profiles;
 }
+
+- (BOOL) exchangeCronEntry: entry;
+
 - (IBAction)addContent:(id)sender;
 - (IBAction)addProfile:(id)sender;
 - (IBAction)chooseTargetDirectory:(id)sender;
@@ -63,4 +69,5 @@
 - (IBAction)startBackup:(id)sender;
 - (IBAction)unscheduleBackup:(id)sender;
 - (IBAction)abortBackup:(id)sender;
+- (IBAction)showConnectionEditor:(id)sender;
 @end
