@@ -3,12 +3,20 @@
 #import <MySQLToolsCommon/NSString_extras.h>
 #include "myx_admin_public_interface.h"
 
+#import <MySQLToolsCommon/NSView_extras.h>
 #import <MySQLToolsCommon/MTextImageCell.h>
 #import <MySQLToolsCommon/MButtonImageCell.h>
 #import <MySQLToolsCommon/MSchemaDatasource.h>
 #import <MySQLToolsCommon/MMySQLDispatcher.h>
 #import <MySQLToolsCommon/MTreeDataSource.h>
+#import <MySQLToolsCommon/MCrontab.h>
+#import <MySQLToolsCommon/MPreferenceEditor.h>
+#import <MySQLToolsCommon/MPreferences.h>
+
 #include <MySQLToolsCommon/myxutil.h>
+
+#define COMMAND_NSTEXT @"mabackup"
+
 //XXX make a protocol with the exported stuff from MAdministrator and include it here
 extern NSString *MASchemaDataSourceChanged;
 
@@ -42,6 +50,7 @@ extern NSString *MASchemaDataSourceChanged;
 - (MABackupTableSelectionItem*)initWithStatus:(MYX_TABLE_STATUS*)status;
 @end
 
+
 @implementation MABackupSelectionItem
 
 - (void)dealloc
@@ -233,7 +242,6 @@ extern NSString *MASchemaDataSourceChanged;
 - (void)openProfile:(NSString*)name;
 - (void)saveProfile;
 - (void)schemaDSChanged:(NSNotification*)notif;
-- (void)setItem:(id)item status:(MYX_SCHEMA_TABLE_STATUS*)status;
 - (MYX_BACKUP_CONTENT*)getContentSelection;
 - (void)controlTextDidEndEditing:(NSNotification *)aNotification;
 - (MABackupSelectionItem*)addSchemaToContents:(MSchemaItem*)schema;
@@ -294,9 +302,8 @@ static struct {
                              &error);
   if (!_profile)
   {
-    NSLog(@"err %i", error);
-    MARunAlertPanelWithError(nil, 
-                             [NSString stringWithFormat:_(@"Could not load profile '%@'."),name],
+    MARunAlertPanelWithError(@"Error", 
+                             [NSString stringWithFormat:_(@"Could not load Project '%@'."),name],
                              error);
   }
   else
@@ -312,6 +319,8 @@ static struct {
       btn= [modeMatrix cellWithTag:50]; // lock all
     else if (_profile->options & MYX_B_SINGLE_TRANSACTION)
       btn= [modeMatrix cellWithTag:51]; // single trans
+	else if (_profile->options & MYX_B_POINT_IN_TIME_BACKUP)
+	  btn= [modeMatrix cellWithTag:49]; // binlog with pos
     else
       btn= [modeMatrix cellWithTag:52]; // normal
     [btn performClick:self];
@@ -342,13 +351,13 @@ static struct {
 											ofItem:cat];
         if (!sch)
         {
-          NSLog(@"Invalid schema '%s.%s' in backup profile", table->catalog, table->schema);
+          NSLog(@"Invalid schema '%s.%s' in backup project", table->catalog, table->schema);
           continue;
         }
       }
       else
       {
-        NSLog(@"Invalid catalog '%s' in backup profile", table->catalog);
+        NSLog(@"Invalid catalog '%s' in backup project", table->catalog);
         continue;
       }
       
@@ -362,11 +371,11 @@ static struct {
         [sitem selectTable:[NSString stringWithUTF8String:table->table]];
       }
     }
-
     [contentOutline reloadData];
   }
 }
 
+
 - (void)saveProfile
 {
   MYX_ADMIN_LIB_ERROR error;  
@@ -382,6 +391,8 @@ static struct {
     _profile->options|= MYX_B_LOCK_ALL_TABLES;
   else if ([[modeMatrix cellWithTag:51] state] == NSOnState)
     _profile->options|= MYX_B_SINGLE_TRANSACTION;
+  else if ([[modeMatrix cellWithTag:49] state] == NSOnState) 
+	_profile->options|= MYX_B_POINT_IN_TIME_BACKUP;
 
   if ([fullBackupButton state] == NSOnState)
     _profile->options|= MYX_B_COMPLETE_SCHEMATAS;
@@ -402,7 +413,7 @@ static struct {
   
   if (error != MYX_ADMIN_NO_ERROR)
   {
-    MARunAlertPanelWithError(nil, _(@"Could not save profile."), error);
+    MARunAlertPanelWithError(@"Error", _(@"Could not save project."), error);
   }
 }
 
@@ -411,32 +422,6 @@ static struct {
   [schemaOutline reloadData];
 }
 
-/*
-- (void)setItem:(id)item status:(MYX_SCHEMA_TABLE_STATUS*)status
-{
-  unsigned int i;
-  [item resetSchemaStatus:status];
-
-  if (_profile)
-  {
-    MABackupSelectionItem *si= (MABackupSelectionItem*)item;
- 
-    // go through the current profile and select tables that were previously selected
-    for (i= 0; i < _profile->backup_content->tables_num; i++)
-    {
-      MYX_BACKUP_TABLE *table= _profile->backup_content->tables+i;
-      if (strcmp(si->_catalog, table->catalog)==0
-          && strcmp(si->_schema, table->schema)==0)
-      {
-        [si selectTable:[NSString stringWithUTF8String:table->table]];
-      }
-    }
-  }
-
-  [contentOutline reloadData];
-}
-*/
-
 - (MYX_BACKUP_CONTENT*)getContentSelection
 {
   MYX_BACKUP_CONTENT *content= g_malloc(sizeof(MYX_BACKUP_CONTENT));
@@ -526,9 +511,6 @@ static struct {
                               argument:[item schema]->schema_name
 						 waitForWindow:[_owner window]
 							   message:@"Retrieving table data..."];
-                      //finishedSelector:@selector(setItem:status:)
-                      //        argument:selItem
-                      //          target:self];
   if (status)
   {
 	[selItem resetSchemaStatus:status];
@@ -546,11 +528,12 @@ static struct {
 		}
 	  }
 	}
-  
+
+	[[_selectionDS root] addChild:selItem];
+
 	[contentOutline reloadData];
   }
   
-  [[_selectionDS root] addChild:selItem];
   [selItem autorelease];
   
   return selItem;
@@ -682,7 +665,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
         msg= @"Internal error."; 
         break;
     }
-    NSRunAlertPanel(nil, 
+    NSRunAlertPanel(@"Error", 
                     @"Error Performing Backup.\n%@", 
                     @"OK", nil, nil, nil, msg);
   }
@@ -699,7 +682,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 {
   NSFileManager *fman= [NSFileManager defaultManager];
   NSString *prefix= [[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"];
-  NSString *name= _(@"New Profile");
+  NSString *name= _(@"New Project");
   unsigned int i;
   MYX_BACKUP_PROFILE profile;
   MYX_ADMIN_LIB_ERROR err;
@@ -709,7 +692,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   i= 1;
   while ([fman fileExistsAtPath:[NSString stringWithFormat:@"%@/%@.mbp", prefix,name]])
   {
-    name= [NSString stringWithFormat:_(@"New Profile %i"),i++];
+    name= [NSString stringWithFormat:_(@"New Project %i"),i++];
   }
 
   profile.profile_name= (char*)[name UTF8String];
@@ -721,7 +704,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
                         [prefix UTF8String], &profile);
   if (err)
   {
-    MARunAlertPanelWithError(@"Error", @"Could not create profile file.", err);
+    MARunAlertPanelWithError(@"Error", @"Could not create project file.", err);
   }
   
   [self refreshProfiles];
@@ -740,6 +723,12 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   int row= [profileTable selectedRow];
   if (row >= 0)
   {
+    MCrontab *cron = [[[MCrontab alloc] init] autorelease];
+    [cron load];
+    MCrontabEntry *entry = [cron findEntryByComment: [self getCommentText]];
+    [cron removeCommand: [self getMABackupToolPath] withComment: [self getCommentText]];
+	[cron installTable];
+
     NSString *path= [NSString stringWithFormat:@"%@/%@.mbp",
       [[NSUserDefaults standardUserDefaults] stringForKey:@"BackupProfileDirectory"],
       [_profiles objectAtIndex:row]];
@@ -803,7 +792,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 {
   NSOpenPanel *panel= [NSOpenPanel openPanel];
   
-  [panel setTitle:@"Import Profile"];
+  [panel setTitle:@"Import Project"];
   [panel setPrompt:@"Import"];
   if ([panel runModal] == NSFileHandlingPanelOKButton)
   {
@@ -820,7 +809,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
                               &err);
     if (!profile)
     {
-      MARunAlertPanelWithError(nil, [NSString stringWithFormat:@"Could not read the profile file '%@'",path], err);
+      MARunAlertPanelWithError(@"Error", [NSString stringWithFormat:@"Could not read the project file '%@'",path], err);
       return;
     }
     i= 1;
@@ -836,7 +825,7 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
                          [profileDir UTF8String],
                          profile) < 0)
     {
-      NSRunAlertPanel(nil, @"Could not copy the profile file.", nil, nil, nil, nil);
+      NSRunAlertPanel(@"Error", @"Could not copy the project file.", nil, nil, nil, nil);
     }
     myx_free_profile(profile);
   }
@@ -845,22 +834,190 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 - (IBAction)startBackup:(id)sender
 {
   int row= [profileTable selectedRow];
+  
   if (row >= 0)
   {
-    [self doStartBackup];
+    [self saveProfile];
+
+    if (_profile->backup_content->tables_num == 0)
+      NSRunAlertPanel(@"Error", @"There are no tables selected to be backed up.",
+                      nil, nil, nil);
+    else
+      [self doStartBackup];
+  }
+  else
+  {
+	NSRunAlertPanel(@"Error", @"You must first select an existing Backup Project or create a new one.",
+					nil, nil, nil);
   }
 }
 
 - (IBAction)chooseTargetDirectory:(id)sender
 {
+	NSOpenPanel *panel = [NSOpenPanel openPanel];
+	[panel setAllowsMultipleSelection: NO];
+	[panel setCanChooseDirectories: YES];
+	[panel setCanChooseFiles: NO];
+	if([panel runModalForTypes: nil] == NSOKButton)
+	{
+		[targetText setStringValue: [[panel filenames] objectAtIndex: 0]];
+	}
+}
+
+- (BOOL) exchangeCronEntry: entry
+{
+	MCrontab *cron = [[MCrontab alloc] init];
+	[cron load];
+
+	MCrontabEntry *prev = [cron findEntryByComment: [self getCommentText]];
+	if(prev != nil)
+	{
+		[cron removeCommand: [self getMABackupToolPath] withComment: [self getCommentText]];
+		prev = nil;
+	}
+
+	if(entry != nil)
+	{
+		[cron addEntry: entry withComment: [self getCommentText]];
+	}
+	[cron installTable];
+	[self updateScheduleStatus: entry updateControls: NO];
+	return YES;
+}
+
+- (void) updateScheduleStatus: (MCrontabEntry *)entry updateControls: (BOOL)controls
+{
+	static const char *wd[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
+
+	if(entry == nil)
+	{
+		[schedulePopUp selectItemAtIndex: 0];
+		[scheduleTab selectTabViewItemAtIndex: 0];
+		[statusText setStringValue: @"Backups are currently not scheduled"];
+	} 
+	else if(entry->_weekday > 0)
+	{
+		if(controls == YES)
+		{
+			[schedulePopUp selectItemAtIndex: 2];
+			[scheduleTab selectTabViewItemAtIndex: 2];
+			[weeklyTimeText setStringValue: [NSString stringWithFormat: @"%02d:%02d", entry->_hour, entry->_minute]];
+		}
+
+		NSString *weekdays = nil;
+		int i;
+		for(i = 0; i < 7; i++)
+		{
+			if(entry->_weekday & (1 << i))
+			{
+				if(weekdays == nil)
+				{
+					weekdays = [NSString stringWithCString: wd[i]];
+				}
+				else
+				{
+					weekdays = [weekdays stringByAppendingFormat: @", %s", wd[i]];
+				}
+				if(controls == YES)
+				{
+					[[[scheduleWeekdays cells] objectAtIndex: i] setIntValue: 1];
+				}
+			}
+		}
+		if(weekdays == nil)
+		{
+			weekdays = @"(none)";
+		}
+		[statusText setStringValue: [NSString stringWithFormat: @"Backups are currently scheduled weekly on %@ at %02d:%02d", weekdays, entry->_hour, entry->_minute]];
+	}
+	else if([entry->_day length] > 0)
+	{
+		if(controls == YES)
+		{
+			[schedulePopUp selectItemAtIndex: 3];
+			[scheduleTab selectTabViewItemAtIndex: 3];
+			[monthlyDaysText setStringValue: entry->_day];
+			[monthlyTimeText setStringValue: [NSString stringWithFormat: @"%02d:%02d", entry->_hour, entry->_minute]];
+		}
+		[statusText setStringValue: [NSString stringWithFormat: @"Backups are currently scheduled monthly on %@ at %02d:%02d", entry->_day, entry->_hour, entry->_minute]];
+	} 
+	else
+	{
+		if(controls == YES)
+		{
+			[schedulePopUp selectItemAtIndex: 1];
+			[scheduleTab selectTabViewItemAtIndex: 1];
+			[scheduleDailyTime setStringValue: [NSString stringWithFormat: @"%02d:%02d", entry->_hour, entry->_minute]];
+		}
+		[statusText setStringValue: [NSString stringWithFormat: @"Backups are currently scheduled daily at %02d:%02d", entry->_hour, entry->_minute]];
+	}
+	
+}
+
+- (NSString *)getCommandText
+{
+	NSString *retval = [NSString stringWithFormat: @"%@ --directory=\"%@\" --connection=\"%@\"", [self getMABackupToolPath], [targetText stringValue], [[connectionPopUp selectedItem] title]];
+	if([[prefixText stringValue] length] > 0)
+	{
+		retval = [retval stringByAppendingFormat: @" --prefix=\"%@\"", [prefixText stringValue]];
+	}
+	retval = [retval stringByAppendingFormat: @" %@", [_profiles objectAtIndex: [profileTable selectedRow]]];
+	return retval;
 }
 
 - (IBAction)scheduleBackup:(id)sender
 {
+	MCrontabEntry *entry = nil;
+	NSCalendarDate *cd = nil;
+	NSArray *weekdays;
+
+	switch([schedulePopUp indexOfSelectedItem])
+	{
+	case 0:
+		break;
+	case 1:
+		cd = [NSCalendarDate dateWithString: [scheduleDailyTime stringValue] calendarFormat: @"%H:%M"];
+		entry = [[MCrontabEntry alloc] init];
+		entry->_minute = [cd minuteOfHour];
+		entry->_hour = [cd hourOfDay];
+		entry->_command = [self getCommandText];
+		break;
+	case 2:
+		weekdays = [scheduleWeekdays cells];
+		int mask = 0;
+		int i;
+		for(i = 0; i < [weekdays count]; i++)
+		{
+			NSButtonCell *wd = [weekdays objectAtIndex: i];
+			if([wd intValue] > 0)
+			{
+				mask |= (1 << i);
+			}
+		}
+		cd = [NSCalendarDate dateWithString: [scheduleWeeklyTime stringValue] calendarFormat: @"%H:%M"];
+		entry = [[MCrontabEntry alloc] init];
+		entry->_minute = [cd minuteOfHour];
+		entry->_hour = [cd hourOfDay];
+		entry->_command = [self getCommandText];
+		entry->_weekday = mask;
+		break;
+	case 3:
+		cd = [NSCalendarDate dateWithString: [monthlyTimeText stringValue] calendarFormat: @"%H:%M"];
+		entry = [[MCrontabEntry alloc] init];
+		entry->_minute = [cd minuteOfHour];
+		entry->_hour = [cd hourOfDay];
+		entry->_command = [self getCommandText];
+		entry->_day = [monthlyDaysText stringValue];
+		break;
+	}
+
+	[self exchangeCronEntry: entry];
+
 }
 
 - (IBAction)unscheduleBackup:(id)sender
 {
+	[self exchangeCronEntry: nil];
 }
 
 - (IBAction)abortBackup:(id)sender
@@ -921,6 +1078,51 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   return YES;
 }
 
+- (NSString *) getActiveProfileName
+{
+	return [_profiles objectAtIndex: [profileTable selectedRow]];
+}
+
+- (BOOL)willClose
+{
+  if (_profile)
+    [self saveProfile];
+  return YES;
+}
+
+- (NSString *) getCommentText
+{
+	return [NSString stringWithFormat: @"added automatically by MySQL Administrator - do not edit %@", [self getActiveProfileName]];
+}
+
+- (void)loadConnections
+{
+	NSString *connFile = [[MPreferences preferences] pathForFile:@"mysqlx_user_connections.xml"];
+	[connectionPopUp removeAllItems];
+	MYX_LIB_ERROR conerr;
+	if(_pconns != NULL)
+	{
+		myx_free_user_connections(_pconns);
+	}
+	_pconns = myx_load_user_connections([connFile cString], &conerr);
+	if(_pconns != NULL)
+	{
+		int i;
+		for(i = 0; i < _pconns->user_connections_num; i++)
+		{
+			if(_pconns->user_connections[i].connection_name != NULL)
+			{
+				[connectionPopUp addItemWithTitle: [NSString stringWithCString: _pconns->user_connections[i].connection_name]];
+				
+			} 
+		}
+	}
+}
+
+- (NSString *) getMABackupToolPath
+{
+	return [NSString stringWithFormat: @"\"%@/%@\"", [[NSBundle mainBundle] resourcePath], @"mabackup"];
+}
 
 - (void)awakeFromNib
 {
@@ -950,9 +1152,21 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   [[[topBox viewWithTag:201] cell] setImageDimsWhenDisabled:NO];
   [[topBox viewWithTag:201] setImage:[NSImage imageNamed:@"del_disabled.png"]];
   [[topBox viewWithTag:201] setEnabled:NO];
+  
+  [[topBox viewWithTag:200] setToolTip:@"Create a new Backup Project."];
+  [[topBox viewWithTag:201] setToolTip:@"Delete the selected Backup Project."];
 
   // disable backup scheduling page for now
-  [tabView removeTabViewItem:[tabView tabViewItemAtIndex:2]];
+  //[tabView removeTabViewItem:[tabView tabViewItemAtIndex:2]];
+
+    [[[tabView tabViewItemAtIndex: 2] view] setEnabledRecursive: [profileTable selectedRow] >= 0];
+
+	[self loadConnections];
+	
+	MCrontab *cron = [[MCrontab alloc] init];
+	[cron load];
+	MCrontabEntry *entry = [cron findEntryByComment: [self getCommentText]];
+	[self updateScheduleStatus: entry updateControls: YES];
 }
 
 
@@ -960,7 +1174,8 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
   int row= [profileTable selectedRow];
-    
+  MCrontabEntry *entry = nil;
+
   if (_profile)
     [self saveProfile];
 
@@ -969,6 +1184,10 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
     [self openProfile: [_profiles objectAtIndex:row]];
     [[topBox viewWithTag:201] setEnabled:YES];
     [[topBox viewWithTag:201] setImage:[NSImage imageNamed:@"del_normal.png"]];
+
+    MCrontab *cron = [[[MCrontab alloc] init] autorelease];
+    [cron load];
+    entry = [cron findEntryByComment: [self getCommentText]];
   }
   else
   {
@@ -976,6 +1195,9 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
     [[topBox viewWithTag:201] setEnabled:NO];
     [[topBox viewWithTag:201] setImage:[NSImage imageNamed:@"del_disabled.png"]];
   }
+
+  [self updateScheduleStatus: entry updateControls: YES];
+  [[[tabView tabViewItemAtIndex: 2] view] setEnabledRecursive: row >= 0];
 }
 
 //========================== Outline Delegate =====================
@@ -1049,6 +1271,13 @@ static void *do_backup(MYSQL *mysql, MYX_BACKUP_PROFILE *profile, char *path,
   }  
 }
 
+- (IBAction)showConnectionEditor:(id)sender;
+{
+  MPreferenceEditor *editor = [[MPreferenceEditor alloc] initForConnectionsFile: [[MPreferences preferences] pathForFile:@"mysqlx_user_connections.xml"]];  
+  [editor runConnectionEditor];
+  [editor release];
+  [self loadConnections];
+}
 
 //======================== Table DataSource =====================
 
