@@ -9,6 +9,8 @@
 #import <MySQLToolsCommon/mxUtils.h>
 #import "MATableMaintenanceController.h"
 
+//XXX switch to use MSchemaEditHelper
+
 @interface MACatalogIndexItem : MTreeItem
 {
   @public
@@ -286,6 +288,12 @@
   [self openTableMaintenance:'O'];
 }
 
+
+- (void)tableChangedObserver:(NSNotification*)notif
+{
+  [self refreshTables:nil];
+}
+
 - (IBAction)createTable:(id)sender
 {
   MYX_SCHEMA *schema= [self selectedSchema];
@@ -301,7 +309,7 @@
   [editor showNewTableForCatalog:NSStr(schema->catalog_name)
                           schema:NSStr(schema->schema_name)];
   
-  [editor setReleaseOnClose:YES];
+  [editor setReleaseOnClose:YES];  
 }
 
 - (IBAction)editTable:(id)sender
@@ -327,23 +335,24 @@
 }
 
 
-static void *dropTables(MYSQL *mysql, void *schema, void *tables)
+static void *dropTables(MYSQL *mysql, void *schema, void *tables_arg)
 {
-  unsigned int i, c= [(NSArray*)tables count];
+  unsigned int i;
+  char **tables= (char**)tables_arg;
   
-  for (i= 0; i < c; i++)
+  for (i= 0; tables[i]!=NULL; i++)
   {
     MYX_LIB_ERROR error;
-    NSString *query= [[NSString alloc] initWithFormat: @"DROP TABLE `%s`.`%@`",
-      (char*)schema, [(NSArray*)tables objectAtIndex:i]];
-    
-    myx_query_execute_direct(mysql, [query UTF8String], &error);
-    [query release];
+	char *query= g_strdup_printf("DROP TABLE `%s`.`%s`",
+								 (char*)schema, tables[i]);
+
+    myx_query_execute_direct(mysql, query, &error);
+	g_free(query);
 
     if (error != MYX_NO_ERROR)
     {
-      return [[NSString alloc] initWithFormat: @"Error dropping table %@: %@", 
-        [(NSArray*)tables objectAtIndex:i], MXGetErrorString(error)];
+      return [[NSString alloc] initWithFormat: @"Error dropping table %s: %s", 
+        tables[i], mysql_error(mysql)];
     }
   }
   return nil;
@@ -356,50 +365,58 @@ static void *dropTables(MYSQL *mysql, void *schema, void *tables)
 {
   MYX_SCHEMA *schema= [self selectedSchema];
   NSString *error;
-  
+
   [[sheet window] orderOut:self];
-  
-  [(id)contextInfo autorelease];
-  
+
   if (!schema || returnCode != NSAlertDefaultReturn)
+  {
+	g_free(contextInfo);
     return;
+  }
   
   error= [[_owner dispatcher] performCallback: dropTables
                                      argument: schema->schema_name
                                      argument: (id)contextInfo 
                                 waitForWindow: [topBox window]
                                       message: @"Dropping tables..."];
-  if (!error)
+  if (error)
   {
     NSRunAlertPanel(@"Error", error, nil, nil, nil);
   }
   
-  [tablesTable reloadData];
+  g_free(contextInfo);
+    
+  [self refreshTables:nil];
 }
 
 
 - (IBAction)dropTable:(id)sender
 {
-  NSMutableArray *tables= [[NSMutableArray array] retain];
   NSIndexSet *indexes= [tablesTable selectedRowIndexes];
-  unsigned int i;
+  unsigned int i, j;
+  char **tables;
   
+  if ([indexes count] == 0)
+	return;
+  
+  tables= g_new(char*, [indexes count]+1);
+  
+  j= 0;
   for (i= [indexes firstIndex]; i != NSNotFound; i= [indexes indexGreaterThanIndex: i])
   {
-    [tables addObject: [NSString stringWithUTF8String:_tableStatus->schema_tables[i].table_name]];
+    tables[j++]= (char*)_tableStatus->schema_tables[i].table_name;
   }
-  
-  
-  if ([tables count] == 0)
-    return;
-  
+  tables[j]= NULL;
+    
   {  
+	char *table_names= g_strjoinv(", ", tables);
     NSAlert *alert = [NSAlert alertWithMessageText:@"Drop Selected Tables"
                                      defaultButton:@"Drop Tables" 
                                    alternateButton:@"Cancel" 
                                        otherButton:nil
-                         informativeTextWithFormat:@"The following tables will be dropped and all data contained in them will be permanently lost!\n\n %@",
-      [tables componentsJoinedByString:@", "]];
+                         informativeTextWithFormat:@"The following tables will be dropped and all data contained in them will be permanently lost!\n\n %s",
+      table_names];
+	g_free(table_names);
     
     [alert beginSheetModalForWindow:[topBox window] modalDelegate:self 
                      didEndSelector:@selector(dropTablesAlertDidEnd:returnCode:contextInfo:) 
@@ -470,8 +487,13 @@ static void *dropTables(MYSQL *mysql, void *schema, void *tables)
     [indexOutline setDataSource:[[MTreeDataSource alloc] init]];
 	
 	[tablesTable sizeLastColumnToFit];
-	[indexOutline sizeLastColumnToFit];
+	[indexOutline sizeLastColumnToFit];    
 	[schemataOutline sizeLastColumnToFit];
+    
+    [[NSNotificationCenter defaultCenter] addObserver:self
+                                             selector:@selector(tableChangedObserver:)
+                                                 name:MTableEditorTableDidChange
+                                               object:nil];    
   }
   return self;
 }
@@ -482,6 +504,7 @@ static void *dropTables(MYSQL *mysql, void *schema, void *tables)
   if (_tableStatus)
     myx_free_schema_table_status(_tableStatus);
   [tableDetailsTab release];
+  [[NSNotificationCenter defaultCenter] removeObserver:self];
   [super dealloc];
 }
 
@@ -497,6 +520,9 @@ static void *dropTables(MYSQL *mysql, void *schema, void *tables)
   [delSchemaButton setImage:[NSImage imageNamed:@"del_disabled.png"]];
   [delSchemaButton setAlternateImage:[NSImage imageNamed:@"del_pressed.png"]];
   [delSchemaButton setEnabled:NO];
+
+  [[topBox viewWithTag:200] setToolTip:@"Create a new schema (database)."];
+  [[topBox viewWithTag:201] setToolTip:@"Drop the selected schema and all its tables and other assets."];
 }
 
 
