@@ -681,7 +681,7 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
   unsigned int pc= [_pages count];
   unsigned int rc, gc;
   BOOL ok;
-  NSString *path= [[MPreferences preferences] pathForFile:@"administrator/mysqladmin_custom_health.xml"];
+  NSString *path= [[MPreferences preferences] pathForFile:@"Administrator/mysqladmin_custom_health.xml"];
   MYX_HEALTH_PAGES *pages= g_new0(MYX_HEALTH_PAGES, 1);
   
   pages->pages_num= pc;
@@ -755,7 +755,7 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
   }
   
   // try customized
-  path= [[MPreferences preferences] pathForFile:@"administrator/mysqladmin_custom_health.xml"];
+  path= [[MPreferences preferences] pathForFile:@"Administrator/mysqladmin_custom_health.xml"];
   pageData= myx_read_in_health_pages([path UTF8String], &err);
   if (!pageData)
   {
@@ -890,7 +890,7 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
   if (returnCode == NSAlertDefaultReturn)
   {
     _saveGraphs= NO;
-    [[NSFileManager defaultManager] removeFileAtPath:[[MPreferences preferences] pathForFile:@"administrator/mysqladmin_custom_health.xml"]
+    [[NSFileManager defaultManager] removeFileAtPath:[[MPreferences preferences] pathForFile:@"Administrator/mysqladmin_custom_health.xml"]
                                              handler:nil];
     [self reloadGraphs];
   }
