@@ -636,18 +636,51 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   return privs;
 }
 
+
+- (void)handleDefaultsChange:(NSNotification*)notif
+{
+  NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
+
+  if ([defaults boolForKey:@"DisplayGlobalPrivileges"] != globalVisible)
+  {
+    globalVisible= !globalVisible;
+    
+    if (!globalVisible)
+      [tabView removeTabViewItem:globalPageView];
+    else
+      [tabView insertTabViewItem:globalPageView atIndex:1];
+  }
+  
+  if ([defaults boolForKey:@"DisplayTablePrivileges"] != tableVisible)
+  {
+    tableVisible= !tableVisible;
+	
+    if (!tableVisible)
+      [tabView removeTabViewItem:tablePageView];  
+    else
+      [tabView insertTabViewItem:tablePageView atIndex:globalVisible?3:2];
+  }
+}
+
 - (id)initWithOwner: (id<MAdministratorProtocol>)owner
 {
   self= [super initWithNibFile: @"UserAdministration" panelOwner: owner];
   _defaultFrame= [[self topView] frame];
+
+  globalPageView= [[tabView tabViewItemAtIndex:1] retain];
+  tablePageView= [[tabView tabViewItemAtIndex:3] retain];
   
+  globalVisible= YES;
+  tableVisible= YES;  
+
+  [self handleDefaultsChange:nil];
+
   // listen for changes in the schema ds
   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector(schemaDSChanged:)
                                                name:MASchemaDataSourceChanged
                                              object:_owner];
   
-  
   // setup data sources
   _userDS= [[MUserListDataSource alloc] init];
   [_userDS setDefaultUserIcon: [NSImage imageNamed:@"16x16_User.png"]];
@@ -657,7 +690,6 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
                       finishedSelector:@selector(userNamesArrived:names:)
                               argument:nil
                                 target:self];
-
   _schemaDS= [[MFilteredSchemaDataSource alloc] initWithDataSource:[_owner sharedSchemaDS]];
   [schemaOutline setDataSource: _schemaDS];
   MXExpandOutline(schemaOutline, NO);
@@ -665,7 +697,6 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   _tableDS= [[MFilteredSchemaDataSource alloc] initWithDataSource:[_owner sharedColumnDS]];
   [tableOutline setDataSource: _tableDS];
   MXExpandOutline(tableOutline, NO);
-
   {
     NSMutableArray *array;
     ALL_PRIVILEGES *privs;
@@ -712,7 +743,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
     [schemaAssignedTable sizeToFit];
     [schemaAvailableTable setDataSource: _schemaPrivDS];
     [schemaAvailableTable sizeToFit];
-    
+
     array= [NSMutableArray array];
     if (privs->table)
     {
@@ -733,7 +764,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
                                                              type: PTable];
     [tableAssignedTable setDataSource: _tablePrivDS];
     [tableAvailableTable setDataSource: _tablePrivDS];
-    
+
     array= [NSMutableArray array];
     if (privs->column)
     {
@@ -757,42 +788,15 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   }
   
   _privilegeIcon= [[NSImage imageNamed:@"16x16_Right.png"] retain];
-  
+
   [self showUser:nil host:nil];
 
   return self;
 }
 
 
-- (void)handleDefaultsChange:(NSNotification*)notif
-{
-  NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
-
-  
-  if ([defaults boolForKey:@"DisplayGlobalPrivileges"] != globalVisible)
-  {
-    globalVisible= !globalVisible;
-    
-    if (!globalVisible)
-      [tabView removeTabViewItem:globalPageView];
-    else
-      [tabView insertTabViewItem:globalPageView atIndex:1];
-  }
-  
-  if ([defaults boolForKey:@"DisplayTablePrivileges"] != tableVisible)
-  {
-    tableVisible= !tableVisible;
-
-    if (!tableVisible)
-      [tabView removeTabViewItem:tablePageView];  
-    else
-      [tabView insertTabViewItem:tablePageView atIndex:globalVisible?3:2];
-  }
-}
-
-
 - (void)awakeFromNib
-{  
+{
   [userOutline setTarget:self];
   [userOutline setDeleteAction:@selector(deleteUserPressed:)];
   
@@ -804,15 +808,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   [[topBox viewWithTag:300] setEnabled:NO];
   [[topBox viewWithTag:301] setEnabled:NO];
   [tabView setEnabledRecursive:NO];
-
-  globalPageView= [[tabView tabViewItemAtIndex:1] retain];
-  tablePageView= [[tabView tabViewItemAtIndex:3] retain];
-  
-  globalVisible= YES;
-  tableVisible= YES;
-  
-  [self handleDefaultsChange:nil];
-    
+      
   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector(handleDefaultsChange:)
                                                name:NSUserDefaultsDidChangeNotification
@@ -823,6 +819,25 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   [self setButton:TAG_UserRemove enabled:NO image:@"deluser"];
   [self setButton:TAG_HostAdd enabled:NO image:@"addhost"];
   [self setButton:TAG_HostRemove enabled:NO image:@"delhost"];
+
+  {
+	NSView *view;
+	view= [[tabView tabViewItemAtIndex:1] view];
+	[[view viewWithTag:50] setToolTip:@"Grant selected privileges to the user."];
+	[[view viewWithTag:51] setToolTip:@"Revoke selected privileges from the user."];
+	view= [[tabView tabViewItemAtIndex:2] view];  
+	[[view viewWithTag:52] setToolTip:@"Grant selected privileges in the selected schema to the user."];
+	[[view viewWithTag:53] setToolTip:@"Revoke selected privileges in the selected schema from the user."];
+	view= [[tabView tabViewItemAtIndex:3] view];
+	[[view viewWithTag:54] setToolTip:@"Grant selected privileges in the selected object to the user."];
+	[[view viewWithTag:55] setToolTip:@"Revoke selected privileges in the selected object from the user."];
+	
+	[[topBox viewWithTag:200] setToolTip:@"Add a new user account. The account will only be valid once you add a host from where it can connect."];
+	[[topBox viewWithTag:201] setToolTip:@"Remove selected user account and all its registered hosts."];
+	
+	[[topBox viewWithTag:202] setToolTip:@"Add a host from where the selected user can connect."];
+	[[topBox viewWithTag:203] setToolTip:@"Remove the selected host from the user's allowed host list."];
+  }
 }
 
 
@@ -1053,9 +1068,11 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
   else if ([notification object] == schemaOutline)
   {
     MSchemaItem *item= nil;
+
     int row= [schemaOutline selectedRow];
     if (row >= 0)
       item= [schemaOutline itemAtRow:row];
+
     if (item && item->type != MSchemaItemType)
       item= nil;
     if (item)
@@ -1065,7 +1082,8 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
 
       if (schema && host && user)
       {
-        NSString *object= [NSString stringWithFormat:@"%s.%s",schema->catalog_name,schema->schema_name];
+        NSString *object= [NSString stringWithFormat:@"%s",schema->schema_name];
+
         privs= [user findPrivilegesForObject:object host:host];
         if (!privs)
           privs= [user addPrivileges:_schemaPrivs forObject:object host:host];
@@ -1096,7 +1114,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
       MYX_SCHEMA_TABLE *table= [item table];
       if (table && host && user)
       {
-        NSString *object=[NSString stringWithFormat:@"%s.%s.%s",schema->catalog_name,schema->schema_name,table->table_name];
+        NSString *object=[NSString stringWithFormat:@"%s.%s",schema->schema_name,table->table_name];
         privs= [user findPrivilegesForObject:object host:host];
         if (!privs)
           privs= [user addPrivileges:_tablePrivs forObject:object host:host];
@@ -1120,7 +1138,7 @@ static ALL_PRIVILEGES *get_all_privileges(MYSQL *mysql)
       
       if (column && host && user)
       {
-        NSString *object= [NSString stringWithFormat:@"%s.%s.%s.%s",schema->catalog_name,schema->schema_name,table->table_name,column->column_name];
+        NSString *object= [NSString stringWithFormat:@"%s.%s.%s",schema->schema_name,table->table_name,column->column_name];
         privs= [user findPrivilegesForObject:object host:host];
         if (!privs)
           privs= [user addPrivileges:_columnPrivs forObject:object host:host];
