@@ -20,6 +20,8 @@
 #include <sys/types.h>
 #include <string.h>
 #include <errno.h>
+#include <stdio.h>
+#include <iconv.h>
 
 #include "glib.h"
 #define PCRE_STATIC
@@ -112,7 +114,37 @@ extern "C" {
 /*
  * Structs
  */
+///////////////////////////////////////////////////////////////////////////////
+/** @brief International File.
+*//////////////////////////////////////////////////////////////////////////////
+#define BUFFER_LEN 1024*10
+#define MAX_BYTES_PER_CHARACTER 6
+#define DETECT_CHARSET_FROM_SQL_FILE_BUFFER_SIZE 4096
+  
+struct myx_intl_file
+{
+  FILE *file;           // the file descriptor for filename
+  const char *filename; // the file we are reading from
+  const char *charset;  // the encoding of the file
+  gboolean charset_is_utf8; // we have this to avoid costly string comparisons
+                            // for every call of myx_read_char_from_intl_file
+  iconv_t conv_to_utf8, conv_from_utf8;
 
+  // untranslated line
+  unsigned char buffer[BUFFER_LEN + MAX_BYTES_PER_CHARACTER]; 
+  size_t buffer_len_in_bytes;
+
+  gsize bytes_translated; // the number of bytes that really got translated
+                          //   from buffer to utf8_buffer
+
+  gchar *utf8_buffer;    // translated line, may not be null-terminated
+  size_t utf8_buffer_len_in_bytes;
+  gchar *next_utf8_char; // points to the character that is going to be
+                         //   returned on the next call to 
+                         //   myx_read_char_from_file
+  bigint file_pos; // the number of bytes that have already been
+                   // read from this file
+};
 
 /*
  * Functions
@@ -127,6 +159,7 @@ MYX_PUBLIC_FUNC int split_schema_table(const char *ident, char **schema, char **
 MYX_PUBLIC_FUNC char *mask_out_string(char *str, const char open_trigger, const char close_trigger, const char mask);
 MYX_PUBLIC_FUNC char *mask_out_string_re(char *str, const char *open_re, const char open_trigger, const char close_trigger, const char mask);
 MYX_PUBLIC_FUNC char *str_g_replace(char *str, const char *search, const char *replace);
+MYX_PUBLIC_FUNC char *str_g_subst(const char *str, const char *search, const char *replace);
 MYX_PUBLIC_FUNC char *str_g_append(char *base_str, const char *addon);
 MYX_PUBLIC_FUNC char *str_g_append_and_free(char *base_str, char *addon);
 MYX_PUBLIC_FUNC char *str_g_insert(const char *base_str, const char *addon, unsigned int index);
