@@ -92,7 +92,6 @@ typedef enum
   MYX_DBM_OT_FUNCTION
 } MYX_DBM_OBJECT_TYPE;
 
-  
 
 // Custom MySQL error codes
 typedef enum MYX_MYSQL_ERROR_CODES
@@ -100,6 +99,12 @@ typedef enum MYX_MYSQL_ERROR_CODES
   MYX_UTF8_OUT_OF_RANGE=50000
 } MYX_MYSQL_ERROR_CODES;
 
+
+typedef enum
+{
+  MYX_QEL_NOTE= 1, MYX_QEL_WARNING, MYX_QEL_ERROR
+} MYX_QUERY_ERROR_LEVEL;
+
 /*
  * Structs
  */
@@ -118,6 +123,18 @@ typedef struct {
   char *value;
 } MYX_NAME_VALUE_PAIR;
 
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQL_ERROR_MSG
+typedef struct {
+  MYX_QUERY_ERROR_LEVEL level;
+  int error;
+  char *text;
+} MYX_MYSQL_ERROR_MSG;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQL_ERROR_MSGS
+typedef struct {
+  unsigned int errors_num;
+  MYX_MYSQL_ERROR_MSG *errors;
+} MYX_MYSQL_ERROR_MSGS;
 
 // Definition of user connections
 
@@ -498,8 +515,6 @@ typedef enum
 {
   MYX_RSAS_NEW, MYX_RSAS_APPLIED, MYX_RSAS_FAILED, MYX_RSAS_DISCARDED
 } MYX_RS_ACTION_STATUS;
-    
-
 
 // --------------------------
 // Query
@@ -661,6 +676,7 @@ typedef struct MYX_RS_ACTIONS
 
 
 typedef struct {
+  MYX_QUERY_ERROR_LEVEL level;
   int error;
   char *error_text;
   MYX_RS_ACTION *action;
@@ -815,6 +831,7 @@ typedef struct {
   unsigned char auto_inc;
 
   char *default_value;
+  int default_value_is_null;
 
   char *comment;
 
@@ -1030,7 +1047,13 @@ MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
 MYX_PUBLIC_FUNC char * get_value_from_text_ex(const char *txt, int txt_length, const char *regexpr, unsigned int substring_nr);
 
 //MySQL connect functions
-MYX_PUBLIC_FUNC MYSQL* myx_mysql_init();
+MYX_PUBLIC_FUNC int myx_mysql_embedded_start();
+MYX_PUBLIC_FUNC void myx_mysql_embedded_prevent_start();
+MYX_PUBLIC_FUNC void myx_mysql_embedded_shutdown();
+MYX_PUBLIC_FUNC MYSQL * myx_mysql_embedded_init();
+
+MYX_PUBLIC_FUNC MYSQL * myx_mysql_init();
+
 MYX_PUBLIC_FUNC void myx_mysql_set_query_hooks(MYSQL *mysql, 
                                                void (*pre_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
                                                void (*post_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
@@ -1048,6 +1071,7 @@ MYX_PUBLIC_FUNC char * myx_mysql_esc(MYSQL *mysql, const char *cmd, ...);
 MYX_PUBLIC_FUNC int myx_mysql_query(MYSQL *mysql, const char *cmd);
 MYX_PUBLIC_FUNC int myx_mysql_query_esc(MYSQL *mysql, const char *cmd, ...);
 MYX_PUBLIC_FUNC int myx_mysql_real_query(MYSQL *mysql, const char *cmd, unsigned int len);
+MYX_PUBLIC_FUNC bigint myx_mysql_affected_rows(MYSQL *mysql);
 
 MYX_PUBLIC_FUNC char * myx_get_mysql_full_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_major_version(MYSQL *mysql);
@@ -1066,7 +1090,11 @@ MYX_PUBLIC_FUNC char * myx_convert_utf8_dbstr(MYSQL *mysql, const char* str);
 
 MYX_PUBLIC_FUNC int myx_mysql_errno(MYSQL *mysql);
 MYX_PUBLIC_FUNC char * myx_mysql_error(MYSQL *mysql);
+MYX_PUBLIC_FUNC void myx_set_mysql_error(char *error, int errn);
 
+MYX_PUBLIC_FUNC MYX_MYSQL_ERROR_MSGS * myx_mysql_error_msgs_fetch(MYSQL *mysql);
+MYX_PUBLIC_FUNC void myx_mysql_error_msgs_free(MYX_MYSQL_ERROR_MSGS *errors);
+  
 //Network functions
 MYX_PUBLIC_FUNC unsigned long myx_resolve_network_name(const char *hostname);
 MYX_PUBLIC_FUNC int myx_ping_host(unsigned long ip, int ping_timeout, MYX_PING_RESULT *ping_result);
@@ -1130,6 +1158,7 @@ MYX_PUBLIC_FUNC int myx_free_syn(MYX_SYN *syn);
 MYX_PUBLIC_FUNC MYX_SQL_HIGHLIGHTING* myx_init_sql_parsing(MYX_SYN* syn);
 MYX_PUBLIC_FUNC int myx_free_sql_highlighting(MYX_SQL_HIGHLIGHTING *h);
 MYX_PUBLIC_FUNC int myx_highlight(MYX_SQL_HIGHLIGHTING *sql_highlighting, const char *text, unsigned int line_no);
+MYX_PUBLIC_FUNC MYX_SYN_TYPE myx_get_identifier_type(MYX_SYN *syn, const char *word);
 
 /* auto-completion */
 MYX_PUBLIC_FUNC MYX_SYN_SUGGESTIONS* myx_lookup_line(MYX_SYN* syn, const char *line);
@@ -1153,6 +1182,7 @@ MYX_PUBLIC_FUNC int myx_query_add_action(MYX_RESULTSET *resultset, MYX_RS_ACTION
 MYX_PUBLIC_FUNC int myx_query_delete_action(MYX_RESULTSET *resultset, unsigned int action);
 MYX_PUBLIC_FUNC MYX_RS_ACTION_ERRORS * myx_query_apply_actions(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_update_resultset(MYX_RESULTSET *resultset);
+MYX_PUBLIC_FUNC int myx_query_pack_resultset(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_discard_actions(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_free_action_errors(MYX_RS_ACTION_ERRORS *errors);
 
