@@ -45,6 +45,8 @@ MYX_PUBLIC_FUNC xmlChar* get_failsafe_utf8(const unsigned char* latin_str);
 
 MYX_PUBLIC_FUNC int file_exists(const unsigned char *filename);
 
+MYX_PUBLIC_FUNC long long file_size(const char *filename);
+
 MYX_PUBLIC_FUNC xmlNodePtr NewTextChild_int_content(xmlNodePtr parent,
                                                     xmlNsPtr ns, 
                                                     const xmlChar *name,
@@ -66,6 +68,10 @@ MYX_PUBLIC_FUNC void try_to_get_char_field(xmlDocPtr doc, xmlNodePtr cur,
 MYX_PUBLIC_FUNC void try_to_get_float_field(xmlDocPtr doc, xmlNodePtr cur,
                                            const char * name, double * field);
 
+MYX_PUBLIC_FUNC xmlNodePtr try_to_get_child(xmlDocPtr doc, 
+                                            xmlNodePtr cur,
+                                            const char * name);
+
 #ifdef __cplusplus
 }
 #endif
