@@ -377,6 +377,53 @@ char *mask_out_string(char *str, const char open_trigger,
   return str;
 }
 
+
+
+static void __sappend(char **str, int *ressize, int *reslen, const char *sbegin, int count)
+{
+  if (*reslen + count + 1 >= *ressize)
+  {
+    *ressize+= count + 100;
+    *str= g_realloc(*str, *ressize);
+  }
+  strncpy(*str+*reslen, sbegin, count);
+  *reslen+= count;
+  (*str)[*reslen]= 0;
+}
+
+
+
+
+char *str_g_subst(const char *str, const char *search, const char *replace)
+{
+  int ressize, reslen;
+  int replsize= (int)strlen(replace);
+  int searchlen= (int)strlen(search);
+  char *res;
+  const char *bptr, *ptr;
+  
+  g_return_val_if_fail(str != NULL && *str, g_strdup(str));
+  g_return_val_if_fail(search != NULL && *search, g_strdup(str));
+  g_return_val_if_fail(replace != NULL, g_strdup(str));
+
+  ressize= (int)strlen(str)+1;
+  reslen= 0;
+  res= g_malloc(ressize);
+
+  bptr= str;
+
+  while ((ptr= strstr(bptr, search)))
+  {
+    __sappend(&res, &ressize, &reslen, bptr, (int)(ptr-bptr));
+    bptr= ptr+searchlen;
+    __sappend(&res, &ressize, &reslen, replace, replsize);
+  }
+  __sappend(&res, &ressize, &reslen, bptr, (int)strlen(bptr));
+
+  return res;
+}
+
+
 /** 
  * Replaces the search string with the replace string
  * in the given str
@@ -388,15 +435,17 @@ char *mask_out_string(char *str, const char open_trigger,
  **/
 char *str_g_replace(char *str, const char *search, const char *replace)
 {
-  char **splited= g_strsplit(str, search, -1);
-  char *replaced= g_strjoinv(replace, splited);
+  char *res;
 
-  g_strfreev(splited);
+  if (!replace)
+    replace= "";
+
+  res= str_g_subst(str, search, replace);
   g_free(str);
-
-  return replaced;
+  return res;
 }
 
+
 /**
  * Appends the @addon string to the end of
  * @base_str. @base_str is expected to have
@@ -1443,19 +1492,6 @@ void strlist_g_append(char ***list, char *value)
 }
 
 
-static void __sappend(char **str, int *ressize, int *reslen, const char *sbegin, int count)
-{
-  if (*reslen + count + 1 >= *ressize)
-  {
-    *ressize+= count + 100;
-    *str= g_realloc(*str, *ressize);
-  }
-  strncpy(*str+*reslen, sbegin, count);
-  *reslen+= count;
-  (*str)[*reslen]= 0;
-}
-
-
 const char *strfindword(const char *str, const char *word)
 {
   const char *ptr;
@@ -1831,7 +1867,7 @@ int myx_mkdir(const char *filename, int mode)
 
 int get_str_index(char **string_list, unsigned int string_list_num, const char *search)
 {
-  unsigned int i= 0, index= -1;
+  unsigned int i= 0;
 
   while (i<string_list_num)
   {
@@ -1841,7 +1877,7 @@ int get_str_index(char **string_list, unsigned int string_list_num, const char *
     i++;
   }
 
-  return i;
+  return -1;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
