@@ -544,6 +544,8 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
 
       for(column= table_data->columns; (row= mysql_fetch_row(res))!=NULL; column++)
       {
+        column->default_value_is_null= 1;
+
         for (i= 0; i < num_fields; i++)
         {
           if (strcmp(fields[i].name, "Field") == 0)
@@ -583,6 +585,8 @@ MYX_DBM_TABLE_DATA * myx_dbm_retrieve_table_data(MYSQL *mysql, MYX_DBM_DATATYPES
           }
           else if (strcmp(fields[i].name, "Default") == 0)
           {
+            column->default_value_is_null= row[i] ? 0 : 1;
+
             column->default_value= myx_convert_dbstr_utf8(mysql, row[i]);
           }
           else if (strcmp(fields[i].name, "Extra")==0)
@@ -986,6 +990,16 @@ MYX_DBM_CHARSETS * myx_dbm_retrieve_charsets(MYSQL *mysql, MYX_LIB_ERROR *error_
   return charsets;
 }
 
+static int string_contains_only(const char* str, char c)
+{
+	if(*str == '\0')
+	{
+		return 0;
+	}
+	while(*str++ == c);
+	return *str == '\0';
+}
+
 char * dbm_get_sql_column_create_code(MYX_DBM_COLUMN_DATA *column, MYX_DBM_SERVER_VERSION *version, int pk_clause)
 {
   char *col_def, *tmp;
@@ -1019,10 +1033,15 @@ char * dbm_get_sql_column_create_code(MYX_DBM_COLUMN_DATA *column, MYX_DBM_SERVE
   if(column->not_null)
     col_def= str_g_append(col_def, " NOT NULL");
 
-  if(!STR_EMPTY(column->default_value))
+  if(!column->default_value_is_null && column->default_value && column->default_value[0])
   {
-    col_def= str_g_append(col_def, " DEFAULT ");
-    col_def= str_g_append(col_def, column->default_value);
+    if(column->datatype_pointer->group == MYX_DBM_DTG_NUMERIC) 
+    {
+      if (strcmp(column->default_value, "") != 0)
+        col_def= g_strdup_printf("%s DEFAULT %s", col_def, column->default_value);
+    }
+    else
+      col_def= g_strdup_printf("%s DEFAULT '%s'", col_def, column->default_value);
   }
 
   if(column->auto_inc)
@@ -1073,7 +1092,7 @@ char * myx_dbm_get_sql_index_create_code(MYX_DBM_INDEX_DATA *index, MYX_DBM_SERV
     tmp= NULL;
 
   //Index type
-  if((version->major_version>=4) && (version->minor_version>=1))
+  if((version->major_version>=4) && (version->minor_version>=1) && (index->index_kind != MYX_DBM_IK_FULLTEXT))
   {
     if(index->index_type == MYX_DBM_IT_BTREE)
       tmp2= g_strdup("USING BTREE");
@@ -1202,14 +1221,14 @@ char * myx_dbm_get_sql_option_create_code(MYX_DBM_TABLE_DATA *table, MYX_DBM_SER
   //Engine
   switch (table->table_engine)
   {
-    case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "TYPE = InnoDB" _br); break;
-    case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "TYPE = MEMORY" _br); break;
-    case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "TYPE = MERGE" _br);  break;
-    case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "TYPE = BDB" _br);    break;
-    case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "TYPE = ISAM" _br);   break;
-    case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "TYPE = NDB" _br);    break;
+    case MYX_DBM_TE_INNODB: sql= str_g_append(sql, "ENGINE = InnoDB" _br); break;
+    case MYX_DBM_TE_MEMORY: sql= str_g_append(sql, "ENGINE = MEMORY" _br); break;
+    case MYX_DBM_TE_MERGE:  sql= str_g_append(sql, "ENGINE = MERGE" _br);  break;
+    case MYX_DBM_TE_BDB:    sql= str_g_append(sql, "ENGINE = BDB" _br);    break;
+    case MYX_DBM_TE_ISAM:   sql= str_g_append(sql, "ENGINE = ISAM" _br);   break;
+    case MYX_DBM_TE_NDB:    sql= str_g_append(sql, "ENGINE = NDB" _br);    break;
     //default: sql= g_strdup(""); break;
-    default: sql= str_g_append(sql, "TYPE = MYISAM" _br); break;
+    default: sql= str_g_append(sql, "ENGINE = MYISAM" _br); break;
   }
 
   //auto_inc
@@ -1387,8 +1406,9 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
   MYX_DBM_COLUMN_DATA *column, *ex_column;
   MYX_DBM_INDEX_DATA *index, *ex_index;
   MYX_DBM_FK_DATA *fk, *ex_fk;
+  unsigned int collation_added= 0;
 
-#define NAMECMP(a,b) (a&&b?strcasecmp(a,b):(a?-1:1))
+#define NAMECMP(a,b) (a&&b?strcmp(a,b):(a?-1:1))
 
   *error_code= MYX_NO_ERROR;
 
@@ -1466,10 +1486,10 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
 
         //Check DefaultValue
         //if there is no default value
-        if (STR_EMPTY(column->default_value))
+        if (STR_EMPTY(column->default_value) || column->default_value_is_null)
         {
           //and the default value is different than 0, "", 0-datetime and 0-date
-          if((ex_column->default_value) &&
+          if((ex_column->default_value || !column->default_value_is_null) &&
             (strcmp2(ex_column->default_value, "0") != 0) &&
             (strcmp2(ex_column->default_value, "") != 0) &&
             (strcmp2(ex_column->default_value, "0000-00-00 00:00:00") != 0) &&
@@ -1805,7 +1825,7 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       for(j= 0; j<altered_table->fks_num; j++)
         if(NAMECMP(ex_fk->name, altered_table->fks[j].name) == 0)
         {
-          fk= altered_table->fks+i;
+          fk= altered_table->fks+j;
           break;
         }
 
@@ -1911,25 +1931,43 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
       alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "DELAY_KEY_WRITE = %i",
                                                       altered_table->delay_key_write));
 
-    //Charset
+    //charset
     if((strcmp3(existing_table->charset, altered_table->charset)!=0) &&
       !STR_EMPTY(altered_table->charset))
     {
       alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET %s",
                                                       altered_table->charset));
+
+      collation_added= 1;
     }
 
-    if ((strcmp3(existing_table->collation, altered_table->collation)!=0) &&
-      !STR_EMPTY(altered_table->collation))
-      if(((version->major_version >= 5) || ((version->major_version >= 4) && (version->minor_version >= 1))))
+    //collation
+    if ((strcmp3(existing_table->collation, altered_table->collation) != 0) &&
+      !STR_EMPTY(altered_table->collation) &&
+      ((version->major_version >= 5) || ((version->major_version >= 4) && (version->minor_version >= 1))))
+    {
+      // if the charset has not been added yet, add it now
+      if ((collation_added == 0) && !STR_EMPTY(altered_table->charset))
       {
-        if(!(strcmp2(existing_table->charset, altered_table->charset)!=0))
-          alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET %s",
-                                                      altered_table->charset));
+        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET %s",
+                                                    altered_table->charset));
 
-        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(" COLLATE %s", 
-                                                        altered_table->collation));
+        collation_added= 1;
       }
+      else if ((collation_added == 0) && STR_EMPTY(altered_table->charset))
+      {
+        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(_br "CHARACTER SET Default"));
+
+        collation_added= 1;
+      }
+
+
+      if (collation_added == 1)
+      {
+        alter_options= str_g_append_and_free(alter_options, g_strdup_printf(" COLLATE %s", 
+                                                      altered_table->collation));
+      }
+    }
 
     //comment
     if(strcmp3(existing_table->comment, altered_table->comment)!=0)
@@ -2213,6 +2251,15 @@ static const char * view_show_create[]=
 static const char ** view_show_create_end=
                  view_show_create + sizeof(view_show_create)/sizeof(char*);
 
+static const char * view_show_create_new[]=
+{
+  "View",       // 0 (col 1)
+  "Create View"// 1 (col 2)
+};
+static const char ** view_show_create_new_end=
+                 view_show_create_new + sizeof(view_show_create_new)/sizeof(char*);
+
+
 MYX_DBM_VIEW_DATA *myx_dbm_get_view_data(MYSQL *mysql, const char *catalog, const char *schema,
                                          const char *view)
 {
@@ -2238,8 +2285,12 @@ MYX_DBM_VIEW_DATA *myx_dbm_get_view_data(MYSQL *mysql, const char *catalog, cons
         num_fields= mysql_num_fields(res);
         fields= mysql_fetch_fields(res);
 
-        build_field_subst(view_show_create,view_show_create_end,
-                          fields,fields+num_fields,fi);
+        if (mysql_full_version_is_later_or_equal_than(mysql, 5, 0, 3))
+          build_field_subst(view_show_create_new,view_show_create_new_end,
+                            fields,fields+num_fields,fi);
+        else
+          build_field_subst(view_show_create,view_show_create_end,
+                            fields,fields+num_fields,fi);
 
         vdata= g_new0(MYX_DBM_VIEW_DATA, 1);
         vdata->catalog= g_strdup(catalog);
@@ -2364,6 +2415,9 @@ char * myx_dbm_get_create_sql(MYSQL *mysql, const char *catalog_name, const char
   unsigned int num_fields;
   char *sql= NULL;
 
+  if ((object_type == MYX_DBM_OT_SCHEMA) && (!mysql_version_is_later_or_equal_than(mysql, 4, 1)))
+    return g_strdup_printf("CREATE DATABASE `%s`", schema_name);
+
   if ((object_type != MYX_DBM_OT_PROCEDURE) && (object_type !=MYX_DBM_OT_FUNCTION))
   {
     if (object_type == MYX_DBM_OT_SCHEMA)
