@@ -36,6 +36,10 @@ typedef float   pfloat;         /* Mixed prototypes can take float */
 #endif
 
 #include <mysql_version.h>
+
+#if defined (__WIN__) || defined(_WIN32) || defined(_WIN64)
+#include <my_global.h>
+#endif
 #include <m_ctype.h>
 
 
@@ -48,6 +52,7 @@ typedef struct {
   short major_version;
   short patchlevel;
   char quote_char;
+  short embedded;
 } MYX_MYSQL;
 
 
@@ -67,6 +72,8 @@ MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql);
 //enum myx_lib_error error_code;
 
 static char *myx_my_cnf_path= NULL;
+static char *mysql_last_error= NULL;
+static int mysql_last_errno= 0;
 
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief parameters of mysql_real_connect()
@@ -77,7 +84,7 @@ typedef struct
   MYSQL         * mysql;
   const char    * unix_socket;
   unsigned long   client_flag;
-  unsigned int    ansi_quotes:1;
+  signed char     ansi_quotes;
 }MYSQL_REAL_CONNECT_PARAMETERS;
 
 
@@ -178,7 +185,7 @@ void oi_ansi_quotes(ADVANCED_OPTION_DESCRIPTION * opt,
                     const char * value,
                     MYSQL_REAL_CONNECT_PARAMETERS * options)
 {
-  options->ansi_quotes= 1;
+  options->ansi_quotes= value ? toupper(*value) == 'Y' : -1;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -342,6 +349,59 @@ MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql)
 }
 
 
+const char *server_args[] =
+{
+  "app_dummy",
+  "--basedir=./embedded",
+  "--datadir=./embedded/data",
+  "--default-character-set=utf8",
+  "--default-collation=utf8_general_ci",
+  "--default-storage-engine=InnoDB",
+  "--log-error=./embedded/error.log"
+};
+static char *server_groups[] = {
+  "none",
+  (char *)NULL
+};
+
+int myx_mysql_embedded_start()
+{
+  return mysql_server_init(sizeof(server_args) / sizeof(char *), (char **)server_args, (char **)server_groups);
+}
+
+void myx_mysql_embedded_prevent_start()
+{
+  mysql_server_init(-1, NULL, NULL);
+}
+
+
+void myx_mysql_embedded_shutdown()
+{
+  mysql_server_end();
+}
+
+MYSQL * myx_mysql_init_handler(int embedded)
+{
+  char *data= g_malloc0(sizeof(MYSQL)+sizeof(MYX_MYSQL));
+
+  if (data)
+  {
+    MYSQL *mysql= (MYSQL*)data;
+    MYX_MYSQL *myx_mysql= myx_mysql_get_private(mysql);
+
+    mysql_init(mysql);
+  
+    if (myx_my_cnf_path)
+      mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, myx_my_cnf_path);
+
+    myx_mysql->embedded= embedded;
+    
+    return mysql;
+  }
+  return NULL;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief create MYSQL handler to work with MySQL API
     @ingroup bridge_to_MySQL_API
@@ -351,22 +411,23 @@ MYX_MYSQL *myx_mysql_get_private(MYSQL *mysql)
     calls mysql_init() and reads configuration file (via mysql_options())
      if myx_set_my_cnf_path() was called before
 *//////////////////////////////////////////////////////////////////////////////
-MYSQL* myx_mysql_init()
+MYSQL * myx_mysql_init()
 {
-  char *data;
-  data= g_malloc0(sizeof(MYSQL)+sizeof(MYX_MYSQL));
-  if (data)
-  {
-    MYSQL *mysql= (MYSQL*)data;
+  return myx_mysql_init_handler(0);
+}
 
-    mysql_init(mysql);
-  
-    if (myx_my_cnf_path)
-      mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, myx_my_cnf_path);
-    
-    return mysql;
-  }
-  return NULL;
+///////////////////////////////////////////////////////////////////////////////
+/** @brief create MYSQL handler to work with MySQL API over the embedded server
+    @ingroup bridge_to_MySQL_API
+
+    @return prepared MYSQL struct
+
+    calls mysql_init() and reads configuration file (via mysql_options())
+     if myx_set_my_cnf_path() was called before
+*//////////////////////////////////////////////////////////////////////////////
+MYSQL * myx_mysql_embedded_init()
+{
+  return myx_mysql_init_handler(1);
 }
 
 
@@ -390,6 +451,72 @@ void myx_mysql_set_query_hooks(MYSQL *mysql, MYX_QUERY_CALLBACK pre_hook,
 #endif
 
 
+static void modify_sql_mode(MYSQL *mysql, int ansi_quotes)
+{
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  char *current_mode= NULL;
+
+  if (!mysql_query(mysql, "SELECT @@sql_mode"))
+  {
+    res= mysql_store_result(mysql);
+    row= mysql_fetch_row(res);
+    current_mode= g_strdup(row[0]);
+    mysql_free_result(res);
+  }
+
+  // make sure we're not in ansi_quotes mode, unless asked to
+  if (mysql_version_is_later_or_equal_than(mysql,4,1))
+  {
+    char *query;
+
+    if (ansi_quotes < 0) // not set, keep current value
+      ;
+    else if (ansi_quotes == 0) // OFF
+    {
+      char *ptr;
+      if (current_mode && (ptr=strstr(current_mode, "ANSI_QUOTES")))
+      {
+        char **tokens= g_strsplit(current_mode, ",", 0);
+        unsigned int i, j;
+        for (i= j= 0; tokens[i]; i++)
+        {
+          if (strcasecmp(tokens[i], "ANSI_QUOTES")==0)
+            ;
+          else
+            j++;
+          tokens[i]= tokens[j];
+        }
+        tokens[j]= NULL;
+        g_free(current_mode);
+        if (tokens[0])
+        {
+          current_mode= g_strjoin(",", tokens);
+          g_strfreev(tokens);
+        }
+        else
+          current_mode= NULL;
+      }
+    }
+    else // ON
+    {
+      if (!current_mode || !strstr(current_mode, "ANSI_QUOTES"))
+      {
+        // add ANSI_QUOTES to mode strstr
+        if (current_mode && strlen(current_mode) > 0) 
+          current_mode= str_g_append(current_mode, ",ANSI_QUOTES");
+        else
+          current_mode= str_g_append(current_mode, "ANSI_QUOTES");
+      }
+    }
+
+    query= g_strdup_printf("SET SESSION sql_mode='%s'", current_mode);
+    mysql_query(mysql, query);
+    g_free(query);
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief prepare a connection to a MySQL server
     @ingroup bridge_to_MySQL_API
@@ -406,7 +533,7 @@ void myx_mysql_set_query_hooks(MYSQL *mysql, MYX_QUERY_CALLBACK pre_hook,
 *//////////////////////////////////////////////////////////////////////////////
 int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
 {
-  MYSQL_REAL_CONNECT_PARAMETERS connect_options= { mysql, 0, 0};  
+  MYSQL_REAL_CONNECT_PARAMETERS connect_options= { mysql, 0, 0, -1};
   char *unix_socket= NULL;
   
   /* setup advanced options */
@@ -432,10 +559,11 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
   if (!connect_options.unix_socket)
   {
     gchar *out= NULL, *err= NULL;
-    gint est;
+//    gint est; can't request exist status because when it's being called
+//    in the GRT, it will have SIGCHLD caught by the JVM and g_spawn needs it
     if (g_spawn_command_line_sync("mysql_config --socket",
-                                  &out, &err, &est,
-                                  NULL) && out && est == 0)
+                                  &out, &err, NULL,
+                                  NULL) && out)
     {
       char *ptr= strchr(out, '\n');
       if (ptr)
@@ -453,6 +581,11 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
   //of mysql_real_connect are in utf8
   //mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
 
+  if (myx_mysql_get_private(mysql)->embedded)
+    mysql_options(mysql, MYSQL_OPT_USE_EMBEDDED_CONNECTION, (char *)1);
+  else
+    mysql_options(mysql, MYSQL_OPT_USE_REMOTE_CONNECTION, (char *)1);
+
   if (!mysql_real_connect(mysql, user_conn->hostname, user_conn->username,
                           user_conn->password, user_conn->schema,
                           user_conn->port, 
@@ -468,14 +601,7 @@ int myx_connect_to_instance(MYX_USER_CONNECTION *user_conn, MYSQL *mysql)
   // set a high timeout value
   mysql_query(mysql, "SET SESSION interactive_timeout=1000000");
 
-  // make sure we're not in ansi_quotes mode, unless asked to
-  if (mysql_version_is_later_or_equal_than(mysql,4,1))
-  {
-    if (connect_options.ansi_quotes)
-      mysql_query(mysql, "SET SESSION sql_mode='ANSI_QUOTES'");
-    else
-      mysql_query(mysql, "SET SESSION sql_mode=''");
-  }
+  modify_sql_mode(mysql, connect_options.ansi_quotes);
 
   //using SET NAMES utf8;
   if ( (mysql_full_version_is_later_or_equal_than(mysql, 4, 1, 8) && 
@@ -616,15 +742,21 @@ unsigned long myx_get_thread_id(MYSQL *mysql)
     @param mysql connection handler
     @return An error code value for the last mysql_xxx call, if it failed.
             zero means no error occurred.
-3A
 *//////////////////////////////////////////////////////////////////////////////
 int myx_mysql_errno(MYSQL *mysql)
 {
-  return mysql_errno(mysql);
+  if(mysql_last_error == NULL)
+  {
+    return mysql_errno(mysql);
+  }
+  return mysql_last_errno;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get mysql_error() for connection
+
+    warning: this function uses static variables and therefore is not thread-safe
+
     @ingroup bridge_to_MySQL_API
     @param mysql connection handler
     @return A null-terminated character string that describes the error.
@@ -633,9 +765,84 @@ int myx_mysql_errno(MYSQL *mysql)
 *//////////////////////////////////////////////////////////////////////////////
 char *myx_mysql_error(MYSQL *mysql)
 {
-  return g_strdup(mysql_error(mysql));
+  char *e;
+  if(mysql_last_error == NULL)
+  {
+    return g_strdup(mysql_error(mysql));
+  }
+  e= mysql_last_error;
+  mysql_last_error= NULL;
+  return e;
 }
 
+void myx_set_mysql_error(char *error, int errn)
+{
+  mysql_last_error= error;
+  mysql_last_errno= errn;
+}
+
+MYX_MYSQL_ERROR_MSGS *myx_mysql_error_msgs_fetch(MYSQL *mysql)
+{
+  MYX_MYSQL_ERROR_MSGS *errors= g_new0(MYX_MYSQL_ERROR_MSGS, 1);
+#if MYSQL_VERSION_ID >= 50000
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  int c;
+
+  c= mysql_warning_count(mysql);
+  if (c > 0 && !myx_mysql_query(mysql, "SHOW WARNINGS") &&
+      (res= mysql_use_result(mysql)))
+  {
+    errors->errors= g_realloc(errors->errors,
+                              sizeof(MYX_MYSQL_ERROR_MSG)*(errors->errors_num+c));
+
+    while ((row= mysql_fetch_row(res)))
+    {
+      MYX_MYSQL_ERROR_MSG *error;
+      error= errors->errors + errors->errors_num++;
+
+      if (strcasecmp(row[0], "Note")==0)
+        error->level= MYX_QEL_NOTE;
+      else
+        error->level= MYX_QEL_WARNING;
+      error->error= atoi(row[1]);
+      error->text= g_strdup(row[2]);
+    }
+    mysql_free_result(res);
+  }
+#endif
+  
+  if (myx_mysql_errno(mysql) != 0)
+  {
+    errors->errors_num++;
+    errors->errors= g_realloc(errors->errors,
+                              sizeof(MYX_MYSQL_ERROR_MSG)*errors->errors_num);
+    errors->errors[errors->errors_num-1].level= MYX_QEL_ERROR;
+    errors->errors[errors->errors_num-1].error= myx_mysql_errno(mysql);
+    errors->errors[errors->errors_num-1].text= myx_mysql_error(mysql);
+  }
+  
+  if (errors->errors_num == 0)
+  {
+    g_free(errors->errors);
+    g_free(errors);
+    errors= NULL;
+  }
+
+  return errors;
+}
+
+
+void myx_mysql_error_msgs_free(MYX_MYSQL_ERROR_MSGS *errors)
+{
+  unsigned int i;
+  for (i= 0; i < errors->errors_num; i++)
+    g_free(errors->errors[i].text);
+  g_free(errors->errors);
+  g_free(errors);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get full version information of mysql server
     @ingroup bridge_to_MySQL_API
@@ -1278,6 +1485,20 @@ int myx_mysql_query(MYSQL *mysql, const char *cmd)
   return rc;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief returns the number of affected rows
+    @ingroup bridge_to_MySQL_API
+
+    @param mysql  connection handler
+
+    @return the number of affected rows
+
+*//////////////////////////////////////////////////////////////////////////////
+bigint myx_mysql_affected_rows(MYSQL *mysql)
+{
+  return (bigint)mysql_affected_rows(mysql);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief execute sql query with given length using mysql connection
     @ingroup bridge_to_MySQL_API
