@@ -133,6 +133,11 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
                                bigint (*read_char)(void *user_data,
                                                    int *bytes_read,
                                                    int *error),
+                               int (*progress_report)(bigint bytes_read,
+                                                       bigint bytes_total,
+                                                       void *user_data),
+                               bigint file_size,
+                               void *user_data,
                                void *arg)
 {
   int h;
@@ -163,6 +168,16 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
   {
     we->bytes_read+= bytes_read;
 
+    if((we->bytes_read % 1024) == 0)
+      {
+      if(progress_report)
+      {
+        if(progress_report(we->bytes_read, file_size, user_data))
+        {
+          return -1;
+        }
+      }
+    }
     if (resize_buffer && h == *buffer_len - 7)
     {
       gchar *tmp;
@@ -246,7 +261,7 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
     }
 
     state= make_state_transition(state, c, we, &delim_i);
-  }
+	}
 
   // readchar returned an error
   if (*error) return 0;
@@ -269,13 +284,22 @@ int myx_get_next_sql_statement(MYX_SQL_PARSE_ENVIRONMENT *we,
 int myx_get_next_sql_statement_file(MYX_SQL_PARSE_ENVIRONMENT *we,
                                     MYX_INTL_FILE *stat,
                                     char **buffer, int *buffer_len,
-                                    int resize_buffer, int *error)
+                                    int resize_buffer, 
+                                    int (*progress_report)(bigint bytes_read,
+                                                           bigint bytes_total,
+                                                           void *user_data),
+                                    bigint file_size,
+                                    void *user_data,
+                                    int *error)
 {
   bigint (* read_char)(void *, int *,int *)= 
         (bigint (*)(void *, int *,int *)) myx_read_char_from_intl_file;
   return myx_get_next_sql_statement(we, buffer, buffer_len,
                                     resize_buffer, error,
                                     read_char,
+                                    progress_report,
+                                    file_size,
+                                    user_data,
                                     (void *)stat);
 }
 
