@@ -46,7 +46,7 @@ typedef enum
 
 //----------------------------------------------------------------------------------------------------------------------
 
-// Convert a primitive name to a number.
+// convert a primitive name to a number.
 GC_PRIMITIVE FindPrimitive(string Name)
 {
   static map<string, GC_PRIMITIVE> Primitives;
@@ -65,11 +65,11 @@ GC_PRIMITIVE FindPrimitive(string Name)
     Primitives["image"] = GC_PRIMITIVE_IMAGE;
   };
 
-  map<string, GC_PRIMITIVE>::iterator Iterator = Primitives.find(Name);
-  if (Iterator == Primitives.end())
+  map<string, GC_PRIMITIVE>::iterator iterator = Primitives.find(Name);
+  if (iterator == Primitives.end())
     return GC_PRIMITIVE_UNKNOWN;
   else
-    return Iterator->second;
+    return iterator->second;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -77,22 +77,22 @@ GC_PRIMITIVE FindPrimitive(string Name)
 /**
  * Scans the given string for coordinate values and fills the Points vector with them.
  */
-void ParsePoints(xmlChar* Raw, CVertexVector* Points)
+void parsePoints(xmlChar* Raw, CVertexVector* Points)
 {
   Points->clear();
 
-  StringTokenizer Tokenizer((char*) Raw, " ,\t\n");
+  StringTokenizer tokenizer((char*) Raw, " ,\t\n");
 
-  while (Tokenizer.HasMoreTokens())
+  while (tokenizer.hasMoreTokens())
   {
     // Invalid input results in a value of 0, so we don't need to take special
     // care for such cases.
-    TVertex V;
-    V.X = Tokenizer.NextTokenAsFloat();
-    if (Tokenizer.HasMoreTokens())
+    TVertex v;
+    v.x = tokenizer.nextTokenAsFloat();
+    if (tokenizer.hasMoreTokens())
     {
-      V.Y = Tokenizer.NextTokenAsFloat();
-      Points->push_back(V);
+      v.y = tokenizer.nextTokenAsFloat();
+      Points->push_back(v);
     };
   };
 }
@@ -114,41 +114,41 @@ CSVGParser::~CSVGParser(void)
 /**
  * Parses the content of a circle definition.
  *
- * @see ParseElement for a description of the parameters.
+ * @see parseElement for a description of the parameters.
  */
-void CSVGParser::ParseCircle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+void CSVGParser::parseCircle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                              float StrokeWidth, CBoundingBoxComputer* BB)
 {
-  GLUquadricObj *Quadric = gluNewQuadric();
-  float InnerRadius = GetFloatAttributeDef(SVG, "inner-radius", 0);
-  float OuterRadius = GetFloatAttributeDef(SVG, "outer-radius", 1);
-  float StartAngle = GetFloatAttributeDef(SVG, "start-angle", 0);
-  float SweepAngle = GetFloatAttributeDef(SVG, "sweep-angle", 360);
-  GLint Slices = GetIntAttributeDef(SVG, "slices", 10);
-  GLint Loops = GetIntAttributeDef(SVG, "rings", 3);
+  GLUquadricObj *quadric = gluNewQuadric();
+  float innerRadius = getFloatAttributeDef(SVG, "inner-radius", 0);
+  float outerRadius = getFloatAttributeDef(SVG, "outer-radius", 1);
+  float startAngle = getFloatAttributeDef(SVG, "start-angle", 0);
+  float sweepAngle = getFloatAttributeDef(SVG, "sweep-angle", 360);
+  GLint Slices = getIntAttributeDef(SVG, "slices", 10);
+  GLint Loops = getIntAttributeDef(SVG, "rings", 3);
 
   // Update the accumulated bounding box.
-  GLfloat Matrix[16];
-  glGetFloatv(GL_MODELVIEW_MATRIX, Matrix);
-  BB->Include(Matrix, -OuterRadius, -OuterRadius);
-  BB->Include(Matrix, OuterRadius, OuterRadius);
+  GLfloat matrix[16];
+  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
+  BB->include(matrix, -outerRadius, -outerRadius);
+  BB->include(matrix, outerRadius, outerRadius);
 
   // Disable polygon smoothing. We cannot have both here, alpha blended figures and polygon smoothing.
   glPushAttrib(GL_ENABLE_BIT);
   glDisable(GL_POLYGON_SMOOTH);
 
-  gluQuadricOrientation(Quadric, GLU_INSIDE);
+  gluQuadricOrientation(quadric, GLU_INSIDE);
 
   if (FillColor != NULL)
     glColor4ubv(FillColor);
 
   if (DoFill)
   {
-    gluQuadricDrawStyle(Quadric, GLU_FILL);
-    gluQuadricNormals(Quadric, GLU_SMOOTH);
-    gluQuadricTexture(Quadric, GL_TRUE);
+    gluQuadricDrawStyle(quadric, GLU_FILL);
+    gluQuadricNormals(quadric, GLU_SMOOTH);
+    gluQuadricTexture(quadric, GL_TRUE);
 
-    gluPartialDisk(Quadric, InnerRadius, OuterRadius, Slices, Loops, StartAngle, SweepAngle);
+    gluPartialDisk(quadric, innerRadius, outerRadius, Slices, Loops, startAngle, sweepAngle);
   };
 
   if (DoStroke)
@@ -156,11 +156,11 @@ void CSVGParser::ParseCircle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bo
     if (StrokeColor != NULL)
       glColor4ubv(StrokeColor);
 
-    gluQuadricDrawStyle(Quadric, GLU_SILHOUETTE);
-    gluPartialDisk(Quadric, InnerRadius, OuterRadius, Slices, Loops, StartAngle, SweepAngle);
+    gluQuadricDrawStyle(quadric, GLU_SILHOUETTE);
+    gluPartialDisk(quadric, innerRadius, outerRadius, Slices, Loops, startAngle, sweepAngle);
   };
 
-  gluDeleteQuadric(Quadric);
+  gluDeleteQuadric(quadric);
   glPopAttrib();
 }
 
@@ -179,7 +179,7 @@ void CSVGParser::ParseCircle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bo
  * @param BB This calculator is used to determine the overall bounding box of the element being parsed.
  * @return Returns a new display list for the content of this element.
  */
-GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+GLuint CSVGParser::parseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                                 float StrokeWidth, float MasterAlpha, CBoundingBoxComputer* BB)
 {
   GC_PRIMITIVE Primitive = FindPrimitive((char*) SVG->name);
@@ -202,41 +202,41 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
       Attribute = xmlGetProp(SVG, (xmlChar*) "texture");
       if (Attribute != NULL)
       {
-        Texture = TextureManager()->FindTexture(string((char*) Attribute));
+        Texture = textureManager()->FindTexture(string((char*) Attribute));
         xmlFree(Attribute);
       };
 
       // See if we have a local color and/or a local opacity.
       float Opacity = MasterAlpha;
-      bool HasOpacity = GetFloatAttribute(SVG, "opacity", Opacity);
-      if (HasOpacity)
+      bool hasOpacity = getFloatAttribute(SVG, "opacity", Opacity);
+      if (hasOpacity)
         Opacity *= MasterAlpha;
 
-      GLubyte LocalFillColor[4];
-      GLubyte* FillColorPtr = NULL;
+      GLubyte localFillColor[4];
+      GLubyte* fillColorPtr = NULL;
 
-      int Conversion = ConvertColor(SVG, "fill", LocalFillColor);
+      int Conversion = convertColor(SVG, "fill", localFillColor);
       // Fill this element if either filling is not completely disabled (fill ="none")
       // and either a local color is given or the parent element used filling already.
-      bool LocalFill = (Conversion != 3) && (DoFill || FillColor != NULL || Conversion == 0);
-      if (LocalFill)
+      bool localFill = (Conversion != 3) && (DoFill || FillColor != NULL || Conversion == 0);
+      if (localFill)
       {
         // If there is no valid local color then use that of the parent.
         if (Conversion != 0)
         {
           // Combine the parent color with this opacity to create a new color.
-          LocalFillColor[0] = FillColor[0];
-          LocalFillColor[1] = FillColor[1];
-          LocalFillColor[2] = FillColor[2];
+          localFillColor[0] = FillColor[0];
+          localFillColor[1] = FillColor[1];
+          localFillColor[2] = FillColor[2];
         };
-        LocalFillColor[3] = GLubyte(255 * Opacity);
-        FillColorPtr = LocalFillColor;
+        localFillColor[3] = GLubyte(255 * Opacity);
+        fillColorPtr = localFillColor;
       };
 
-      GLubyte LocalStrokeColor[4];
-      GLubyte* StrokeColorPtr = NULL;
+      GLubyte localStrokeColor[4];
+      GLubyte* strokeColorPtr = NULL;
 
-      Conversion = ConvertColor(SVG, "stroke", LocalStrokeColor);
+      Conversion = convertColor(SVG, "stroke", localStrokeColor);
       bool LocalStroke = (Conversion != 3) && (DoStroke || StrokeColor != NULL || Conversion == 0);
       if (LocalStroke)
       {
@@ -244,15 +244,15 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
         if (Conversion != 0)
         {
           // Combine the parent color with this opacity to create a new color.
-          LocalStrokeColor[0] = StrokeColor[0];
-          LocalStrokeColor[1] = StrokeColor[1];
-          LocalStrokeColor[2] = StrokeColor[2];
+          localStrokeColor[0] = StrokeColor[0];
+          localStrokeColor[1] = StrokeColor[1];
+          localStrokeColor[2] = StrokeColor[2];
         };
-        LocalStrokeColor[3] = GLubyte(255 * Opacity);
-        StrokeColorPtr = LocalStrokeColor;
+        localStrokeColor[3] = GLubyte(255 * Opacity);
+        strokeColorPtr = localStrokeColor;
       };
 
-      float LocalStrokeWidth = GetFloatAttributeDef(SVG, "stroke-width", StrokeWidth);
+      float LocalStrokeWidth = getFloatAttributeDef(SVG, "stroke-width", StrokeWidth);
 
       // Prepare child display lists before starting with the one for this element.
       GLuint LocalList = 0;
@@ -261,12 +261,17 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
         case GC_PRIMITIVE_TEXT:
         case GC_PRIMITIVE_TSPAN:
           {
-            LocalList = ParseText(SVG, LocalFill, FillColorPtr, LocalStroke, StrokeColorPtr, LocalStrokeWidth, Opacity, BB);
+            LocalList = parseText(SVG, localFill, fillColorPtr, LocalStroke, strokeColorPtr, LocalStrokeWidth, Opacity, BB);
             break;
           };
         case GC_PRIMITIVE_GROUP:
           {
-            LocalList = ParseGroup(SVG, LocalFill, FillColorPtr, LocalStroke, StrokeColorPtr, LocalStrokeWidth, Opacity, BB);
+            LocalList = parseGroup(SVG, localFill, fillColorPtr, LocalStroke, strokeColorPtr, LocalStrokeWidth, Opacity, BB);
+            break;
+          };
+        case GC_PRIMITIVE_IMAGE:
+          {
+            parseImage(SVG, BB, false);
             break;
           };
       };
@@ -276,9 +281,7 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
       // This is needed to compute the bounding box correctly.
       glNewList(ResultList, GL_COMPILE_AND_EXECUTE);
 
-      // Save current transformation matrix, color and texture values.
       glPushMatrix();
-      glPushAttrib(GL_CURRENT_BIT);
 
       if (Texture != NULL)
         Texture->ActivateTexture();
@@ -286,7 +289,7 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
       Attribute = xmlGetProp(SVG, (xmlChar*) "transform");
       if (Attribute != NULL)
       {
-        ParseTransformation((char*) Attribute);
+        parseTransformation((char*) Attribute);
         xmlFree(Attribute);
       };
 
@@ -294,27 +297,27 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
       {
         case GC_PRIMITIVE_RECT:
           {
-            ParseRectangle(SVG, LocalFill, FillColorPtr, LocalStroke, StrokeColorPtr, LocalStrokeWidth, BB);
+            parseRectangle(SVG, localFill, fillColorPtr, LocalStroke, strokeColorPtr, LocalStrokeWidth, BB);
             break;
           };
         case GC_PRIMITIVE_LINE:
           {
-            ParseLine(SVG, LocalStroke, StrokeColorPtr, LocalStrokeWidth, BB);
+            parseLine(SVG, LocalStroke, strokeColorPtr, LocalStrokeWidth, BB);
             break;
           };
         case GC_PRIMITIVE_POLYLINE:
           {
-            ParsePolyline(SVG, LocalFill, FillColorPtr, LocalStroke, StrokeColorPtr, LocalStrokeWidth, BB);
+            parsePolyline(SVG, localFill, fillColorPtr, LocalStroke, strokeColorPtr, LocalStrokeWidth, BB);
             break;
           };
         case GC_PRIMITIVE_POLYGON:
           {
-            ParsePolygon(SVG, LocalFill, FillColorPtr, LocalStroke, StrokeColorPtr, LocalStrokeWidth, BB);
+            parsePolygon(SVG, localFill, fillColorPtr, LocalStroke, strokeColorPtr, LocalStrokeWidth, BB);
             break;
           };
         case GC_PRIMITIVE_CIRCLE:
           {
-            ParseCircle(SVG, LocalFill, FillColorPtr, LocalStroke, StrokeColorPtr, LocalStrokeWidth, BB);
+            parseCircle(SVG, localFill, fillColorPtr, LocalStroke, strokeColorPtr, LocalStrokeWidth, BB);
             break;
           };
         case GC_PRIMITIVE_TEXT:
@@ -327,7 +330,7 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
           };
         case GC_PRIMITIVE_IMAGE:
           {
-            ParseImage(SVG, BB);
+            parseImage(SVG, BB, true);
             break;
           };
       };
@@ -335,7 +338,6 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
       if (Texture != NULL)
         Texture->DeactivateTexture();
 
-      glPopAttrib();
       glPopMatrix();
 
       glEndList();
@@ -351,27 +353,27 @@ GLuint CSVGParser::ParseElement(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
  * Collects all data in an <svg:g> element.
  *
  * @return A new display list comprising all subelements.
- * @see ParseElement for the description of the parameters.
+ * @see parseElement for the description of the parameters.
  */
-GLuint CSVGParser::ParseGroup(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+GLuint CSVGParser::parseGroup(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                               float StrokeWidth, float MasterAlpha, CBoundingBoxComputer* BB)
 {
-  xmlNodePtr Child = SVG->children;
-  vector<GLuint> Lists;
-  while (Child != NULL)
+  xmlNodePtr child = SVG->children;
+  vector<GLuint> lists;
+  while (child != NULL)
   {
-    if (Child->type == XML_ELEMENT_NODE)
+    if (child->type == XML_ELEMENT_NODE)
     {
-      GLuint List = ParseElement(Child, DoFill, FillColor, DoStroke, StrokeColor, StrokeWidth, MasterAlpha, BB);
-      Lists.push_back(List);
+      GLuint List = parseElement(child, DoFill, FillColor, DoStroke, StrokeColor, StrokeWidth, MasterAlpha, BB);
+      lists.push_back(List);
     };
-    Child = Child->next;
+    child = child->next;
   };
 
   GLuint ResultList = glGenLists(1);
   glNewList(ResultList, GL_COMPILE);
-  for (vector<GLuint>::iterator Iterator = Lists.begin(); Iterator != Lists.end(); Iterator++)
-    glCallList(*Iterator);
+  for (vector<GLuint>::iterator iterator = lists.begin(); iterator != lists.end(); iterator++)
+    glCallList(*iterator);
   glEndList();
 
   return ResultList;
@@ -383,47 +385,59 @@ GLuint CSVGParser::ParseGroup(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, b
  * Parses the content of an image definition.
  *
  * @param SVG The XML SVG element containing the definition.
+ * @param Render If true then the appropriate OpenGL calls for rendering are issued, otherwise setup is performed.
  */
-void CSVGParser::ParseImage(xmlNodePtr SVG, CBoundingBoxComputer* BB)
+void CSVGParser::parseImage(xmlNodePtr SVG, CBoundingBoxComputer* BB, bool Render)
 {
-  float X = GetFloatAttributeDef(SVG, "x", 0);
-  float Y = GetFloatAttributeDef(SVG, "y", 0);
-  float Width = GetFloatAttributeDef(SVG, "width", 0);
-  float Height = GetFloatAttributeDef(SVG, "height", 0);
-  string Filename = GetStringAttributeDef(SVG, "href", "");
-  string ID = GetStringAttributeDef(SVG, "id", Filename);
+  string Filename = getStringAttributeDef(SVG, "href", "");
+  string id = getStringAttributeDef(SVG, "id", Filename);
 
-  // Update the accumulated bounding box.
-  GLfloat Matrix[16];
-  glGetFloatv(GL_MODELVIEW_MATRIX, Matrix);
-  BB->Include(Matrix, X, Y);
-  BB->Include(Matrix, X + Width, Y + Height);
-
-  // Reuse an existing image.
-  CGCTexture* Texture = TextureManager()->FindTexture(ID);
-  if (Texture == NULL)
+  if (!Render)
   {
-    TLODList LODs;
-    LODs.push_back(Filename);
+    // Create the image only if not yet done.
+    CGCTexture* Texture = textureManager()->FindTexture(id);
+    if (Texture == NULL)
+    {
+      TLODList LODs;
+      LODs.push_back(Filename);
 
-    Texture = TextureManager()->CreateTextureEntry(LODs, ID, "clamp-to-edge", "clamp-to-edge", "linear-mipmap-linear", 
-      "linear", 2, "modulate");
+      Texture = textureManager()->CreateTextureEntry(LODs, id, "clamp-to-edge", "clamp-to-edge", "linear-mipmap-linear", 
+        "linear", 2, "modulate");
+    };
+  }
+  else
+  {
+    float x = getFloatAttributeDef(SVG, "x", 0);
+    float y = getFloatAttributeDef(SVG, "y", 0);
+    float Width = getFloatAttributeDef(SVG, "width", 0);
+    float Height = getFloatAttributeDef(SVG, "height", 0);
+
+    // Update the accumulated bounding box.
+    GLfloat matrix[16];
+    glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
+    BB->include(matrix, x, y);
+    BB->include(matrix, x + Width, y + Height);
+
+    CGCTexture* Texture = textureManager()->FindTexture(id);
+    if (Texture)
+      Texture->ActivateTexture();
+
+    glColor4f(1, 1, 1, 1);
+    glTranslatef(x, y, 0);
+    glBegin(GL_POLYGON);
+      glTexCoord2d(0, 0);
+      glVertex2f(0, 0);
+      glTexCoord2d(1, 0);
+      glVertex2f(Width, 0);
+      glTexCoord2d(1, 1);
+      glVertex2f(Width, Height);
+      glTexCoord2d(0, 1);
+      glVertex2f(0, Height);
+    glEnd();
+
+    if (Texture)
+      Texture->DeactivateTexture();
   };
-  Texture->ActivateTexture();
-
-  glColor4f(1, 1, 1, 1);
-  glTranslatef(X, Y, 0);
-  glBegin(GL_POLYGON);
-    glTexCoord2d(0, 0);
-    glVertex2f(0, 0);
-    glTexCoord2d(1, 0);
-    glVertex2f(Width, 0);
-    glTexCoord2d(1, 1);
-    glVertex2f(Width, Height);
-    glTexCoord2d(0, 1);
-    glVertex2f(0, Height);
-  glEnd();
-  Texture->DeactivateTexture();
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -437,18 +451,18 @@ void CSVGParser::ParseImage(xmlNodePtr SVG, CBoundingBoxComputer* BB)
  * @param StrokeWidth The width of the line.
  * @param BB This calculator is used to determine the overall bounding box of the element being parsed.
  */
-void CSVGParser::ParseLine(xmlNodePtr SVG, bool DoStroke, GLubyte* StrokeColor, float StrokeWidth, CBoundingBoxComputer* BB)
+void CSVGParser::parseLine(xmlNodePtr SVG, bool DoStroke, GLubyte* StrokeColor, float StrokeWidth, CBoundingBoxComputer* BB)
 {
-  float X1 = GetFloatAttributeDef(SVG, "x1", 0);
-  float Y1 = GetFloatAttributeDef(SVG, "y1", 0);
-  float X2 = GetFloatAttributeDef(SVG, "x2", 0);
-  float Y2 = GetFloatAttributeDef(SVG, "y2", 0);
+  float X1 = getFloatAttributeDef(SVG, "x1", 0);
+  float Y1 = getFloatAttributeDef(SVG, "y1", 0);
+  float X2 = getFloatAttributeDef(SVG, "x2", 0);
+  float Y2 = getFloatAttributeDef(SVG, "y2", 0);
 
   // Update the accumulated bounding box.
-  GLfloat Matrix[16];
-  glGetFloatv(GL_MODELVIEW_MATRIX, Matrix);
-  BB->Include(Matrix, X1, Y1);
-  BB->Include(Matrix, X2, Y2);
+  GLfloat matrix[16];
+  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
+  BB->include(matrix, X1, Y1);
+  BB->include(matrix, X2, Y2);
 
   if (DoStroke)
   {
@@ -468,19 +482,19 @@ void CSVGParser::ParseLine(xmlNodePtr SVG, bool DoStroke, GLubyte* StrokeColor,
 /**
  * Parses the content of a polygon definition.
  *
- * @see ParseElement for a description of the parameters.
+ * @see parseElement for a description of the parameters.
  */
-void CSVGParser::ParsePolygon(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+void CSVGParser::parsePolygon(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                               float StrokeWidth, CBoundingBoxComputer* BB)
 {
   xmlChar* Raw = xmlGetProp(SVG, (xmlChar*) "points");
   if (Raw != NULL)
   {
     CVertexVector Vertices;
-    ParsePoints(Raw, &Vertices);
+    parsePoints(Raw, &Vertices);
     xmlFree(Raw);
 
-    RenderVertices(DoFill, FillColor, DoStroke, StrokeColor, Vertices, NULL, StrokeWidth, true, BB);
+    renderVertices(DoFill, FillColor, DoStroke, StrokeColor, Vertices, NULL, StrokeWidth, true, BB);
   };
 }
 
@@ -489,19 +503,19 @@ void CSVGParser::ParsePolygon(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, b
 /**
  * Parses the content of a polyline definition.
  *
- * @see ParseElement for a description of the parameters.
+ * @see parseElement for a description of the parameters.
  */
-void CSVGParser::ParsePolyline(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+void CSVGParser::parsePolyline(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                                float StrokeWidth, CBoundingBoxComputer* BB)
 {
   xmlChar* Raw = xmlGetProp(SVG, (xmlChar*) "points");
   if (Raw != NULL)
   {
     CVertexVector Vertices;
-    ParsePoints(Raw, &Vertices);
+    parsePoints(Raw, &Vertices);
     xmlFree(Raw);
 
-    RenderVertices(DoFill, FillColor, DoStroke, StrokeColor, Vertices, NULL, StrokeWidth, false, BB);
+    renderVertices(DoFill, FillColor, DoStroke, StrokeColor, Vertices, NULL, StrokeWidth, false, BB);
   };
 }
 
@@ -510,25 +524,25 @@ void CSVGParser::ParsePolyline(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
 /**
  * Parses the content of a rectangle definition.
  *
- * @see ParseElement for a description of the parameters.
+ * @see parseElement for a description of the parameters.
  */
-void CSVGParser::ParseRectangle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+void CSVGParser::parseRectangle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                                 float StrokeWidth, CBoundingBoxComputer* BB)
 {
-  float X, Y, Width, Height, RX, RY;
+  float x, y, Width, Height, RX, RY;
 
-  if (!GetFloatAttribute(SVG, "x", X))
-    X = 0;
-  if (!GetFloatAttribute(SVG, "y", Y))
-    Y = 0;
-  if (!GetFloatAttribute(SVG, "width", Width))
+  if (!getFloatAttribute(SVG, "x", x))
+    x = 0;
+  if (!getFloatAttribute(SVG, "y", y))
+    y = 0;
+  if (!getFloatAttribute(SVG, "width", Width))
     Width = 1;
-  if (!GetFloatAttribute(SVG, "height", Height))
+  if (!getFloatAttribute(SVG, "height", Height))
     Height = 1;
 
   // There are a few more things to check with the corner radii.
-  bool RXfound = GetFloatAttribute(SVG, "rx", RX);
-  bool RYfound = GetFloatAttribute(SVG, "ry", RY);
+  bool RXfound = getFloatAttribute(SVG, "rx", RX);
+  bool RYfound = getFloatAttribute(SVG, "ry", RY);
   if (!RXfound && !RYfound)
   {
     RX = 0;
@@ -549,29 +563,29 @@ void CSVGParser::ParseRectangle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
 
   // TODO: Consider rounded corners.
   CVertexVector Vertices;
-  TVertex V;
-  V.X = X;
-  V.Y = Y;
-  Vertices.push_back(V);
-  V.X = X + Width;
-  Vertices.push_back(V);
-  V.Y = Y + Height;
-  Vertices.push_back(V);
-  V.X = X;
-  Vertices.push_back(V);
+  TVertex v;
+  v.x = x;
+  v.y = y;
+  Vertices.push_back(v);
+  v.x = x + Width;
+  Vertices.push_back(v);
+  v.y = y + Height;
+  Vertices.push_back(v);
+  v.x = x;
+  Vertices.push_back(v);
 
   CVertexVector TextureCoordinates;
-  V.X = 0;
-  V.Y = 0;
-  TextureCoordinates.push_back(V);
-  V.X = Width;
-  TextureCoordinates.push_back(V);
-  V.Y = Height;
-  TextureCoordinates.push_back(V);
-  V.X = 0;
-  TextureCoordinates.push_back(V);
+  v.x = 0;
+  v.y = 0;
+  TextureCoordinates.push_back(v);
+  v.x = Width;
+  TextureCoordinates.push_back(v);
+  v.y = Height;
+  TextureCoordinates.push_back(v);
+  v.x = 0;
+  TextureCoordinates.push_back(v);
 
-  RenderVertices(DoFill, FillColor, DoStroke, StrokeColor, Vertices, &TextureCoordinates, StrokeWidth, true, BB);
+  renderVertices(DoFill, FillColor, DoStroke, StrokeColor, Vertices, &TextureCoordinates, StrokeWidth, true, BB);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -588,63 +602,67 @@ void CSVGParser::ParseRectangle(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor,
  * @param BB This calculator is used to determine the overall bounding box of the element being parsed.
  * @return A new display list comprising all subelements.
  */
-GLuint CSVGParser::ParseText(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+GLuint CSVGParser::parseText(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                              float StrokeWidth, float MasterAlpha, CBoundingBoxComputer* BB)
 {
   // Collect font information.
-  string FontFamily = GetStringAttributeDef(SVG, "font-family", DefaultFontFamily);
-  string FontStyle = GetStringAttributeDef(SVG, "font-style", DefaultFontStyle);
-  string FontWeightString = GetStringAttributeDef(SVG, "font-weight", DefaultFontWeight);
-  int Weight = ConvertFontWeight(FontWeightString);
-  int FontSize = GetIntAttributeDef(SVG, "font-size", DefaultFontSize);
+  string FontFamily = getStringAttributeDef(SVG, "font-family", DefaultFontFamily);
+  string FontStyle = getStringAttributeDef(SVG, "font-style", DefaultFontStyle);
+  string FontWeightString = getStringAttributeDef(SVG, "font-weight", DefaultFontWeight);
+  int Weight = convertFontWeight(FontWeightString);
+  int FontSize = getIntAttributeDef(SVG, "font-size", DefaultFontSize);
 
-  vector<GLuint> Lists;
-  xmlNodePtr Child = SVG->children;
-  while (Child != NULL)
+  vector<GLuint> lists;
+  xmlNodePtr child = SVG->children;
+  while (child != NULL)
   {
-    if (XML_IS(Child, "tspan"))
+    if (XML_IS(child, "tspan"))
     {
-      GLuint LocalList = ParseElement(Child, DoFill, FillColor, DoStroke, StrokeColor, StrokeWidth, MasterAlpha, BB);
-      Lists.push_back(LocalList);
+      GLuint LocalList = parseElement(child, DoFill, FillColor, DoStroke, StrokeColor, StrokeWidth, MasterAlpha, BB);
+      lists.push_back(LocalList);
     }
     else
     {
-      string Source((char*) Child->content);
-      wstring Output = Utf8ToUtf16(Source);
+      string Source((char*) child->content);
+      wstring Output = utf8ToUtf16(Source);
 
       // Make the font manager create all display lists before we build ours.
-      TBoundingBox Box;
-      FontManager()->BoundingBox(Output, FontFamily, FontStyle, Weight, FontSize, &Box);
+      TBoundingBox box;
+      fontManager()->boundingBox(Output, FontFamily, FontStyle, Weight, FontSize, &box);
 
       // Update the accumulated bounding box.
-      GLfloat Matrix[16];
-      glGetFloatv(GL_MODELVIEW_MATRIX, Matrix);
-      BB->Include(Matrix, &Box);
+      GLfloat matrix[16];
+      glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
+      BB->include(matrix, &box);
 
       GLuint LocalList = glGenLists(1);
       glNewList(LocalList, GL_COMPILE);
-        FontManager()->TextOut(Output, FontFamily, FontStyle, Weight, FontSize);
+        glColor3f(1, 0, 1);
+        // FTGL uses the lower left corner as origin, we however have the upper left corner for this task.
+        // Adjust the vertical coordinate accordingly.
+        glRasterPos2f(0, box.lower.y);
+        fontManager()->textOut(Output, FontFamily, FontStyle, Weight, FontSize);
       glEndList();
-      Lists.push_back(LocalList);
+      lists.push_back(LocalList);
     };
 
-    Child = Child->next;
+    child = child->next;
   };
 
   // Create final display list.
   GLuint DisplayList = glGenLists(1);
   glNewList(DisplayList, GL_COMPILE);
 
-  float X = GetFloatAttributeDef(SVG, "x", 0);
-  float Y = GetFloatAttributeDef(SVG, "y", 0);
+  float x = getFloatAttributeDef(SVG, "x", 0);
+  float y = getFloatAttributeDef(SVG, "y", 0);
 
-  glTranslatef(X, Y, 0);
+  glTranslatef(x, y, 0);
 
   if (FillColor !=  NULL)
     glColor4ubv(FillColor);
 
-  for (vector<GLuint>::iterator Iterator = Lists.begin(); Iterator != Lists.end(); Iterator++)
-    glCallList(*Iterator);
+  for (vector<GLuint>::iterator iterator = lists.begin(); iterator != lists.end(); iterator++)
+    glCallList(*iterator);
   glEndList();
 
   return DisplayList;
@@ -658,16 +676,16 @@ GLuint CSVGParser::ParseText(xmlNodePtr SVG, bool DoFill, GLubyte* FillColor, bo
  *
  * @param Transformation The textual representation of the transformation to parse and perform.
  */
-void CSVGParser::ParseTransformation(char* Transformation)
+void CSVGParser::parseTransformation(char* Transformation)
 {
   string Delimiters(" ,()\t\n");
-  StringTokenizer Tokenizer(Transformation, Delimiters);
+  StringTokenizer tokenizer(Transformation, Delimiters);
 
   // Each run in the outer loop is one transformation.
-  while (Tokenizer.HasMoreTokens())
+  while (tokenizer.hasMoreTokens())
   {
     // Skip leading white spaces.
-    string Token = Tokenizer.NextToken();
+    string Token = tokenizer.nextToken();
 
     int Type = 0;
     if (Token == "matrix")
@@ -692,14 +710,14 @@ void CSVGParser::ParseTransformation(char* Transformation)
 
     switch (Type)
     {
-      case 1: // Matrix
+      case 1: // matrix
         {
           // Argument list found. Read 6 float numbers.
           // | a  d  0  0 |
           // | b  e  0  0 |
           // | 0  0  1  0 |
           // | c  f  0  1 |
-          GLfloat Matrix[16] = {
+          GLfloat matrix[16] = {
             1, 0, 0, 0, 
             0, 1, 0, 0, 
             0, 0, 1, 0, 
@@ -708,20 +726,20 @@ void CSVGParser::ParseTransformation(char* Transformation)
 
           for (int J = 0; J < 2; J++)
             for (int I = 0; I < 2; I++)
-              Matrix[J * 4 + I] = Tokenizer.NextTokenAsFloat();
-          Matrix[12] = Tokenizer.NextTokenAsFloat();
-          Matrix[13] = Tokenizer.NextTokenAsFloat();
-          glMultMatrixf(Matrix);
+              matrix[J * 4 + I] = tokenizer.nextTokenAsFloat();
+          matrix[12] = tokenizer.nextTokenAsFloat();
+          matrix[13] = tokenizer.nextTokenAsFloat();
+          glMultMatrixf(matrix);
 
           break;
         };
       case 2: // Translation
         {
           // One or two floats are possible here.
-          float TX = Tokenizer.NextTokenAsFloat();
+          float TX = tokenizer.nextTokenAsFloat();
           float TY = 0;
-          if (Tokenizer.HasMoreTokens())
-            TY = Tokenizer.NextTokenAsFloat();
+          if (tokenizer.hasMoreTokens())
+            TY = tokenizer.nextTokenAsFloat();
           glTranslatef(TX, TY, 0);
 
           break;
@@ -729,10 +747,10 @@ void CSVGParser::ParseTransformation(char* Transformation)
       case 3: // Scaling
         {
           // One or two floats are possible here.
-          float SX = Tokenizer.NextTokenAsFloat();
+          float SX = tokenizer.nextTokenAsFloat();
           float SY = SX;
-          if (Tokenizer.HasMoreTokens())
-            SY = Tokenizer.NextTokenAsFloat();
+          if (tokenizer.hasMoreTokens())
+            SY = tokenizer.nextTokenAsFloat();
           glScalef(SX, SY, 0);
 
           break;
@@ -740,26 +758,26 @@ void CSVGParser::ParseTransformation(char* Transformation)
       case 4: // Rotation
         {
           // An angle (in degrees) and an optional rotation point are possible here.
-          float Angle = Tokenizer.NextTokenAsFloat();
-          float X = 0;
-          float Y = 0;
-          if (Tokenizer.HasMoreTokens())
+          float Angle = tokenizer.nextTokenAsFloat();
+          float x = 0;
+          float y = 0;
+          if (tokenizer.hasMoreTokens())
           {
-            X = Tokenizer.NextTokenAsFloat();
-            Y = Tokenizer.NextTokenAsFloat();
+            x = tokenizer.nextTokenAsFloat();
+            y = tokenizer.nextTokenAsFloat();
           };
 
-          glTranslatef(X, Y, 0);
+          glTranslatef(x, y, 0);
           glRotatef(Angle, 0, 0, 1);
-          glTranslatef(-X, -Y, 0);
+          glTranslatef(-x, -y, 0);
 
           break;
         };
-      case 5: // X Skewing
-      case 6: // Y Skewing
+      case 5: // x Skewing
+      case 6: // y Skewing
         {
           // Only one value is expected, which gives the skewing angle in degrees.
-          GLdouble Matrix[16] = {
+          GLdouble matrix[16] = {
             1, 0, 0, 0, 
             0, 1, 0, 0, 
             0, 0, 1, 0, 
@@ -767,8 +785,8 @@ void CSVGParser::ParseTransformation(char* Transformation)
           };
 
           int Index = (Type == 5) ? 4 : 1;
-          Matrix[Index] = tan(Tokenizer.NextTokenAsFloat() * M_PI / 180);
-          glMultMatrixd(Matrix);
+          matrix[Index] = tan(tokenizer.nextTokenAsFloat() * M_PI / 180);
+          glMultMatrixd(matrix);
           break;
         };
     };
@@ -792,14 +810,12 @@ void CSVGParser::ParseTransformation(char* Transformation)
  *                   connected to the first point).
  * @param BB This calculator is used to determine the overall bounding box of the element being parsed.
  */
-void CSVGParser::RenderVertices(bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
+void CSVGParser::renderVertices(bool DoFill, GLubyte* FillColor, bool DoStroke, GLubyte* StrokeColor, 
                                 const CVertexVector& Vertices, CVertexVector* TextureCoordinates, float StrokeWidth, 
                                 bool CloseShape, CBoundingBoxComputer* BB)
 {
-  glDisable(GL_POLYGON_SMOOTH);
-
-  GLfloat Matrix[16];
-  glGetFloatv(GL_MODELVIEW_MATRIX, Matrix);
+  GLfloat matrix[16];
+  glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
 
   if (DoStroke)
   {
@@ -809,7 +825,7 @@ void CSVGParser::RenderVertices(bool DoFill, GLubyte* FillColor, bool DoStroke,
     {
       glBegin(CloseShape ? GL_LINE_LOOP : GL_LINE_STRIP);
       for (int I = 0; I < (int) Vertices.size(); I++)
-        glVertex2f(Vertices[I].X, Vertices[I].Y);
+        glVertex2f(Vertices[I].x, Vertices[I].y);
       glEnd();
     }
     else
@@ -826,26 +842,26 @@ void CSVGParser::RenderVertices(bool DoFill, GLubyte* FillColor, bool DoStroke,
         if (Index == Vertices.size() - 1)
           NextIndex = 0;
         // We need the angle of the current line relative to the horizontal axis.
-        float dX = Vertices[NextIndex].X - Vertices[Index].X;
-        float dY = Vertices[NextIndex].Y - Vertices[Index].Y;
+        float dX = Vertices[NextIndex].x - Vertices[Index].x;
+        float dY = Vertices[NextIndex].y - Vertices[Index].y;
         float Angle = atan2(dY, dX);
 
         // Compute the four corners for the rectangle. We can use symmetry to speed up computation.
         dX = HalfWidth * sin(Angle);
         dY = HalfWidth * cos(Angle);
         glBegin(GL_POLYGON);
-          glVertex2f(Vertices[Index].X - dX, Vertices[Index].Y + dY);
+          glVertex2f(Vertices[Index].x - dX, Vertices[Index].y + dY);
           // Update the accumulated bounding box.
-          BB->Include(Matrix, Vertices[Index]);
+          BB->include(matrix, Vertices[Index]);
 
-          glVertex2f(Vertices[NextIndex].X - dX, Vertices[NextIndex].Y + dY);
-          BB->Include(Matrix, Vertices[Index]);
+          glVertex2f(Vertices[NextIndex].x - dX, Vertices[NextIndex].y + dY);
+          BB->include(matrix, Vertices[Index]);
 
-          glVertex2f(Vertices[NextIndex].X + dX, Vertices[NextIndex].Y - dY);
-          BB->Include(Matrix, Vertices[Index]);
+          glVertex2f(Vertices[NextIndex].x + dX, Vertices[NextIndex].y - dY);
+          BB->include(matrix, Vertices[Index]);
 
-          glVertex2f(Vertices[Index].X + dX, Vertices[Index].Y - dY);
-          BB->Include(Matrix, Vertices[Index]);
+          glVertex2f(Vertices[Index].x + dX, Vertices[Index].y - dY);
+          BB->include(matrix, Vertices[Index]);
         glEnd();
 
         Index++;
@@ -862,11 +878,11 @@ void CSVGParser::RenderVertices(bool DoFill, GLubyte* FillColor, bool DoStroke,
     for (int I = 0; I < (int) Vertices.size(); I++)
     {
       if (TextureCoordinates != NULL)
-        glTexCoord2d((*TextureCoordinates)[I].X, (*TextureCoordinates)[I].Y);
-      glVertex2f(Vertices[I].X, Vertices[I].Y);
+        glTexCoord2d((*TextureCoordinates)[I].x, (*TextureCoordinates)[I].y);
+      glVertex2f(Vertices[I].x, Vertices[I].y);
 
       // Update the accumulated bounding box.
-      BB->Include(Matrix, Vertices[I]);
+      BB->include(matrix, Vertices[I]);
     };
     glEnd();
   };
@@ -885,18 +901,18 @@ void CSVGParser::RenderVertices(bool DoFill, GLubyte* FillColor, bool DoStroke,
  *       height attribute is needed to convert from SVG's top-down to OpenGL's bottom-up coordinate system.
  *       Without height attribute the element will be drawn head-down.
  */
-void CSVGParser::Convert(xmlNodePtr SVG, GLuint DisplayList, CBoundingBoxComputer* BB)
+void CSVGParser::convert(xmlNodePtr SVG, GLuint DisplayList, CBoundingBoxComputer* BB)
 {
-  vector<GLuint> Lists;
-  xmlNodePtr Child = SVG->children;
-  while (Child != NULL)
+  vector<GLuint> lists;
+  xmlNodePtr child = SVG->children;
+  while (child != NULL)
   {
-    if (Child->type == XML_ELEMENT_NODE)
+    if (child->type == XML_ELEMENT_NODE)
     {
-      GLuint LocalList = ParseElement(Child, false, NULL, false, NULL, 1, 1, BB);
-      Lists.push_back(LocalList);
+      GLuint LocalList = parseElement(child, false, NULL, false, NULL, 1, 1, BB);
+      lists.push_back(LocalList);
     };
-    Child = Child->next;
+    child = child->next;
   };
 
   // Not necessary for creating the list but used to create a clean starting point to transform bounding boxes.
@@ -906,8 +922,8 @@ void CSVGParser::Convert(xmlNodePtr SVG, GLuint DisplayList, CBoundingBoxCompute
 
   glColor4f(0, 0, 0, 1);
 
-  for (vector<GLuint>::iterator Iterator = Lists.begin(); Iterator != Lists.end(); Iterator++)
-    glCallList(*Iterator);
+  for (vector<GLuint>::iterator iterator = lists.begin(); iterator != lists.end(); iterator++)
+    glCallList(*iterator);
   glEndList();
 }
 
@@ -919,16 +935,16 @@ void CSVGParser::Convert(xmlNodePtr SVG, GLuint DisplayList, CBoundingBoxCompute
  * @param Definition The definition to parse.
  * @Model The model to which the new style is to be added.
  */
-void CSVGParser::ParseDefinition(xmlNodePtr Definition, CGCModel* Model)
+void CSVGParser::parseDefinition(xmlNodePtr Definition, CGCModel* Model)
 {
-  string ID; 
-  if (GetStringAttribute(Definition, "id", ID))
+  string id; 
+  if (getStringAttribute(Definition, "id", id))
   {
     // Handle sub entries of the layout definition.
     xmlNodePtr Element = Definition->children;
 
     // This call will create an entry if there is none yet.
-    CGCStyle* Style = Model->Style(Utf8ToUtf16(ID));
+    CGCStyle* Style = Model->style(utf8ToUtf16(id));
     if (Style->FDisplayList == 0)
       Style->FDisplayList = glGenLists(1);
     while (Element != NULL)
@@ -938,9 +954,9 @@ void CSVGParser::ParseDefinition(xmlNodePtr Definition, CGCModel* Model)
         if (XML_IS(Element, "svg"))
         {
           CBoundingBoxComputer BBComputer;
-          Convert(Element, Style->FDisplayList, &BBComputer);
-          Style->FBoundingBox = BBComputer.BoundingBox();
-          Model->CheckError();
+          convert(Element, Style->FDisplayList, &BBComputer);
+          Style->FBoundingBox = BBComputer.boundingBox();
+          Model->checkError();
         };
 
         // Ignore everything else.
