@@ -17,6 +17,7 @@
 #ifndef __MYX_GRT_H__
 #define __MYX_GRT_H__
 
+#define GRT_VERSION "2.0.6 beta"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 # define __LCC__
@@ -55,6 +56,13 @@ extern "C" {
 /// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_public_interface"
 /// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library\include\myx_public_interface.h"
 
+
+/**
+ * Constants and defines
+ */
+
+#define MAX_NESTING 100 // The maximum number of path parts in a GRT path.
+
 /*
  * Enums
  */
@@ -80,8 +88,23 @@ typedef enum
   MYX_GRT_JAVA_NOT_FOUND,
   MYX_GRT_JAVA_REGISTRY_CORRUPTED,
   MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED
+
 } MYX_GRT_ERROR;
 
+typedef enum
+{
+  MYX_GRTA_OK= 0,
+  MYX_GRTA_BUSY,
+  MYX_GRTA_EXECUTING,
+  MYX_GRTA_FINISHED,
+  MYX_GRTA_CANCELLED,
+  MYX_GRTA_ERROR,
+  
+  MYX_GRTA_UNREACHABLE,
+  MYX_GRTA_REMOTE_ERROR
+
+} MYX_GRT_AGENT_STATUS;
+  
 typedef enum 
 {
   MYX_ANY_VALUE= 0,
@@ -188,6 +211,8 @@ typedef struct MYX_GRT_STRUCT
   char *caption;
   char *desc;
 
+  char *bridge;
+
   unsigned int members_num;
   MYX_GRT_STRUCT_MEMBER *members; // list of members
 } MYX_GRT_STRUCT;
@@ -203,9 +228,9 @@ typedef struct
   MYX_GRT_VALUE_TYPE content_type; /* only used if this is a typed list, meaning it can only hold values of the given type */
   char *content_struct_name; /* only if content_type == dict, used to make sure only dicts of the given struct can be added */
 
-  MYX_GRT *grt; // the grt member is only set if this is a proxy list
-  MYX_GRT_MODULE *proxy_module; // if set, this list becomes a proxy list and it's items are returned by the specified module
-  char *proxy_path; // only set if this is a proxy list
+  //MYX_GRT *grt; // the grt member is only set if this is a proxy list
+  //MYX_GRT_MODULE *proxy_module; // if set, this list becomes a proxy list and it's items are returned by the specified module
+  //char *proxy_path; // only set if this is a proxy list
 
   unsigned int items_num;
   struct MYX_GRT_VALUE **items;
@@ -220,14 +245,14 @@ typedef struct
 typedef struct
 {
   char *struct_name;
-  MYX_GRT *grt; // the grt member is only set if this struct has been added to the GRT reference cache or this is a proxy object
+  MYX_GRT *grt; // the grt member is only set if this struct has been added to the GRT reference cache
   char *object_path; // the path is only set if this struct has been added to the GRT reference cache
 
   MYX_GRT_VALUE_TYPE content_type; // only used if this is a typed dict, meaning it can only hold values of the given type 
   char *content_struct_name; // only if content_type == dict, used to make sure only dicts of the given struct can be added 
 
-  MYX_GRT_MODULE *proxy_module; // if set, this dict becomes a proxy object and it's items are returned by the specified module
-  char *proxy_path; // only set if this is a proxy object
+  //MYX_GRT_MODULE *proxy_module; // if set, this dict becomes a proxy object and it's items are returned by the specified module
+  //char *proxy_path; // only set if this is a proxy object
 
   unsigned int items_num;
   MYX_GRT_DICT_ITEM *items; // this must be in strcmp() order of name
@@ -245,6 +270,13 @@ typedef struct MYX_GRT_VALUE
     MYX_GRT_LIST *l;
     MYX_GRT_DICT *d;
   } value;
+
+  MYX_GRT_MODULE *bridge_module; // if set, callbacks to the bridge are made if the value is changed
+  void *bridge_data_object; // data set by the bridge so it is possible to associate bridged objects with GRT values
+  void *bridge_data_owner; // data set by the bridge so it is possible to associate bridged objects with GRT values
+  char *bridge_dict_key; // if this is a dict item, store the key name so the bridge can identify the property associated with the value
+  unsigned int bridge_list_index; // if this is a list item, store the index so the bridge can identify the item associated with the value
+
 } MYX_GRT_VALUE;
   
 
@@ -290,12 +322,22 @@ typedef enum
 //    MYX_PERL_MODULE_TYPE,
 } MYX_GRT_MODULE_TYPE;
 
+typedef enum
+{
+  MYX_GRT_IO_STANDARD = 0,
+  MYX_GR_IO_PASSWORD = 1
+} MYX_GRT_INPUT_OPTIONS;
+
+  
+typedef struct MYX_GRT_AGENT_SESSION MYX_GRT_AGENT_SESSION;
+  
 /*
  * Functions
  */
 
 typedef void (*MYX_GRT_PRINT_CALLBACK)(const char *, void*);
 typedef int (*MYX_GRT_MESSAGE_CALLBACK)(MYX_GRT_MSGS *msgs, void *user_data);
+typedef int (*MYX_GRT_INPUT_CALLBACK)(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data);
 typedef void (*MYX_GRT_LOG_CALLBACK)(MYX_GRT *, int, const char *, const char*);
 
 MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
@@ -308,6 +350,8 @@ JNIEXPORT jstring JNICALL Java_Grt_callGrtFunction
   (JNIEnv *, jobject, jlong, jstring, jstring, jstring);
 #endif
 
+const char *myx_grt_error_string(MYX_GRT_ERROR error);
+
 // ----------------------------------------------------------------------------------------------------------------------
 // Callbacks
 
@@ -315,10 +359,15 @@ MYX_PUBLIC_FUNC void myx_grt_set_output_callback(MYX_GRT *grt, void *user_data,
                                  void (*process_output_func)(const char *text, void *user_data));
 MYX_PUBLIC_FUNC void myx_grt_set_message_callback(MYX_GRT *grt, void *user_data,
                                  int (*process_message_func)(MYX_GRT_MSGS *msgs, void *user_data));
+MYX_PUBLIC_FUNC void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
+                                 int (*process_input_func)(const char *caption, MYX_GRT_INPUT_OPTIONS options, const char **text, void *user_data));
+
 
 // ----------------------------------------------------------------------------------------------------------------------
 // GRT global functions
 
+MYX_PUBLIC_FUNC const char * myx_grt_version();
+
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_get_root(MYX_GRT *grt);
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root);
 
@@ -331,7 +380,7 @@ MYX_PUBLIC_FUNC void myx_grt_reference_cache_rescan(MYX_GRT *grt);
 MYX_PUBLIC_FUNC MYX_GRT_MSGS * myx_grt_messages_convert(MYX_GRT_VALUE *msgs_list);
 MYX_PUBLIC_FUNC void myx_grt_messages_free(MYX_GRT_MSGS *msgs);
 
-MYX_PUBLIC_FUNC void myx_grt_messages_stack_add(MYX_GRT *grt, int msg_type, const char *message, 
+MYX_PUBLIC_FUNC void myx_grt_messages_stack_add(MYX_GRT *grt, int msg_type, const char *message,
                                           MYX_STRINGLIST *details, int copy_details, int progress);
 MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_message(MYX_GRT *grt, const char *message, 
                                           MYX_STRINGLIST *details, int copy_details, ...);
@@ -395,6 +444,30 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_create_error_result(const char
 MYX_PUBLIC_FUNC char * myx_grt_function_check_error(MYX_GRT_VALUE *res, int allow_null_as_result);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_function_extract_value_from_result(MYX_GRT_VALUE *result);
 
+// ----------------------------------------------------------------------------------------------------------------------
+// GRT Remote Agents
+
+// Server Side
+MYX_PUBLIC_FUNC void myx_grt_start_agent(MYX_GRT *grt, int port,
+                                         char **allowed_modules, unsigned int allowed_modules_num);
+
+// Client Side
+MYX_PUBLIC_FUNC MYX_GRT_AGENT_SESSION * myx_grt_remote_connect(const char *host, int port,
+                                              const char *passkey);
+MYX_PUBLIC_FUNC MYX_GRT_AGENT_STATUS myx_grt_remote_function_invoke(MYX_GRT_AGENT_SESSION *sess,
+                                                    const char *module, const char *function_name,
+                                                    MYX_GRT_VALUE *argument);
+MYX_PUBLIC_FUNC MYX_GRT_AGENT_STATUS myx_grt_remote_function_check(MYX_GRT_AGENT_SESSION *sess);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE *myx_grt_remote_function_finish(MYX_GRT_AGENT_SESSION *sess,
+                                                              MYX_GRT_ERROR *error,
+                                                              MYX_GRT_AGENT_STATUS *status);
+MYX_PUBLIC_FUNC MYX_GRT_AGENT_STATUS myx_grt_remote_function_cancel(MYX_GRT_AGENT_SESSION *sess);
+MYX_PUBLIC_FUNC MYX_GRT_MSGS * myx_grt_remote_get_messages(MYX_GRT_AGENT_SESSION *sess);
+MYX_GRT_VALUE *myx_grt_remote_get_tree(MYX_GRT_AGENT_SESSION *sess, MYX_GRT_AGENT_STATUS *status);
+void myx_grt_remote_set_tree(MYX_GRT_AGENT_SESSION *sess, MYX_GRT_VALUE *tree, MYX_GRT_AGENT_STATUS *status);
+MYX_PUBLIC_FUNC int myx_grt_remote_session_close(MYX_GRT_AGENT_SESSION *sess);
+
+
 // ----------------------------------------------------------------------------------------------------------------------
 // GRT Structs
 
@@ -432,7 +505,11 @@ MYX_PUBLIC_FUNC const char * myx_grt_struct_get_parent_name(MYX_GRT_STRUCT *gstr
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_caption(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct, int *inherited_caption);
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_desc(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
 
+MYX_PUBLIC_FUNC const char * myx_grt_struct_get_bridge_name(MYX_GRT_STRUCT *gstruct);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_grt_struct_get_bridge(MYX_GRT *grt, MYX_GRT_STRUCT *gstruct);
+
 MYX_PUBLIC_FUNC int myx_grt_struct_check_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
+MYX_PUBLIC_FUNC int myx_grt_struct_check_is_or_inherits_from(MYX_GRT *grt, const char *struct_name, const char *parent_name);
 
 MYX_PUBLIC_FUNC const char * myx_grt_struct_get_icon(MYX_GRT *grt, const char *source_path, 
                  MYX_GRT_STRUCT *gstruct, MYX_ICON_TYPE icon_type, unsigned int *length);
@@ -479,9 +556,9 @@ MYX_PUBLIC_FUNC int myx_grt_value_get_current_reference_count(MYX_GRT_VALUE *val
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_dup(MYX_GRT_VALUE *value);
   
 MYX_PUBLIC_FUNC char * myx_grt_value_to_xml(MYX_GRT_VALUE *value);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml(const char *str, size_t size);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_xml(MYX_GRT *grt, const char *str, size_t size);
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_retrieve_from_file(const char *filename);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_retrieve_from_file(MYX_GRT *grt, const char *filename);
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename);
 
 MYX_PUBLIC_FUNC char * myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *value);
@@ -491,7 +568,7 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_value_get_type(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_get_value_type_as_string(MYX_GRT_VALUE_TYPE value_type);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_get_value_type_from_string(const char *value_type_name, MYX_GRT_ERROR *error);
 MYX_PUBLIC_FUNC int myx_grt_value_is_simple_type(MYX_GRT_VALUE *value);
-MYX_PUBLIC_FUNC int myx_grt_value_is_proxy_object(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_value_is_bridged(MYX_GRT_VALUE *value);
 
 MYX_PUBLIC_FUNC int myx_grt_value_as_int(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC double myx_grt_value_as_real(MYX_GRT_VALUE *value);
@@ -504,19 +581,16 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_int(int i);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_real(double d);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_from_string(const char *s);
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_change_int(MYX_GRT_VALUE *value, int i);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_change_real(MYX_GRT_VALUE *value, double d);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_change_string(MYX_GRT_VALUE *value, const char *s);
-
 MYX_PUBLIC_FUNC int myx_grt_value_print(MYX_GRT *grt, MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC char * myx_grt_value_as_lua_code(MYX_GRT_VALUE *value, int depth);
 
 // ----------------------------------------------------------------------------------------------------------------------
 // GRT List Values
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_new(MYX_GRT_VALUE_TYPE content_type, const char *struct_name);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_create_from_stringlist(MYX_STRINGLIST *sl);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_new_proxy(MYX_GRT *grt, MYX_GRT_MODULE *proxy_module, const char *proxy_path, 
-                                                       MYX_GRT_VALUE_TYPE content_type, const char *struct_name);
+/*MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_new_proxy(MYX_GRT *grt, MYX_GRT_MODULE *proxy_module, const char *proxy_path, 
+                                                       MYX_GRT_VALUE_TYPE content_type, const char *struct_name);*/
 
 
 MYX_PUBLIC_FUNC MYX_STRINGLIST * myx_grt_list_as_stringlist(MYX_GRT_VALUE *list);
@@ -536,7 +610,7 @@ MYX_PUBLIC_FUNC unsigned int myx_grt_list_item_count(MYX_GRT_VALUE *list);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get(MYX_GRT_VALUE *list, unsigned int index);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_list_item_get_reference_value(MYX_GRT *grt, MYX_GRT_VALUE *list, unsigned int index);
 MYX_PUBLIC_FUNC const char * myx_grt_list_item_get_as_string(MYX_GRT_VALUE *list, unsigned int index);
-MYX_PUBLIC_FUNC int myx_grt_list_set_item(MYX_GRT_VALUE *list, unsigned int index, MYX_GRT_VALUE *new_value);
+MYX_PUBLIC_FUNC int myx_grt_list_item_set(MYX_GRT_VALUE *list, unsigned int index, MYX_GRT_VALUE *value);
 
 // Struct/content handling
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_list_content_get_type(MYX_GRT_VALUE *list);
@@ -548,16 +622,14 @@ MYX_PUBLIC_FUNC int myx_grt_list_content_set_struct_name(MYX_GRT_VALUE *list, co
 // ----------------------------------------------------------------------------------------------------------------------
 // GRT Dictionary Values
 
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new(const char *struct_name);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new(MYX_GRT *grt, const char *struct_name);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new_typed(MYX_GRT_VALUE_TYPE content_type, const char *content_struct_name);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new_proxy(MYX_GRT *grt, MYX_GRT_MODULE *proxy_module, 
-                                                       const char *proxy_path, const char *struct_name);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new_typed_proxy(MYX_GRT *grt, MYX_GRT_MODULE *proxy_module, const char *proxy_path, 
-                                                             MYX_GRT_VALUE_TYPE content_type, const char *content_struct_name);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_create(const char *struct_name, const char *key, ...);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_create(MYX_GRT *grt, const char *struct_name, const char *key, ...);
 //MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_create_from_list(MYX_GRT_VALUE *list);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new_obj(MYX_GRT *grt, const char *struct_name, 
-                                                 const char *name, const char *_id, const char *owner);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_new_obj(MYX_GRT *grt, const char *struct_name, const char *name, 
+                                                     const char *_id, const char *owner);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_init_obj(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *name, 
+                                                      const char *_id, const char *owner);
 
 // Item set functions
 MYX_PUBLIC_FUNC int myx_grt_dict_item_set_value(MYX_GRT_VALUE *dict, const char *key, MYX_GRT_VALUE *value);
@@ -584,6 +656,7 @@ MYX_PUBLIC_FUNC int myx_grt_dict_item_by_index(MYX_GRT_VALUE *dict, unsigned int
                                           const char **retkey, MYX_GRT_VALUE **retvalue);
 
 // Items, by path
+MYX_PUBLIC_FUNC char * myx_get_parent_path(const char *path);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_item_get_by_path(MYX_GRT_VALUE *dict, const char *path);
 MYX_PUBLIC_FUNC int myx_grt_dict_item_set_by_path(MYX_GRT_VALUE *dict, const char *path, MYX_GRT_VALUE *new_value);
 
@@ -601,7 +674,8 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_dict_generate_id(MYX_GRT_VALUE *dict);
 // Struct/content handling
 MYX_PUBLIC_FUNC const char * myx_grt_dict_struct_get_name(MYX_GRT_VALUE *dict);
 MYX_PUBLIC_FUNC MYX_GRT_STRUCT * myx_grt_dict_struct_get(MYX_GRT *grt, MYX_GRT_VALUE *dict);
-MYX_PUBLIC_FUNC int myx_grt_dict_struct_set_name(MYX_GRT_VALUE *dict, const char *struct_name);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_set_name(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *struct_name);
+MYX_PUBLIC_FUNC int myx_grt_dict_struct_set_name_no_cache_register(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *struct_name);
 MYX_PUBLIC_FUNC int myx_grt_dict_struct_validate(MYX_GRT *grt, MYX_GRT_VALUE *value, const char *struct_name, int strict);
 
 MYX_PUBLIC_FUNC int myx_grt_dict_struct_inherits_from(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *parent_struct_name);
@@ -610,9 +684,49 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_dict_content_get_type(MYX_GRT_VALUE *
 MYX_PUBLIC_FUNC const char * myx_grt_dict_content_get_struct_name(MYX_GRT_VALUE *dict);
 
 MYX_PUBLIC_FUNC void myx_grt_dict_content_set_type(MYX_GRT_VALUE *dict, MYX_GRT_VALUE_TYPE content_type);
+MYX_PUBLIC_FUNC void myx_grt_dict_content_set_struct_name(MYX_GRT_VALUE *dict, const char *struct_name);
 
 MYX_PUBLIC_FUNC const char * myx_grt_dict_get_object_path(MYX_GRT_VALUE *dict);
 
+// ----------------------------------------------------------------------------------------------------------------------
+// Bridge functions
+
+MYX_PUBLIC_FUNC void myx_grt_value_bridge_module_set(MYX_GRT_VALUE *value, MYX_GRT_MODULE *module);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_grt_value_bridge_module_get(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC void myx_grt_value_bridge_data_object_set(MYX_GRT_VALUE *value, void *data);
+MYX_PUBLIC_FUNC void * myx_grt_value_bridge_data_object_get(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC void myx_grt_value_bridge_data_owner_set(MYX_GRT_VALUE *value, void *data);
+MYX_PUBLIC_FUNC void * myx_grt_value_bridge_data_owner_get(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC void myx_grt_value_bridge_dict_key_set(MYX_GRT_VALUE *value, const char *key);
+MYX_PUBLIC_FUNC const char * myx_grt_value_bridge_dict_key_get(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC void myx_grt_value_bridge_list_index_set(MYX_GRT_VALUE *value, int index);
+MYX_PUBLIC_FUNC int myx_grt_value_bridge_list_index_get(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_value_change_int(MYX_GRT_VALUE *value, int i);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_value_change_real(MYX_GRT_VALUE *value, double d);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_value_change_string(MYX_GRT_VALUE *value, const char *s);
+
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value(MYX_GRT_VALUE *dict, const char *key, MYX_GRT_VALUE *value, int do_bridge_callback);
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_string(MYX_GRT_VALUE *dict, const char *key, const char *s, int do_bridge_callback);
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_int(MYX_GRT_VALUE *dict, const char *key, int i, int do_bridge_callback);
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_item_set_value_from_real(MYX_GRT_VALUE *dict, const char *key, double d, int do_bridge_callback);
+
+MYX_PUBLIC_FUNC int myx_grt_bridge_value_as_int(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC double myx_grt_bridge_value_as_real(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC const char * myx_grt_bridge_value_as_string(MYX_GRT_VALUE *value);
+
+MYX_PUBLIC_FUNC unsigned int myx_grt_bridge_list_item_count(MYX_GRT_VALUE *list);
+MYX_PUBLIC_FUNC int myx_grt_bridge_list_item_insert(MYX_GRT_VALUE *list, int index, MYX_GRT_VALUE *value, int do_bridge_callback);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_list_item_get(MYX_GRT_VALUE *list, unsigned int index, int do_bridge_callback);
+MYX_PUBLIC_FUNC int myx_grt_bridge_list_item_del(MYX_GRT_VALUE *list, int index, int do_bridge_callback);
+
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_bridge_dict_new(MYX_GRT *grt, const char *struct_name, void *bridge_data);
+MYX_PUBLIC_FUNC int myx_grt_bridge_dict_struct_set_name(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *struct_name, 
+                                                        int register_in_cache, void *bridge_data);
 
 // ----------------------------------------------------------------------------------------------------------------------
 // Java module
@@ -663,6 +777,7 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE *make_return_value_error(const char *message, cons
 MYX_PUBLIC_FUNC char * myx_grt_get_guid();
 
 
+
 #ifdef __cplusplus
 }
 #endif
