@@ -117,7 +117,7 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
       myx_grt_messages_stack_add_error(grt, "The JRE 1.4 cannot be found." _br
         "Please make sure the Sun JRE 1.4 is installed correctly." _br
         "Verify the following registry key:" _br
-        "HKEY_LOCAL_MACHINE\\SOFTWARE\\JavaSoft\\Java Runtime Environment\\1.4\\RuntimeLib", 
+        "HKEY_LOCAL_MACHINE\\SOFTWARE\\JavaSoft\\Java Runtime Environment\\1.4\\RuntimeLib",
                                        NULL, 0, 0);
 
       goto errorfree;
@@ -146,7 +146,7 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   g_dir_close(dir);
 
   //base class path
-  jvm_options[0].optionString= g_strdup_printf("-Djava.class.path=%s%s", grt_class_path, JAVA_PATH_SEPARATOR);
+  jvm_options[0].optionString= g_strdup_printf("-Djava.class.path=.%s%s%s", JAVA_PATH_SEPARATOR, grt_class_path, JAVA_PATH_SEPARATOR);
 
   //add library path to class path
   if (grt_class_path_libraries)
@@ -286,7 +286,15 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   }
 
   (*priv->env)->CallVoidMethodA(priv->env, grt_object, setCallback_methodID, args);
-
+  if ((*priv->env)->ExceptionOccurred(priv->env))
+  {
+    (*priv->env)->ExceptionDescribe(priv->env);
+    (*priv->env)->ExceptionClear(priv->env);
+    
+    myx_grt_messages_stack_add_error(grt, "An exception occurred while setting up a native callback. Please check if all required libraries are present.",
+                                     NULL, 0);
+    goto error;
+  }
   g_free(args);
   args= NULL;
 
@@ -715,7 +723,7 @@ static MYX_GRT_ERROR java_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALU
   args[1].l= (*env)->NewStringUTF(env, function->name);
   args[2].l= (*env)->NewStringUTF(env, jfunc->java_signature);
   args[3].l= j_xml_object_from_grt_value(env, value);
-
+  
   res= (*env)->CallStaticObjectMethodA(env, jloader->grt_class, jloader->grt_call_func, args);
   if (res)
   {
@@ -848,7 +856,7 @@ static MYX_GRT_VALUE *j_grt_value_from_java_xml_object(MYX_GRT *grt, JNIEnv *env
     if (grt)
       value= myx_grt_value_from_xml_global_object(grt, jstr, strlen(jstr));
     else
-      value= myx_grt_value_from_xml(jstr, strlen(jstr));
+      value= myx_grt_value_from_xml(grt, jstr, strlen(jstr));
 
     (*env)->ReleaseStringUTFChars(env, strobj, jstr);
   }
@@ -1133,7 +1141,7 @@ static MYX_GRT_VALUE *java_object_to_global_grt_value(MYX_GRT *grt, JNIEnv *env,
     fprintf(f, "struct_name: %s" _br, struct_name);*/
 
     // Create object
-    res= myx_grt_dict_new(struct_name);
+    res= myx_grt_dict_new(grt, struct_name);
 
     gstruct= myx_grt_struct_get(grt, struct_name);
 
