@@ -23,9 +23,25 @@
 #include "myx_aux_functions.h"
 
 
+static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const char *assets_name, void *data);
+
+static char *get_sql_create_script_assets(MYX_GRT_VALUE *schema, const char *assets_name);
+
+static char * get_sql_create_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema);
+static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table);
+static char * get_sql_create_view(MYX_GRT *grt, MYX_GRT_VALUE *view);
+static char * get_sql_create_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure);
+
+static char * get_sql_drop_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema);
+static char * get_sql_drop_table(MYX_GRT *grt, MYX_GRT_VALUE *table);
+static char * get_sql_drop_view(MYX_GRT *grt, MYX_GRT_VALUE *view);
+static char * get_sql_drop_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure);
+
+
+
 MYX_GRT_VALUE *generate_sql_create_statements(MYX_GRT_VALUE *param, void *data)
 {
-  MYX_GRT *grt= (MYX_GRT *) data;
+//  MYX_GRT *grt= (MYX_GRT *) data;
 
   if (!param)
     return myx_grt_function_create_error_result("You have to submit one object as parameter.", NULL);
@@ -79,7 +95,7 @@ static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const c
 
 MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
 {
-  MYX_GRT *grt= (MYX_GRT *) data;
+//  MYX_GRT *grt= (MYX_GRT *) data;
   MYX_GRT_VALUE *result= NULL;
   char *sql= g_strdup(
     "-- ----------------------------------------------------------------------" _br
@@ -214,11 +230,11 @@ MYX_GRT_VALUE *get_sql_drop(MYX_GRT_VALUE *param, void *data)
 
 MYX_GRT_VALUE *get_sql_merge(MYX_GRT_VALUE *param, void *data)
 {
-  MYX_GRT *grt= (MYX_GRT *) data;
-  MYX_GRT_VALUE *result= NULL;
+//  MYX_GRT *grt= (MYX_GRT *) data;
+//  MYX_GRT_VALUE *result= NULL;
   MYX_GRT_VALUE *existing= NULL;
   MYX_GRT_VALUE *altered= NULL;
-  char *sql= NULL;
+//  char *sql= NULL;
 
   if (!((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) != 2)))
     return myx_grt_function_create_error_result("The submitted parameter has to be a list with two elements.", 
@@ -350,6 +366,7 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
     const char *is_nullable= (myx_grt_dict_item_get_as_int(column, "isNullable") == 1) ? "NULL" : "NOT NULL";
     const char *datatype= myx_grt_dict_item_get_as_string(column, "datatypeName");
     const char *datatype_explicit_params= myx_grt_dict_item_get_as_string(column, "datatypeExplicitParams");
+    const char *datatype_unsigned= (myx_grt_dict_item_get_as_int(column, "unsigned") == 1) ? " UNSIGNED" : "";
     char *datatype_definition;
     char *column_comment;
     char *column_length= myx_grt_dict_item_get_formated_as_string(column, "length");
@@ -399,11 +416,12 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
         datatype_definition= g_strdup(datatype);
     }
 
-    sql_cols= str_g_append_and_free(sql_cols, g_strdup_printf("  %s%s%s %s %s", 
+    sql_cols= str_g_append_and_free(sql_cols, g_strdup_printf("  %s%s%s %s%s %s", 
       quote_char,
       myx_grt_dict_item_get_as_string(column, "name"), 
       quote_char,
       datatype_definition,
+      datatype_unsigned,
       is_nullable));
 
     // add column auto increment
@@ -790,4 +808,4 @@ static char * get_sql_drop_procedure(MYX_GRT *grt, MYX_GRT_VALUE *procedure)
   g_free(quoted_schema_name);
     
   return sql;
-}
\ No newline at end of file
+}
