@@ -29,18 +29,32 @@ static MPreferences *instance= nil;
 {
   NSString *home= NSHomeDirectory();
   NSString *result;
+  FSRef ref;
   
+  if (FSFindFolder(kUserDomain, 
+                   kApplicationSupportFolderType,
+                   YES,
+                   &ref) == noErr)
+  {
+    CFURLRef urlRef;
+    
+    urlRef= CFURLCreateFromFSRef(kCFAllocatorSystemDefault, &ref);
+    if (urlRef)
+      home= (NSString*)CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
+  }
+
   if (path)
-    result= [NSString stringWithFormat:@"%@/.mysqlgui/%@",home,path];
+    result= [NSString stringWithFormat:@"%@/MySQL/%@",home,path];
   else
-    result= [NSString stringWithFormat:@"%@/.mysqlgui",home];
+    result= [NSString stringWithFormat:@"%@/MySQL",home];
+  
   if ([[NSFileManager defaultManager] fileExistsAtPath:result])
     return result;
   
   if (path)
   {
     NSArray *parts= [path pathComponents];
-    NSMutableString *comp= [[[NSMutableString alloc] initWithFormat:@"%@/.mysqlgui/",home] autorelease];
+    NSMutableString *comp= [[[NSMutableString alloc] initWithFormat:@"%@/MySQL/",home] autorelease];
     int i;
     
     if (![[NSFileManager defaultManager] fileExistsAtPath:comp])
