@@ -10,6 +10,8 @@
 #include "myx_public_interface.h"
 
 
+extern NSString *MSchemaItemPboardType;
+
 typedef enum {
   MCatalogItemType,
   MSchemaItemType,
@@ -72,8 +74,11 @@ typedef enum {
   NSImage *_schemaIcon;
   NSImage *_tableIcon;
   NSImage *_columnIcon;
-  NSImage *_keyIcon;  
+  NSImage *_keyIcon;
   
+  MSchemaItem *_draggedItem;
+
+  MYX_CATALOGS *_catalogs;
 @public
   MSchemaDSItemType _rootType;
   MSchemaDSItemType _leafType;
@@ -118,7 +123,9 @@ objectValueForTableColumn:(NSTableColumn *)tableColumn
 - (void)updateWithCatalogs:(MYX_CATALOGS*)catalog;
 - (void)updateSchema:(MSchemaItem*)schema
 		  withTables:(MYX_SCHEMA_TABLES*)tables;
+- (MYX_CATALOGS*)catalogs;
 
+- (MSchemaItem*)draggedItem;
 @end
 
 
