@@ -6,8 +6,11 @@
 //  Copyright (c) 2004 MySQL AB. All rights reserved.
 //
 
+#import <Cocoa/Cocoa.h>
 #import "MSchemaDataSource.h"
 
+NSString *MSchemaItemPboardType= @"MSchemaItemPboardType";
+
 
 @implementation MSchemaItem
 
@@ -445,10 +448,17 @@
 }
 
 
+- (MYX_CATALOGS*)catalogs
+{
+  return _catalogs;
+}
+
 - (void)updateWithCatalogs:(MYX_CATALOGS*)catalogs
 {
   int i;
   NSMutableArray *catArray;
+
+  _catalogs= catalogs;
   
   catArray= [_children objectForKey:@"root"];
   if (catArray)
@@ -539,13 +549,13 @@
   else
   {
 	catArray= [NSMutableArray arrayWithCapacity:catalogs->catalogs_num];
-	[_children setObject:catArray forKey:@"root"];
 	
 	for (i= 0; i < catalogs->catalogs_num; i++)
 	{
 	  [self addCatalog:catalogs->catalogs+i 
 			   toArray:catArray];
 	}
+    [_children setObject:catArray forKey:@"root"];
   }
 }
 
@@ -634,6 +644,31 @@ objectValueForTableColumn:(NSTableColumn *)tableColumn
     return item; // NSString for Loading...
 }
 
+
+- (MSchemaItem*)draggedItem
+{
+  return _draggedItem;
+}
+
+- (BOOL)outlineView:(NSOutlineView *)outlineView
+         writeItems:(NSArray *)items
+       toPasteboard:(NSPasteboard *)pboard
+{
+  MSchemaItem *item= [items lastObject];
+  
+  [pboard declareTypes:[NSArray arrayWithObject:MSchemaItemPboardType] owner:nil];
+  
+  _draggedItem= item;
+  
+  //XXX add an option to set what kinds of objects can be dragged
+  if (item->type == MTableItemType)
+  {
+    [pboard setData:[NSData data] forType:MSchemaItemPboardType];
+    return YES;
+  }
+  return NO;
+}
+
 @end
 
 
