@@ -16,17 +16,18 @@ typedef enum {
 } MTabViewItemState;
 
 
-@interface MTabViewItem : NSObject {
+@interface MTabViewItem : NSResponder {
+  @private
+  id _view;
+  
+  @protected
   NSString *_identifier;
 
   MTabViewItemState _state;
   MTabViewItemState _closeState;
   
   NSImage *_icon;
-  NSView *_view;
   NSString *_label;
-  
-  id _representedObject;
 }
 
 - (id)initWithIdentifier:(NSString*)identifier;
@@ -43,13 +44,11 @@ typedef enum {
 - (NSView*)view;
 - (void)setView:(NSView*)view;
 
-- (void)setRepresentedObject:(id)object;
-- (id)representedObject;
-
 @end
 
 
-@interface MTabView : NSView {
+@interface MTabView : NSView 
+{
   IBOutlet id delegate;
 
   NSMutableArray *_pages;
@@ -68,6 +67,7 @@ typedef enum {
 
   float _tabY;
   float _tabWidth;
+  float _maxTabWidth;
   int _visibleTabCount;
 
   int _currentPage;
@@ -76,9 +76,16 @@ typedef enum {
 - (void)setTabsOnBottom:(BOOL)flag;
 
 - (int)addTabViewItem:(MTabViewItem*)item;
+- (void)removeTabViewItem:(MTabViewItem*)item;
 - (void)selectPage:(int)page;
 - (MTabViewItem*)selectedPage;
+- (int)numberOfPages;
+
+- (MTabViewItem*)pageAtIndex:(int)page;
 
 - (void)setDelegate:(id)delegate;
 
+- (void)setMaxTabSize:(float)size;
+
+- (void)recalcTabSizes;
 @end
